var gulp = require('gulp');
var fs = require('fs');
var path = require('path');
var es = require('event-stream');
var gutil = require('gulp-util');
var bower = require('bower');
var concat = require('gulp-concat');
var compass = require('gulp-compass');
var notify = require("gulp-notify");
// var handleErrors = require('./handleErrors');
var templateCache = require('gulp-angular-templatecache');
var minifyCss = require('gulp-minify-css');
var rename = require('gulp-rename');
var sh = require('shelljs');
var extend = require('gulp-extend');
var wrap = require('gulp-wrap');
var gettext = require('gulp-angular-gettext');

function errorAlert() {
  var args = Array.prototype.slice.call(arguments);
  // Send error to notification center with gulp-notify
  notify.onError({
    title: "Compile Error",
    message: "<%= error.message %>"
  }).apply(this, args);
  // Keep gulp from hanging on this task
  this.emit('end');
};

var paths = {
  sass: ['./src/scss/**/*.scss'],
  js: ['./src/**/*.js'],
  tpls: ['./src/templates/**/*.html'],
  po: ['./po/**/*.po']
};

gulp.task('serve:before', ['default']);

gulp.task('default', ['js', 'compass', 'templates', 'pot', 'translations', 'watch']);

gulp.task('watch', function () {
  gulp.watch(paths.sass, ['compass']);
  gulp.watch(paths.js, ['js']);
  gulp.watch(paths.tpls, ['templates']);
  gulp.watch(paths.js, ['pot']);
  gulp.watch(paths.tpls, ['pot']);
  gulp.watch(paths.po, ['translations']);
});

gulp.task('install', ['git-check'], function() {
  return bower.commands.install()
    .on('log', function(data) {
      gutil.log('bower', gutil.colors.cyan(data.id), data.message);
    });
});

gulp.task('git-check', function(done) {
  if (!sh.which('git')) {
    console.log(
      '  ' + gutil.colors.red('Git is not installed.'),
      '\n  Git, the version control system, is required to download Ionic.',
      '\n  Download git here:', gutil.colors.cyan('http://git-scm.com/downloads') + '.',
      '\n  Once git is installed, run \'' + gutil.colors.cyan('gulp install') + '\' again.'
    );
    process.exit(1);
  }
  done();
});

gulp.task('js', function (done) {

  function getFolders(dir){
    return fs.readdirSync(dir)
      .filter(function(file){
        return fs.statSync(path.join(dir, file)).isDirectory();
      });
  }

  var scriptsPath = './src/js/';
  var scriptsDest = './www/js/';
  var folders = getFolders(scriptsPath);

  gulp.src(['src/js/*.js'])
    .pipe(gulp.dest(scriptsDest))

  var tasks = folders.map(function(folder) {
    return gulp.src(path.join(scriptsPath, folder, '/*.js'))
      .pipe(concat(folder + '.js'))
      .pipe(gulp.dest(scriptsDest));
  });

  return es.concat.apply(null, tasks);

});

gulp.task('compass', function(done) {
  gulp.src(paths.sass)
    .pipe(compass({
      config_file: './config.rb',
      sass: 'src/scss',
      css: 'www/css'
    }))
    .on('error', errorAlert)
    .pipe(gulp.dest('./www/css/'))
    .pipe(minifyCss({
      keepSpecialComments: 0
    }))
    .pipe(rename({ extname: '.min.css' }))
    .pipe(gulp.dest('./www/css/'))
    .on('end', done);
});

gulp.task('templates', function(done) {
  gulp.src(paths.tpls)
    .pipe(templateCache({
      module: "ib.templates",
      root: "templates/"
    }))
    .pipe(gulp.dest('./www/js/'))
    .on('end', done);
});

gulp.task('pot', function () {
  return gulp.src(paths.tpls.concat(paths.js))
    .pipe(gettext.extract('translations.pot', {
        // options
    }))
    .pipe(gulp.dest('po/'));
});

gulp.task('translations', function() {
  // https://github.com/gabegorelick/gulp-angular-gettext/issues/13#issuecomment-69728371
  return gulp.src(paths.po)
    .pipe(gettext.compile({format: 'json'}))
    .pipe(extend('.tmp.json'))
    .pipe(wrap(
      '\'use strict\';\n' +
      'angular.module(\'gettext\').run(function (gettextCatalog) {\n' +
      '<% var langs = Object.keys(contents); var i = langs.length; while (i--) {' +
      '  var lang = langs[i]; var translations = contents[lang]; %>'+
      '  gettextCatalog.setStrings(\'<%= lang %>\', <%= JSON.stringify(translations) %>);\n'+
      '<% }; %>' +
      '});'))
    .pipe(rename('translations.js'))
    .pipe(gulp.dest('www/js'));
});

// https://angular-gettext.rocketeer.be/dev-guide/lazy-loading/
// gulp.task('translations', function () {
//   return gulp.src(paths.po)
//     .pipe(gettext.compile({
//       // options
//       format: 'json'
//     }))
//     .pipe(gulp.dest('www/locale'));
// });
