'use strict';
angular.module('ib', [
    'ionic',
    'ionic-modal-select',
    'ib.templates',
    'ib.services',
    'ib.controllers',
    'ib.directives',
    'LocalStorageModule',
    'gettext',
    'ngCordova'
  ])

  .run(function($location, $ionicPlatform, $rootScope, $window, gettextCatalog) {

    // Dev/Live
    $rootScope.ibVersion = '2.2.02';
    $rootScope.ibPackage = 'live';

    // Drupal end-point
    $rootScope.endPointRoot = 'http://' + $rootScope.ibPackage + '-breedit-anywhere.pantheonsite.io/';

    if ($rootScope.ibPackage == 'live') {
      $rootScope.endPointRoot = "http://mobile.ibreedit.com/";
    }

    $rootScope.graylogIp = 'http://ec2-52-16-33-19.eu-west-1.compute.amazonaws.com:12203/';
    $rootScope.graylogEndPoint = $rootScope.graylogIp + 'gelf';
    $rootScope.needUpdate = false;
    $rootScope.endPoint = $rootScope.endPointRoot + 'ib/rest/app/client';
    $rootScope.tokenEndPoint = $rootScope.endPointRoot + 'services/session/token';
    //@TODO: get client id from the server.

    $ionicPlatform.ready(function() {
      // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
      // for form inputs)
      $rootScope.$broadcast('app.run');
      if($window.cordova && $window.cordova.plugins.Keyboard) {
        $rootScope.mode = "mobile";
        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
      }
      else {
        $rootScope.mode = "pc";
      }
      if($window.StatusBar) {
        // org.apache.cordova.statusbar required
        StatusBar.styleDefault();
      }
    });

    // Set language
    // gettextCatalog.setCurrentLanguage('es');
    // gettextCatalog.debug = true;
  })

  .config(function ($ionicConfigProvider) {
    // $ionicConfigProvider.views.maxCache(0);
    $ionicConfigProvider.views.transition('android');
  })

  .config(function (localStorageServiceProvider) {
    localStorageServiceProvider.setPrefix('ib');
  })

  .config(['$httpProvider', function($httpProvider) {
    $httpProvider.interceptors.push('logToServerInterceptor');
  }])

  .constant('$ionicLoadingConfig', {
    template: '<ion-spinner icon="android" class="spinner-stable"></ion-spinner>',
    noBackdrop: true
  })

  .config(function($stateProvider, $urlRouterProvider) {
    $stateProvider
      .state('app', {
        url: "/app",
        abstract: true,
        cache: false,
        templateUrl: "templates/menu.html",
        controller: 'AppCtrl',
        resolve:{
          init: ['InitService', function(InitService){
            return InitService.run();
          }],
          settings: ['init', 'DbService', function(init, DbService){
            return DbService.getSettings().then(function(settings){
              return settings;
            });
          }]
        }
      })
      .state('app.settings', {
        abstract: true,
        url: "/settings",
        cache: false,
        views: {
          'menuContent' :{
            templateUrl: "templates/settingsAbs.html"
          }
        }
      })
      .state('app.s', {
        abstract: true,
        url: "/s/:a/:b",
        cache: false,
        resolve:{
          a: ['init', function(init){
          }]
        },

        views: {
          'menuContent' :{
            templateUrl: "templates/settingsAbs.html",
            controller: function ($location, $stateParams) {
            }

          }
        }
      })
      .state('app.s.s', {
        url: "/edit",
        cache: false,
        reloadOnSearch: false,
        views: {
          'settingsContent' :{
            templateUrl: "templates/settingsAbs.html",
            controller: function ($location, $stateParams) {
            }
          }
        }
      })
      .state('app.settings.menu', {
        url: "/settings-menu",
        cache: false,
        views: {
          'settingsContent' :{
            templateUrl: "templates/settings.html",
            controller: 'SettingsCtrl'
          }
        }
      })
      .state('app.settings.keyboard', {
        url: "/keyboard",
        cache: false,
        views: {
          'settingsContent' :{
            templateUrl: "templates/keyboard-settings.html",
            controller: 'KeyboardSettingsCtrl'
          }
        }
      })
      .state('app.settings.network', {
        url: "/network",
        cache: false,
        views: {
          'settingsContent' :{
            templateUrl: "templates/network-settings.html",
            controller: 'NetworkSettingsCtrl'
          }
        }
      })

      .state('app.jobs', {
        url: "/jobs",
        cache: false,
        resolve:{
          jobs: ['init', 'JobsFactory', function(init, JobsFactory){
            return JobsFactory.getJobs().then(function(results) {
              return results;
            });
          }]
        },
        views: {
          'menuContent' :{
            templateUrl: "templates/jobs.html",
            controller: 'JobsCtrl'
          }
        }
      })
        .state('app.job', {
        abstract: true,
        url: "/job/:jobId",
        cache: false,
        resolve:{
          jobInit: ['init', 'JobsFactory', '$stateParams', function(init, JobsFactory, $stateParams){
            return JobsFactory.setActiveJob($stateParams.jobId).then(function(data) {
              return true;
            });
          }]
        },
        views: {
          'menuContent' :{
            templateUrl: "templates/jobabs.html",
            controller: function ($scope, jobInit) {
              $scope.jobInit = jobInit;
            }
          }
        }
      })
      .state('app.job.child', {
        url: "/child",
        // cache: true,
        cache: false,
        resolve:{
          fieldsStatus: ['jobInit', 'DbService', function(jobInit, DbService){
            return DbService.getFieldsStatus(0).then(function(fieldsStatus) {
              return fieldsStatus;
            });
          }],
          fod: ['FieldObjFactory', 'jobInit', 'JobsFactory', function(FieldObjFactory,  jobInit, JobsFactory){
            delete FieldObjFactory.fieldObj;
            return FieldObjFactory.getFod(JobsFactory.job.data, 0);
          }],
          objs: ['FieldObjFactory', '$stateParams', 'fieldsStatus', function(FieldObjFactory, $stateParams, fieldsStatus){
            return FieldObjFactory.setJobFieldChildes($stateParams.jobId, 0).then(function() {
              return true;
            });
          }],
          objLength: ['objs', 'FieldObjFactory', function(objs, FieldObjFactory){
            return FieldObjFactory.objs.length;
          }],
          propList: ['FieldObjFactory', 'fod', function(FieldObjFactory, fod){
            return FieldObjFactory.getPropList(fod);
          }],
          tableViewFieldsMap: ['FieldObjFactory', 'fod', function(FieldObjFactory, fod){
            return FieldObjFactory.getPropListForTableView(fod);
          }],

          childType: ['FieldObjFactory', 'fod', 'jobInit', 'JobsFactory', function(FieldObjFactory, fod, jobInit, JobsFactory){
            return FieldObjFactory.getChildType(fod);
          }],


          childChildType: ['FieldObjFactory', 'fod', 'jobInit', 'JobsFactory', '$filter', function(FieldObjFactory, fod, jobInit, JobsFactory, $filter){
            if (fod.ScObjChildDefId) {
              var cfod = $filter('filter')(JobsFactory.job.data.ScObjDefList.ScObjDef, {ScObjDefId: fod.ScObjChildDefId
              }, true)[0];
              return FieldObjFactory.getChildType(cfod);
            }
            else {
              return false;
            }

          }]
        },
        views: {
          'jobContent' :{
            templateUrl: "templates/job.html",
            controller: 'JobCtrl'
          }
        }
      })
      .state('app.job.obj', {
        abstract: true,
        reloadOnSearch: false,
        url: "/obj",
        cache: false,
        resolve:{
          //parent code
          search: ['$stateParams', '$location', function($stateParams, $location){
            return $location.search();
          }],
          objId: ['search', function(search){
            return search.objId;
          }],
          parentId: ['search', function(search){
            return search.parentId;
          }],
          tab: ['search', function(search){
            return search.tab;
          }],
          sortedListInit:['FieldObjFactory', 'jobInit', '$stateParams', 'parentId', function(FieldObjFactory, jobInit, $stateParams, parentId){
            return FieldObjFactory.getSortedList($stateParams.jobId, parentId).then(function() {
              return true;
            });
          }],
          settings:['jobInit','settings', function(jobInit, settings){
            return settings;
          }],
          jobInit:['jobInit', function(jobInit){
            return jobInit;
          }],
          parentValues:['jobInit', '$stateParams', 'DbService', 'parentId', function(jobInit, $stateParams, DbService, parentId){
            return DbService.getFieldObjectValues(parentId).then(function(values) {
              return values;
            });
          }],
          fod: ['FieldObjFactory', 'objId', 'JobsFactory', 'jobInit', function(FieldObjFactory, objId, JobsFactory, jobInit){
            return FieldObjFactory.setFod(JobsFactory.job.data, objId);
          }],
          sortedItemId:['FieldObjFactory', 'objId', 'sortedListInit', function(FieldObjFactory, objId, sortedListInit){
            return FieldObjFactory.getSortedItemId(FieldObjFactory.sortedList, objId);
          }],
          fieldsStatus: ['init', 'DbService', 'objId', function(init, DbService, objId){
            return DbService.getFieldsStatus(objId).then(function(fieldsStatus) {
              return fieldsStatus;
            });
          }],
          fieldObj: ['sortedListInit', 'FieldObjFactory', 'objId', 'jobInit', 'parentValues', 'JobsFactory', function(sortedListInit, FieldObjFactory, objId, jobInit, parentValues, JobsFactory){
            return FieldObjFactory.setFieldObjProp(FieldObjFactory.findObject(JobsFactory.job.data.ScoredObjectsList.ScoredObject, objId), parentValues);
          }],
          formFieldInfo: ['fieldObj', 'FieldObjFactory', function(fieldObj, FieldObjFactory){
            return FieldObjFactory.getFormFieldInfo(fieldObj);
          }],
          currObj: ['FieldObjFactory', 'objId', 'sortedItemId', 'sortedListInit', function(FieldObjFactory, objId, sortedItemId, sortedListInit){
            return FieldObjFactory.getItemStatus(objId, FieldObjFactory.sortedList[sortedItemId]).then(function(res) {
              return res;
            });
          }],
          objs: ['FieldObjFactory', '$stateParams', 'jobInit', 'fieldsStatus', 'objId', function(FieldObjFactory, $stateParams, jobInit, fieldsStatus, objId){
            return FieldObjFactory.setJobFieldChildes($stateParams.jobId, objId).then(function() {
              return true;
            });
          }],
          objLength: ['objs', 'FieldObjFactory', function(objs, FieldObjFactory){
            return FieldObjFactory.objs.length;
          }],
          childFod: ['FieldObjFactory', 'objId', 'jobInit', 'JobsFactory', function(FieldObjFactory, objId, jobInit, JobsFactory){
            return FieldObjFactory.getChildFod(JobsFactory.job.data, objId);
          }],
          propList: ['FieldObjFactory', 'childFod', function(FieldObjFactory, childFod){
            return FieldObjFactory.getPropList(childFod);
          }],
          tableViewFieldsMap: ['FieldObjFactory', 'childFod', function(FieldObjFactory, childFod){
            return FieldObjFactory.getPropListForTableView(childFod);
          }],

          childType: ['FieldObjFactory', 'childFod', 'jobInit', 'JobsFactory', function(FieldObjFactory, childFod, jobInit, JobsFactory){
            return FieldObjFactory.getChildType(childFod);
          }],
          pics: ['jobInit', 'DbService', '$stateParams', 'objId', function(jobInit, DbService, $stateParams, objId){
            return DbService.getJobImages($stateParams.jobId, objId).then(function(images){
              return images;
            });
          }],
          relatedPlots: ['jobInit', 'DbService', '$stateParams', 'fieldObj', 'FieldObjFactory', function(jobInit, DbService, $stateParams, fieldObj, FieldObjFactory){
            if ('RelatedPlotsList' in fieldObj) {
              FieldObjFactory.relatedPlots = fieldObj.RelatedPlotsList.RelatedPlot;
              if (!(Array.isArray(FieldObjFactory.relatedPlots))) {
                FieldObjFactory.relatedPlots = [FieldObjFactory.relatedPlots];
              }

              return FieldObjFactory.relatedPlots.length;
            }
            else {
              FieldObjFactory.relatedPlots = false;
              return false;
            }
          }],
        },
        views: {
          'jobContent@app.job' :{
            templateUrl: "templates/objabs.html",
            controller: 'ObjCtrl'
          }
        }
      })

      .state('app.job.obj.quick-edit', {
        url: "/quick-edit",
        reloadOnSearch: false,
        resolve:{
          formObject: function($stateParams, FieldObjFactory, fieldObj, jobInit, settings, fod){
            return FieldObjFactory.getProcessedForm($stateParams.jobId, 1).then(function(formObject){
              return formObject;
            });
          },
          // COMMENTED TO REMOVE DECISIONS FIELDS FROM QUICK EDIT
          // decisionsFormObject: function(formObject, $stateParams, FieldObjFactory, fieldObj, jobInit, settings, fod){
          //   return FieldObjFactory.getProcessedForm($stateParams.jobId, 2).then(function(formObject){
          //     return formObject;
          //   });
          // },
          // quickFormObject: function(formObject, decisionsFormObject){
          //   return formObject.concat(decisionsFormObject);
          // }
          quickFormObject: function(formObject){
            return formObject;
          }
        },
        views: {
          'objContent' :{
            templateUrl: 'templates/quick-edit.html',
            controller: 'ObjQuickEditCtrl'
          }
        }
      })

      .state('app.job.obj.tabs', {
        abstract: true,
        url: "/tabs",
        views: {
          'objContent' :{
            templateUrl: "templates/obj.html"
          }
        }
      })

      .state('app.job.obj.tabs.tab-edit', {
        url: "/edit",
        cache: false,
        reloadOnSearch: false,
        //@TODO Yuval add has decisions and has input to the fieldObjCache

        resolve:{
          formObject: function($stateParams, FieldObjFactory, fieldObj, jobInit, settings, fod){
            return FieldObjFactory.getProcessedForm($stateParams.jobId, 1).then(function(formObject){
              return formObject;
            });
          }
        },

        views: {
          'tab-edit' :{
            templateUrl: "templates/tab-edit.html",
            controller: 'ObjEditCtrl'
          }
        }
      })
      .state('app.job.obj.tabs.tab-decision', {
        url: "/decision",
        cache: false,
        reloadOnSearch: false,
        resolve:{
          decisionsFormObject: function($stateParams, FieldObjFactory, fieldObj, jobInit, settings, fod){
            return FieldObjFactory.getProcessedForm($stateParams.jobId, 2).then(function(formObject){
              return formObject;
            });
          }
        },
        views: {
          'tab-decision' :{
            templateUrl: "templates/tab-decision.html",
            controller: 'DescObjEditCtrl'
          }
        }
      })
      .state('app.job.obj.tabs.tab-info', {
        url: "/info",
        cache: false,
        reloadOnSearch: false,
        resolve:{
          objInfo: ['FieldObjFactory', 'fieldObj', 'jobInit', 'JobsFactory', function(FieldObjFactory, fieldObj, jobInit, JobsFactory){
            return FieldObjFactory.getObjectInfo(JobsFactory.job.data);
          }]
        },

        views: {
          'tab-info' :{
            templateUrl: "templates/tab-info.html",
            controller: 'ObjInfoCtrl'
          }
        }
      })
      .state('app.job.obj.tabs.tab-pics', {
        url: "/pics",
        cache: false,
        reloadOnSearch: false,
        views: {
          'tab-pics' :{
            templateUrl: "templates/tab-pics.html",
            controller: 'PicsCtrl'
          }
        }
      })
      .state('app.job.obj.tabs.tab-items', {
        url: "/items",
        cache: false,
        reloadOnSearch: false,
        views: {
          'tab-items' :{
            templateUrl: "templates/tab-items.html",
            controller: 'ObjItemsCtrl'
          }
        }
      });
    // if none of the above states are matched, use this as the fallback
    $urlRouterProvider.otherwise('/app/jobs');
  })

  .filter('paginate', function(Paginator) {
    return function(input, rowsPerPage) {

      if (!input) {
        return input;
      }

      if (rowsPerPage) {
        Paginator.rowsPerPage = rowsPerPage;
      }

      Paginator.itemCount = input.length;

      if (input.length < rowsPerPage) {
        return input;
      }

      return input.slice(parseInt(Paginator.page * Paginator.rowsPerPage), parseInt((Paginator.page + 1) * Paginator.rowsPerPage + 1) - 1);
    };
  })

  .filter('forLoop', function() {
    return function(input, start, end) {
      input = new Array(end - start);
      for (var i = 0; start < end; start++, i++) {
        input[i] = start;
      }

      return input;
    }
  })

  .service('Paginator', function ($rootScope) {
    this.page = 0;
    this.rowsPerPage = 20;
    this.itemCount = 0;
    this.limitPerPage = 20;

    this.setPage = function (page) {
      if (page > this.pageCount()) {
        return;
      }
      this.page = page;
    };

    this.nextPage = function () {
      if (this.isLastPage()) {
        return;
      }

      this.page++;
    };

    this.perviousPage = function () {
      if (this.isFirstPage()) {
        return;
      }

      this.page--;
    };

    this.firstPage = function () {
      this.page = 0;
    };

    this.lastPage = function () {
      this.page = this.pageCount() - 1;
    };

    this.isFirstPage = function () {
      return this.page == 0;
    };

    this.isLastPage = function () {
      return this.page == this.pageCount() - 1;
    };

    this.pageCount = function () {
      return Math.ceil(parseInt(this.itemCount) / parseInt(this.rowsPerPage));
    };

    this.lowerLimit = function() {
      var pageCountLimitPerPageDiff = this.pageCount() - this.limitPerPage;

      if (pageCountLimitPerPageDiff < 0) {
        return 0;
      }

      if (this.page > pageCountLimitPerPageDiff + 1) {
        return pageCountLimitPerPageDiff;
      }

      var low = this.page - (Math.ceil(this.limitPerPage/2) - 1);

      return Math.max(low, 0);
    };
  })

.filter('direction', function () {
    return function (text) {
      if (text) {
        // Find Hebrew letters (א-ת).
        return text.match(/[\u0590-\u05FF]/i) ? 'rtl' : 'ltr';
      }
    };
  })
  .filter('orderObjectBy', function() {
    return function(items, field, reverse) {
      var filtered = [];
      angular.forEach(items, function(item) {
        filtered.push(item);
      });
      filtered.sort(function (a, b) {
        return (a[field] > b[field] ? 1 : -1);
      });
      if(reverse) filtered.reverse();
      return filtered;
    };
  })
  .filter('setTimeFormat', function(HelperService) {
    return function(date, selector, formatLength) {
      if (date === 0) {
        return '';
      }
      return HelperService.formatDate(date, selector, formatLength);
    }
  });

// Will be populated later by gulp in templates.js
angular.module("ib.templates", [])

Array.prototype.diff = function(a) {
  return this.filter(function(i) {return a.indexOf(i) < 0;});
};

// https://tc39.github.io/ecma262/#sec-array.prototype.find
if (!Array.prototype.find) {
  Object.defineProperty(Array.prototype, 'find', {
    value: function(predicate) {
      // 1. Let O be ? ToObject(this value).
      if (this == null) {
        throw new TypeError('"this" is null or not defined');
      }

      var o = Object(this);

      // 2. Let len be ? ToLength(? Get(O, "length")).
      var len = o.length >>> 0;

      // 3. If IsCallable(predicate) is false, throw a TypeError exception.
      if (typeof predicate !== 'function') {
        throw new TypeError('predicate must be a function');
      }

      // 4. If thisArg was supplied, let T be thisArg; else let T be undefined.
      var thisArg = arguments[1];

      // 5. Let k be 0.
      var k = 0;

      // 6. Repeat, while k < len
      while (k < len) {
        // a. Let Pk be ! ToString(k).
        // b. Let kValue be ? Get(O, Pk).
        // c. Let testResult be ToBoolean(? Call(predicate, T, « kValue, k, O »)).
        // d. If testResult is true, return kValue.
        var kValue = o[k];
        if (predicate.call(thisArg, kValue, k, o)) {
          return kValue;
        }
        // e. Increase k by 1.
        k++;
      }

      // 7. Return undefined.
      return undefined;
    }
  });
}