angular.module('ib.controllers', [])
angular.module('ib.controllers')
  .controller('AppCtrl', function($scope, $ionicPlatform, $ionicModal, $ionicPopup, $filter, $interval, HelperService, DbService, SyncService, AuthenticationService, localStorageService, settings, $rootScope, $log, $templateCache, $http, $compile, FieldObjFactory, JobsFactory, gettextCatalog, $timeout, $cordovaFile) {
    $rootScope.$on('$stateChangeStart',
      function(event, toState, toParams, fromState, fromParams){
        $log.info("state changed enter time " + Date.now(), {'stateChangedStartTime' : Date.now(), fromState : fromState, toState: toState});
      });
    $scope.$on('$ionicView.afterEnter', function(){
      $log.info("ionicView.afterEnter " + Date.now(), {'startTime' : Date.now()});
    });
    $scope.$on('$ionicView.beforeLeave', function(){
      $log.info("ionicView.beforeLeave " + Date.now(), {'startTime' : Date.now()});
    });
    $scope.settings = settings;
    $ionicModal.fromTemplateUrl('templates/login.html', function(modal) {
        $scope.loginModal = modal;
      },
      {
        scope: $scope,
        animation: 'slide-in-up',
        focusFirstInput: true,
        backdropClickToClose: false,
        hardwareBackButtonClose: false
      }
    );
    //Be sure to cleanup the modal by removing it from the DOM
    $scope.$on('$destroy', function() {
      $scope.loginModal.remove();
    });

    $scope.$on('event:auth-logout-complete', function() {
      $scope.loginModal.show();
    });

    $scope.displayname = localStorageService.get('username');
    $scope.$on('event:auth-loginConfirmed', function() {
      $scope.displayname = localStorageService.get('username');
    });

    $scope.logout = function() {
      if ($scope.connectionType < 0) {
        $ionicPopup.alert({
          title: gettextCatalog.getString('You are offline.'),
          template: gettextCatalog.getString("You must be connected in order to log out."),
          okText: gettextCatalog.getString('OK'),
          okType: 'button-balanced'
        });
      }
      else {
        AuthenticationService.logout();
      }
    };

    $scope.login = function() {
      if (localStorageService.get('token')) {
        AuthenticationService.logout();
      }
      else {
        $scope.loginModal.show();
      }
    };

    $scope.clearJobs = function() {
      var confirmPopup = $ionicPopup.confirm({
        title: gettextCatalog.getString('Are you sure?'),
        template: gettextCatalog.getString("You are about to delete all data from the device.</br>Are you sure you want to proceed?"),
        okText: gettextCatalog.getString('CLEAR DATA'),
        okType: 'button-assertive'
      });
      confirmPopup.then(function (ok) {
        if (ok) {
          DbService.clearJobs();
          // LogDbService.createTables(true);

          $scope.$broadcast('data.clear');
        }
        else {
          // Do nothing
          return false;
        }
      });

    };

    // Sync
    $scope.isSyncing = false;
    $scope.isSyncingLog = false;
    var lastSyncDate = DbService.getLastSync();
    $scope.lastSync = lastSyncDate ? $filter('setTimeFormat')(lastSyncDate, 'date and time', 'short') : 'Never'; // 15:13 - 1/1/15

    var offLineSync = function() {


      try {
        DbService.getNeedSyncJobsIdList().then(function(results) {
          if (results.rows.length > 0) {
            results.rows.forEach(function (id) {
              DbService.getJobValues(id).then(function (jobValues) {
                DbService.getJobOfflineImages(id).then(function (picValues) {
                  $cordovaFile.writeFile(cordova.file.externalDataDirectory, id + ".json", JSON.stringify(jobValues.rows.concat(picValues.rows)), true).then(function(result) {
                  // Success!
                  }, function(err) {
                    $log.error(err);
                    // An error occured. Show a message to the user
                  });
                });
              });
            });
          }
        });
      }
      catch (e) {
        $log.error(e);
      }
    }

    var syncNow = function () {
      return;
      //@TODO delete the code since we sync in the BG...

      if ($scope.isSyncing == false) {
        $scope.isSyncing = true;
        SyncService.sync().then(function() {
          $scope.isSyncing = false;
          $scope.lastSync = $filter('setTimeFormat')(DbService.setLastSync(), 'date and time', 'short'); // 15:13 - 1/1/15
        });
      }
    };

    function updateOnlineStatus() {
      $scope.connectionType = HelperService.checkConnectionMode();
      if ($scope.connectionType > -1) {
        $scope.enableSync = true;
      }
      else {
        $scope.enableSync = false;
      }
    }
    // Watch for changes
    updateOnlineStatus();
    window.addEventListener('online',  updateOnlineStatus);
    window.addEventListener('offline', updateOnlineStatus);
    $scope.syncLog = function() {
      if ($scope.isSyncingLog == false && HelperService.checkConnectionMode() != -1) {
        // $scope.isSyncingLog = true;
        // logService.sync().then(function() {
        //   $scope.isSyncingLog = false;
        // });
      }
    }
      $scope.syncData = function(mode) {
      $scope.connectionType = HelperService.checkConnectionMode();
      if ($scope.isSyncing) {
        return;
      }
      mode = mode || false;
      SyncService.checkSyncLogic($scope.connectionType, settings).then(function(res) {
        if (res == -1) {
          offLineSync();
          return false;
        }
        else if(res == 1) {
          syncNow();
        }
        else if(res == 0) {
          if (mode == 'manual') {
            var confirmPopup = $ionicPopup.confirm({
              title: gettextCatalog.getString('No WiFi Connection'),
              template: gettextCatalog.getString("You are about to upload images.</br>Are you sure you want to sync?"),
              okText: gettextCatalog.getString('SYNC'),
              okType: 'button-balanced'
            });
            confirmPopup.then(function (ok) {
              if (ok) {
                // May sync
                syncNow();
              }
              else {
                // Do nothing
                return false;
              }
            });
          }
          return false;
        }
      })
    };

    // Auto-Sync
    if (settings.sync && settings.sync.autoSyncInterval && settings.sync.autoSyncInterval.value > 0) {
      var interval = (settings.sync.autoSyncInterval.value * 1000 * 60);
      $interval($scope.syncData, interval);
    };

    //@TODO wrap with settings check
    $interval($scope.syncLog, 1000 * 60);
  })

angular.module('ib.controllers').controller('ChildListCtrl', function($scope, $state, $stateParams, $filter, $window, $ionicPopup, $ionicLoading, $cordovaBarcodeScanner, FieldObjFactory, $location, $ionicTabsDelegate, $ionicHistory, $ionicScrollDelegate, $ionicModal, GotoService, gettextCatalog, DbService, JobsFactory, Paginator, $timeout, $ionicPosition) {

  $scope.rowsPerPage = 20;
  // 1. Filter
  $scope.$on('childList.refilter', function() {
    // Reset
    FieldObjFactory.setFieldsStatus(FieldObjFactory.objs, $scope.fieldsStatus);
    FieldObjFactory.set('filteredObjects', FieldObjFactory.objs);
    // Filter to show only pending
    var predicate;
    if ($scope.showOnlyPending) {
      predicate = {statusClass: 'pending'};
      FieldObjFactory.set('filteredObjects', $filter('filter')(
        FieldObjFactory.filteredObjects, predicate, true
      ));
    }
    // Filter by title only / by any field-status
    if ($scope.searchByTitleOnly) {
      predicate = {title: $scope.search.title};
      FieldObjFactory.set('filteredObjects', $filter('filter')(
        FieldObjFactory.filteredObjects, predicate
      ));
    }
    else {
      FieldObjFactory.set('filteredObjects', $filter('filter')(
        FieldObjFactory.filteredObjects, $scope.search
      ));
    }
    $scope.$broadcast('childList.reorder');
  });

  // 2. Order by...
  $scope.$on('childList.reorder', function() {
    if ($scope.predicate) {
      FieldObjFactory.set('orderedObjects', $filter('orderBy')(FieldObjFactory.filteredObjects, $scope.predicate, $scope.reverse));
    }
    else {
      FieldObjFactory.set('orderedObjects', FieldObjFactory.filteredObjects);
    }
    $scope.limitedObjects = FieldObjFactory.filteredObjects;
    $scope.$broadcast('childList.finishedOrdering');
  });

  // 3. Statuses
  $scope.$on('childList.finishedOrdering', function() {
    $scope.limitedObjects = FieldObjFactory.setFieldsStatus(FieldObjFactory.orderedObjects, $scope.fieldsStatus);

    if ($scope.tableView.show) {
      $scope.tableViewHeaderUpdate();
    }
  });

  // Update cached view
  $scope.$on('$ionicView.beforeEnter', function(scopes, states) {
    if (states.fromCache) {
      DbService.getFieldsStatus($scope.objId).then(function(fs) {
        $scope.fieldsStatus = fs;
        $scope.$broadcast('childList.refilter');
      });
    }
  });

  $scope.tableViewHeaderUpdate = function() {
    var pds = [];

    $scope.tableViewFieldsMap.forEach(function (item) {
      if (item.showInTableView) {
        pds.push(item.id);
      }
    });
    FieldObjFactory.getObjectsInfoForTableView(JobsFactory.job.data, $scope.limitedObjects, pds);

    // Save to localStorage.
    var level = $scope.objMeta ? $scope.objMeta.Type : 'job';
    FieldObjFactory.setTableViewConfig($scope.jobId, level, pds);

  };

  function initTableView() {

    var level = $scope.objMeta ? $scope.objMeta.Type : 'job';
    var pds = FieldObjFactory.getTableViewConfig($scope.jobId, level);
    $scope.tableView = $scope.tableView || {};
    $scope.tableView.show = FieldObjFactory.getTableViewStatus($scope.jobId, level);
    $scope.tableView.maxHeight = ($window.innerHeight - 300) + 'px';

    if (pds != []) {
      $scope.tableViewFieldsMap.forEach(function (item) {
        if (pds.indexOf(item.id) >= 0) {
          item.showInTableView = true;
        }
      });
    }

  }

  var viewId;

  function showActiveItem(newViewId) {

    // Polyfill Array.prototype.findIndex()
    if (!Array.prototype.findIndex) {
      Array.prototype.findIndex = function(predicate) {
        'use strict';
        if (this == null) {
          throw new TypeError('Array.prototype.findIndex called on null or undefined');
        }
        if (typeof predicate !== 'function') {
          throw new TypeError('predicate must be a function');
        }
        var list = Object(this);
        var length = list.length >>> 0;
        var thisArg = arguments[1];
        var value;

        for (var i = 0; i < length; i++) {
          value = list[i];
          if (predicate.call(thisArg, value, i, list)) {
            return i;
          }
        }
        return -1;
      };
    }

    var scrollToActiveItem = function() {
      var activeElement = document.querySelector('.view-id-' + viewId + ' .active-item');
      if (activeElement) {
        var activePosition = $ionicPosition.position(angular.element(activeElement));
        var scrollHandle = $scope.tableView.show ? 'tableViewScroll' : 'childListScroll';
        if ($scope.tableView.show) {
          // Table-view
          $ionicScrollDelegate.$getByHandle('tableViewScroll').scrollTo(0, activePosition.top, true);
          // Scroll wrapping view to top
          $ionicScrollDelegate.$getByHandle('childListScroll').scrollTop(true);
        }
        else {
          // List view
          $ionicScrollDelegate.$getByHandle('childListScroll').scrollTo(0, activePosition.top, true);
        }
      }
    };

    // Use last viewId when got nothing.
    viewId = newViewId ? newViewId : viewId;

    // Find active item's page
    var activeIndex = $scope.limitedObjects.findIndex(function(item) {
      return item.id == $scope.activeItem;
    });
    if (activeIndex > -1) {
      var activePage = Math.floor(activeIndex / Paginator.rowsPerPage);
      if (activePage > -1) {
        // Wait for the pager to load
        $timeout(function() {
          Paginator.setPage(activePage);
          // Wait for the right page to load
          $timeout(function() {
            scrollToActiveItem();
          });
        });
      }
      else {
        // Wait for the page to load
        $timeout(function() {
          scrollToActiveItem();
        });
      }
    }
  }

  function initActiveItem() {
    $scope.activeItem = FieldObjFactory.getActiveItem($scope.jobId, $scope.objId || 0);
    if ($scope.activeItem) {
      $scope.$on('$ionicView.enter', function(event, data) {
        if(event.targetScope !== $scope) {
          return;
        }
        showActiveItem(event.targetScope.$id);
      });
    }
  }

  function initScope() {
    //$scope.buffer = 12;
    //$scope.limit = $scope.buffer;

    $scope.sortSelect = [
      {value: 'none', name: gettextCatalog.getString('Sort By...')},
      {value: 'title', name: gettextCatalog.getString('Title')}
    ];
    $scope.propList.forEach(function(prop, index) {
      $scope.sortSelect.push({value: 'properties[' + index + '].Value', name: prop});
    });
    $scope.storedSort = FieldObjFactory.getSort($scope.jobId, $scope.objId || 0) || {};
    $scope.predicate = $scope.storedSort.predicate || '';
    $scope.reverse = $scope.storedSort.reverse || false;
    $scope.sort = $scope.predicate || 'none';
    $scope.search = {};
    $scope.search.$ = $scope.storedSort.filter || '';
    if ($scope.storedSort.fav) {
      $scope.search.fav = true;
    }
    $scope.search.title = $scope.storedSort.titleOnly || '';
    $scope.searchByTitleOnly = !!$scope.storedSort.titleOnly || false;
    $scope.showOnlyPending = $scope.storedSort.showOnlyPending || false;
    $scope.childListFilters = {show: false};
    initTableView();
    initActiveItem();
    // Enable load more.
    //$scope.moreDataCanBeLoaded = true;
    //if ($scope.limit >= $scope.objLength  ) {
    //  $scope.moreDataCanBeLoaded = false;
    //}
    // Filter
    $scope.$broadcast('childList.refilter');

  }

  initScope();

  $scope.$on('objctrl.update', function() {
    initScope();
    $scope.$emit('childobj.updated');
  });

  // Scroll to top when clicking bottom pager.
  $scope.bottomPagerScrollTop = function($event) {
    var el = $event.srcElement,
        enabled = false,
        aEl;
    // Iterate over element & parents (a-la jQuery.closest())
    // Check if it's an enabled pager item.
    while (el !== null) {
      aEl = angular.element(el);
      if (aEl.hasClass('bottom-pager')) {
        // Reached pager wrapper and didn't find active element.
        break;
      }
      else if (aEl.attr('ng-click') && !aEl.hasClass('disabled') && !aEl.hasClass('active')) {
        // Found active element. break.
        enabled = true;
        break;
      }
      el = el.parentElement;
    }
    // Scroll only if the clicked item is enabled
    if (enabled) {
      $ionicScrollDelegate.scrollTop(true);
    }
  };

  $scope.goToWithLoading = function(objId, parentUniqueId) {
    GotoService.goTo(parentUniqueId, objId, "edit");
  };

  // $scope.scanQR = function() {
  //   $cordovaBarcodeScanner
  //     .scan()
  //     .then(function(barcodeData) {
  //       var res = $filter('filter')(FieldObjFactory.objs, barcodeData.text);
  //       var keys = Object.keys(res);
  //       if (keys.length == 1) {
  //         GotoService.goTo($scope.objId || 0, res[0].id, "edit");
  //       }
  //
  //       $scope.search.$ = barcodeData.text;
  //
  //     }, function(error) {
  //       $ionicPopup.alert({
  //         title: gettextCatalog.getString('Something went wrong'),
  //         template: gettextCatalog.getString('Unable to scan the QR code.')
  //       });
  //   });
  // };

  // Search/Favorite filter
  $scope.filterList = function(prop) {
    if ($scope.objs2delete) {
      $scope.$broadcast('childList.resetDelete');
    }
    // Favorite (toggle)
    if (prop == 'fav') {
      if ($scope.search.fav) {
        delete $scope.search.fav;
      }
      else {
        $scope.search.fav = true;
      }
    }
    // Filter
    else if (prop == 'filter') {
      // Nothing special to do here, just save.
    }
    FieldObjFactory.setSort($scope.jobId, $scope.objId || 0, $scope.predicate, $scope.reverse, $scope.search, $scope.search.title, $scope.showOnlyPending);
    $scope.$broadcast('childList.refilter');
    $timeout(function() {
      showActiveItem();
    });
  };

  // Reset search
  $scope.resetSearch = function() {
    $scope.search.$ = '';
    $scope.search.title = '';
    $scope.childListFilters.show = false;
    $scope.filterList('filter');
  };

  // Sort
  $scope.sortList = function (prop) {
    if ($scope.objs2delete) {
      $scope.$broadcast('childList.resetDelete');
    }
    if (prop == 'reverse') {
      $scope.reverse = !$scope.reverse;
    }
    // Sort term (select box)
    else if (prop == 'none') {
      $scope.predicate = '';
    }
    else {
      $scope.predicate = prop;
    }
    FieldObjFactory.setSort($scope.jobId, $scope.objId || 0, $scope.predicate, $scope.reverse, $scope.search, $scope.search.title);
    $scope.$broadcast('childList.reorder');
  };

  $scope.getItemStatus = function(status) {
    var statusStr;
    switch (status) {
      case "pending":
        statusStr = gettextCatalog.getString('PENDING');
        break;
      case "inProgress":
        statusStr = gettextCatalog.getString('IN PROGRESS');
        break;
      case "completed":
        statusStr = gettextCatalog.getString('COMPLETED');
        break;
      case "done":
      case "sync":
        statusStr = gettextCatalog.getString('DONE');
        break;
      default:
        statusStr = gettextCatalog.getString('PENDING');
    }
    return statusStr;
  };

  // Table view
  $scope.tableViewToggleState = function() {
    $scope.childListFilters.show = false;
    if ($scope.tableView.show === true) {
      $scope.tableViewShowMenu();
      $scope.tableViewHeaderUpdate();
    }

    // Save to localStorage.
    var level = $scope.objMeta ? $scope.objMeta.Type : 'job';
    FieldObjFactory.setTableViewStatus($scope.jobId, level, $scope.tableView.show);

    showActiveItem();
  };

  $scope.tableViewShowMenu = function() {
    $ionicModal.fromTemplateUrl('templates/table-view-menu.html', function(modal) {
      $scope.tableViewMenu = modal;
      $scope.tableViewMenu.show();
    }, {
      scope: $scope,
      animation: 'slide-in-up',
      backdropClickToClose: false
    });
  };
  // close video modal
  $scope.tableViewCloseMenu = function() {
    $scope.tableViewMenu.hide();
  };
  $scope.$on('modal.hidden', function() {
  });
  $scope.$on('$destroy', function() {
    if ($scope.tableViewMenu) {
      $scope.tableViewMenu.remove();
    }
  });

  $scope.checkAll = false;
  $scope.tableViewMenuToggleCheckAll = function(checkAll) {
    $scope.checkAll = checkAll;

    $scope.tableViewFieldsMap.forEach(function (item) {
      item.showInTableView = checkAll;
    });
    $scope.tableViewFieldsMap.showInTableView = checkAll;

    $scope.tableViewHeaderUpdate();
  };

});

angular.module('ib.controllers')
.controller('FormController', function($stateParams, $scope, DbService, FieldObjFactory, HelperService, $log, JobsFactory, $ionicModal, $ionicPopup, $ionicPosition, $ionicScrollDelegate, $cordovaBarcodeScanner, gettextCatalog) {
  $log.info("FormController enter time " + Date.now(), {'controllerStartTime' : Date.now()});

  // Handling form ng-changes and JS changes via event 'form.updated'
  $scope.updated = function(modelValue, form) {

    if (angular.isDefined($scope.quickEditAutoAdvance)) {
      $scope.quickEditAutoAdvance(form);
    }
    if (angular.isUndefined(modelValue)) {
      return;
    }
    if ($scope.jobDone) {
      return;
    }
    form.feedback = validateValue(modelValue, form);
    if (form.feedback && !form.allowSave) {
      return;
    }
    var insertValue = modelValue;
    if (form.type && form.type == 'date') {
      insertValue = Date.parse(modelValue) / 1000;
    }
    else if (form.type && form.type == 'select') {
      form.displayValue = form.titleMap.find(function(item){return item.value == modelValue}).name
    }
    DbService.insertJobValue($scope.jobId, $scope.formFieldInfo.fid, form.pd, Date.now() / 1000, insertValue, $scope.formFieldInfo.pfid, $scope.formFieldInfo.userFid, $scope.formFieldInfo.odefId).then(function() {

      FieldObjFactory.checkField(JobsFactory.job, $scope.formFieldInfo.fid, FieldObjFactory.getFod(JobsFactory.job.data, $scope.objId)).then(function(status) {
        $scope.currObj.statusClass = HelperService.getStatusClass(status);
      });
    });
  };

  $scope.$on('form.updated', function(event, args) {
    $scope.updated(args.modelValue, args.form);
  });

  // Validate form elements
  function validateValue(val, form) {

    // Required
    if (form.required && !val && val !== 0) {
      form.allowSave = true; // Save invalid required value (Feature #550).
      return gettextCatalog.getString("This field is required");
    }

    switch(form.type){
      case "number":
      case "range":
      case "tel":

        // Type
        if (isNaN(val)) {
          return gettextCatalog.getString("Please enter a valid number");
        }

        // Precision (float/integer)
        var precisionRegex;
        if (form.precision) { // Float decimal precision
          precisionRegex = new RegExp("^\\s*-?(\\d+\\.?\\d{0," + form.precision + "})?\\s*$", "g");
          if (!precisionRegex.test(val)) {
            return gettextCatalog.getString("Please enter up to {{precision}} decimal places", { precision: form.precision });
          }
        }
        else { // Integer
          precisionRegex = new RegExp("^\\s*-?\\d*\\s*$", "g");
          if (!precisionRegex.test(val)) {
            return gettextCatalog.getString("Integers only please");
          }
        }

        // Minimum value
        if (!isNaN(form.minimum) && val < form.minimum) {
          return gettextCatalog.getString("The minimum value is {{minimum}}", { minimum: form.minimum });
        }

        // Maximum value
        if (!isNaN(form.maximum) && val > form.maximum) {
          return gettextCatalog.getString("The maximum value is {{maximum}}", { maximum: form.maximum });
        }
        break;
      case "string":
        // Max length
        if (!isNaN(form.maxLength) && val.length > form.maxLength) {
          return gettextCatalog.getString("Only {{maxLength}} characters are allowed", { maxLength: form.maxLength });
        }

        break;
    }

    return false; // Valid!
  }

  $scope.toggleKeyboard = function(event, form) {
    // Hide other keyboards.
    $scope.$emit('keyboard.hideOthers', {id: form.pd});
    // Register once, hide keyboard when other one opens.
    if (!form.showKeyboard && !form.keyboardRegisterEvent) {
      $scope.$on('keyboard.hideOthers', function(event, args) {
        if (args.id !== form.pd) {
          form.showKeyboard = false;
        }
        form.keyboardRegisterEvent = true;
      });
    }
    // Toggle
    form.showKeyboard = form.showKeyboard ? false : true;

    // Scroll to show keyboard if not visible.
    if (form.showKeyboard) {
      var formItem = angular.element(event.srcElement.parentNode),
          offset = $ionicPosition.offset(formItem);

      if (offset.top + offset.height + 50 > window.innerHeight) {
        var scrollAmount = offset.top + offset.height - window.innerHeight + 60;
        $ionicScrollDelegate.$getByHandle('formContent').scrollBy(0, scrollAmount, true);
      }
    }
  }

  $scope.hasHistoryBtn = function(form) {
    // True if there's a history or an oldVal (not empty).
    return form.historyList || form.oldval;
  }

  $scope.insertOldVal = function(form) {
    if ($scope.jobDone) {
      return false;
    }
    switch(form.type){
      case "select":
        for (var i = 0; i < form.titleMap.length; i++) {
          if (form.titleMap[i].name == form.oldval) {
            form.currVal = form.titleMap[i].value;
            break;
          }
        }

        break;
      case "boolean":
        form.currVal = form.oldval == '-' + gettextCatalog.getString('True') + '-' ? true : false;
        break;
      default:
        form.currVal = form.oldval;
        break;
    }
    $scope.$emit('form.updated', { modelValue: form.currVal, form: form });
  }

  // Image select modal
  $scope.imageSelectModal = function(form) {
    $ionicModal.fromTemplateUrl('templates/image-select-modal.html', function(modal) {
      $scope.gridModal = modal;
      if (form && form.imagesList) {
        // ToDo: Unify the structure of the form element & info element
        $scope.imagesTitle = form.title;
        $scope.imagesList = form.imagesList;
        $scope.gridModal.show();
      }
    }, {
      scope: $scope,
      animation: 'slide-in-up'
    });

    $scope.imageSelect = function(image) {
      form.currVal = image.title;
      // ToDo: validate and save.
      $scope.$emit('form.updated', { modelValue: image.value, form: form });
    };

  };
  // close video modal
  $scope.closeImagesModal = function() {
    $scope.gridModal.hide();
  };

  $scope.$on('$destroy', function() {
    if ($scope.gridModal) {
      $scope.gridModal.remove();
    }
  });

  // Image select - quick edit
  $scope.quickEditImageSelect = function(image, form) {
    form.currVal = image.title;
    // ToDo: validate and save.
    $scope.$emit('form.updated', { modelValue: image.value, form: form });
  };

  // Barcode scanner
  $scope.scanQR = function(form) {
    $cordovaBarcodeScanner
      .scan()
      .then(function(barcodeData) {
        form.currVal = barcodeData.text;
        $scope.$emit('form.updated', { modelValue: barcodeData.text, form: form });
      }, function(error) {
        $ionicPopup.alert({
          title: gettextCatalog.getString('Something went wrong'),
          template: gettextCatalog.getString('Unable to scan the code.')
        });
    });
  };

});

'use strict';
angular.module('ib.controllers')
.controller('JobCtrl', function($scope, $rootScope, $stateParams, $ionicPopup, $ionicLoading, RestService, DbService, HelperService, objs, $state, propList, childChildType, objLength, fieldsStatus, JobsFactory, FieldObjFactory, gettext, gettextCatalog, childType, tableViewFieldsMap) {
  $scope.JobName = JobsFactory.job.data.JobName;
  $scope.fieldsStatus = fieldsStatus;
  $scope.childChildType = childChildType;
  $scope.childChildTypes = childChildType + 's';
  $scope.objLength = objLength;
  $scope.propList = propList;
  $scope.tableViewFieldsMap = tableViewFieldsMap;
  $scope.jobId = $stateParams.jobId;
  $scope.childType = childType;
  $scope.childTypes = childType + 's';
  $scope.$on('$stateChangeSuccess', function() {
    $ionicLoading.hide();
  });

  $scope.completeJob = function(id) {
    if (JobsFactory.job.status >= 3) {
      return false;
    }
    $scope.popup = {
      done: false,
      subTitle: gettextCatalog.getString('You will not be able to update this job<br />THIS ACTION IS IRREVERSIBLE'),
      description: false
    };
    // if (HelperService.isOnlineCheck()) {
      $ionicPopup.show({
        title: gettextCatalog.getString('Complete Job'),
        subTitle: $scope.popup.subTitle,
        template: '<ion-checkbox class="checkbox-assertive" ng-model="popup.done">I understand.</ion-checkbox><div class="assertive" ng-show="popup.description">' + gettextCatalog.getString("Please confirm") + '</div>',
        scope: $scope,
        buttons: [{
          text: gettextCatalog.getString('COMPLETE'),
          type: 'button-balanced',
          onTap: function(e) {
            if (!$scope.popup.done) {
              e.preventDefault();
              $scope.popup.description = true;
            } else {
              DbService.updateJobStatus(id, 3);
              $rootScope.$broadcast('jobs.jobUpdate', {id: id, status: 3});

              // $scope.syncData('manual');
              $state.go('app.jobs');
            }
          }
        }, {
          text: gettextCatalog.getString('CANCEL'),
          type: 'button-default'
        }]
      });
    // }
    // else {
    //   $ionicPopup.alert({
    //     title: gettextCatalog.getString('No internet connection'),
    //     okType: 'button-balanced'
    //   });
    // }
  };

  function jobBtnState() {
    var bState;
    switch (JobsFactory.job.status) {
      case 0:
      case 1:
      case 2:
        bState = 'off';
        break;
      // case 3:
      //   bState = 'process';
      //   break;
      case 3:
      case 4:
        bState = 'done';
    }
    return bState;
  }
  $scope.btnState = jobBtnState();

  function jobBtnText() {
    var bText;
    switch (JobsFactory.job.status) {
      case 0:
      case 1:
      case 2:
        bText = gettext('DONE');
        break;
      // case 3:
      //   bText = gettext('SYNC');
      //   break;
      case 3:
      case 4:
        bText = gettext('COMPLETED');
    }
    return bText;
  }
  $scope.btnText = jobBtnText();

  $scope.$on('jobs.jobUpdate', function(){
    $scope.btnText = jobBtnText();
    $scope.btnState = jobBtnState();
  });

  // Exit related mode
  if ($state.current.name == 'app.job.child') {
    FieldObjFactory.exitRelatedMode();
  }

  // function toggleCompleteBtn() {
  //   $scope.completeBtn = $scope.completeBtn == 'DONE' ? 'COMPLETED' : 'DONE';
  //   $scope.btnState = $scope.completeBtn == 'DONE' ? 'off' : 'on';
  // }

})

angular.module('ib.controllers')
.controller('JobsCtrl', function($scope, $rootScope, $ionicPlatform, $timeout, $state, $ionicPopup, $ionicLoading, JobsFactory, HelperService, jobs, $filter, gettextCatalog, DbService) {
  $scope.jobs = jobs;
  $scope.jobsCount = Object.keys($scope.jobs).length || 0;
  $scope.jobs2delete = 0;

  $scope.$on('$stateChangeSuccess', function() {
    $ionicLoading.hide();
  });

  $scope.goToWithLoading = function(jobId) {
    $ionicLoading.show();
    $state.go('app.job.child', {
      jobId: jobId
    });
  }

  $scope.getJobStatus = function(status) {
    var statusStr;
    switch (status) {
      case "pending":
        statusStr = gettextCatalog.getString('PENDING');
        break;
      case "inProgress":
        statusStr = gettextCatalog.getString('IN PROGRESS');
        break;
      case "completed":
        statusStr = gettextCatalog.getString('COMPLETED');
        break;
      case "done":
      case "sync":
        statusStr = gettextCatalog.getString('DONE');
        break;
      default:
        statusStr = gettextCatalog.getString('PENDING');
    }
    return statusStr;
  }

  $scope.formatTime = function(timestamp) {
    var date = new Date(timestamp * 1000);
    return date.toDateString();
  };

  $scope.toggleItemDeletion = function(jobId) {
    var job = $scope.jobs.filter(function( job ) {
      return job.id == jobId;
    })[0];
    if (job) {
      // Flag it
      job.toDelete = job.toDelete ? false : true;
      // Counter
      job.toDelete ? $scope.jobs2delete++ : $scope.jobs2delete--;
    }
  };

  function resetDelete() {
    $scope.jobs.filter(function( job ) {
      return job.toDelete === true;
    }).forEach(function(job) {
      delete job.toDelete;
    });
    $scope.jobs2delete = 0;
  }
  // $scope.$on('job.resetDelete', function() {
  //   resetDelete();
  // });

  $scope.deleteJobs = function() {
    $scope.popup = {
      done: false,
      subTitle: gettextCatalog.getString('THIS ACTION IS IRREVERSIBLE'),
      description: false
    };
    $ionicPopup.show({
      title: gettextCatalog.getString('Delete') + ' ' + ($scope.jobs2delete > 1 ? gettextCatalog.getString('Jobs') : gettextCatalog.getString('Job')),
      subTitle: $scope.popup.subTitle,
      template: '<ion-checkbox class="checkbox-assertive" ng-model="popup.done">' + gettextCatalog.getString('I understand.') + '</ion-checkbox><div class="assertive" ng-show="popup.description">' + gettextCatalog.getString('Please confirm') + '</div>',
      scope: $scope,
      buttons: [{
        text: gettextCatalog.getString('DELETE'),
        type: 'button-assertive',
        onTap: function(e) {
          if (!$scope.popup.done) {
            e.preventDefault();
            $scope.popup.description = true;
          } else {
            $scope.jobs.filter(function( job ) {
              return job.toDelete === true;
            }).forEach(function(job) {
              DbService.deleteJobData([job.id]);
              job.deleted = true;
              delete job.toDelete;
            });
            resetDelete();
          }
        }
      }, {
        text: gettextCatalog.getString('CANCEL'),
        type: 'button-default'
      }]
    });
  };

  $scope.doRefresh  = function() {
    if (HelperService.isOnlineCheck()) {
      JobsFactory.refreshJobs().then(function(result) {
        if (result == true) {
          JobsFactory.getJobs().then(function(results) {
            $scope.jobs = results;
            $scope.jobsCount = Object.keys($scope.jobs).length || 0;
            $scope.$broadcast('scroll.refreshComplete');
          });
        }
        else {
          $scope.$broadcast('scroll.refreshComplete');
        }
      });
    }
    else {
      //@TODO: alert the user.
      $scope.$broadcast('scroll.refreshComplete');
      $ionicPopup.alert({
        title: gettextCatalog.getString('No internet connection'),
        okType: 'button-balanced'
      });
    }
  }
  if (JobsFactory.init == false) {
    if (HelperService.isOnlineCheck()) {
      $scope.doRefresh();
    }
  }
  $scope.$on('data.clear', function() {
    $scope.jobs = {};
    $scope.jobsCount = 0;
  });

  // Back button functionality.
  var deregister = $ionicPlatform.registerBackButtonAction(
    function(e) {
      if($state.is('app.jobs')) {
        var confirmPopup = $ionicPopup.confirm({
          title: gettextCatalog.getString('Confirm Exit'),
          template: gettextCatalog.getString("Are you sure you want to quit?"),
          okText: gettextCatalog.getString('QUIT'),
          okType: 'button-balanced'
        });
        confirmPopup.then(function (close) {
          if (close) {
            // Close the app.
            ionic.Platform.exitApp();
          }
          else {
            // Do nothing
            e.preventDefault();
            return false;
          }
        });
      }
    }, 101
  );
  $scope.$on('$destroy', deregister);

})

angular.module('ib.controllers')
.controller('KeyboardSettingsCtrl', function($scope, DbService) {

  $scope.model = $scope.model || {};

  $scope.moveKeyboardItem = function(item, fromIndex, toIndex) {
    $scope.settings.keyboard.splice(fromIndex, 1);
    $scope.settings.keyboard.splice(toIndex, 0, item);
    DbService.storeSettings($scope.settings);
  };

  $scope.keyboardItemDelete = function(item) {
    $scope.settings.keyboard.splice($scope.settings.keyboard.indexOf(item), 1);
    DbService.storeSettings($scope.settings);
  };

  $scope.addKeyboardItem = function() {
    var newItem = {name: '', value: ''};
    $scope.settings.keyboard.push(newItem);
    DbService.storeSettings($scope.settings);
  };

})

angular.module('ib.controllers')
.controller('LoginCtrl', function($scope, $state, AuthenticationService, localStorageService, gettextCatalog) {
  $scope.isLoginState = false;
  $scope.message = "";

  $scope.user = {
    username: localStorageService.get('username') || null,
    password: null
  };

  $scope.login = function() {
    AuthenticationService.login($scope.user);
  };


  $scope.$on('event:auth-forbidden', function(e, rejection) {
    if (rejection.config.url.indexOf($scope.endPointRoot) > -1) {
      $scope.user = {
        username: localStorageService.get('username'),
        password: localStorageService.get('keystring')
      };
      if ($scope.user.username && $scope.user.password) {
        AuthenticationService.login($scope.user);
      }
      else {
        $scope.isLoginState = true;
        $scope.loginModal.show();
      }
    }
  });

  $scope.$on('event:auth-loginConfirmed', function() {
    $scope.message = "";
    // $scope.user.username = null;
    $scope.user.password = null;
    $scope.loginModal.hide();
    $scope.isLoginState = false;
  });

  $scope.$on('event:auth-login-failed', function(e, status) {
    if (!$scope.isLoginState) {
      $scope.isLoginState = true;
      $scope.loginModal.show();
    }
    else {
      var error = "Login failed.";
      if (status == 401) {
        error = gettextCatalog.getString("Invalid Username or Password.");
      }
      $scope.message = error;
    }
  });

})

angular.module('ib.controllers')
.controller('NetworkSettingsCtrl', function($scope, DbService) {

  $scope.settings.imageUploadNotify = typeof $scope.settings.imageUploadNotify !== 'undefined' ? $scope.settings.imageUploadNotify : true;

  $scope.toggleSyncPref = function(item, fromIndex, toIndex) {
    $scope.settings.imageUploadNotify == $scope.settings.imageUploadNotify == true ? false : true;
    DbService.storeSettings($scope.settings);
  };

})

angular.module('ib.controllers')
.controller('ObjCtrl', function($scope, $state, $stateParams, $ionicPopup, $ionicModal, $ionicLoading, $cordovaCamera, FieldObjFactory, DbService, fieldObj, pics, currObj, $timeout, propList, childType, objLength, sortedListInit, sortedItemId, settings, fieldsStatus, formFieldInfo, $log, JobsFactory, jobInit, $location, $q, parentId, $ionicTabsDelegate, $ionicHistory, objId, tab, GotoService, $filter, fod, gettextCatalog, tableViewFieldsMap, relatedPlots, $cordovaFile) {
  $scope.relatedPlots = relatedPlots;
  $scope.fieldsStatus = fieldsStatus;
  $log.info("ObjCtrl enter time " + Date.now(), {'controllerStartTime' : Date.now()});

  $scope.$on('$stateChangeSuccess', function() {
    var search = $location.search();
    if (!('objId' in search) || !('parentId' in search)) {
      // $scope.loadedViewClass = "loaded-view";
      $location.search({parentId: $scope.parentId,objId: $scope.objId, tab: $state.current.name.replace("app.job.obj.tabs.tab-", "")});
    }
  });

  $scope.$on('$ionicView.afterEnter', function(scopes, states) {
    var path = $location.path();
    if (path.indexOf('tabs') > -1) {
      $scope.tab = $state.current.name.replace('app.job.obj.tabs.tab-', '');
    }
    else if (path.indexOf('quick-edit') > -1) {
      $scope.tab = 'quick-edit';
    }
    if (states.stateParams) { // Otherwise run too early (twice)
      loadedViewHandler();
    }
  });

  $scope.$on('childobj.updated', function() {
    loadedViewHandler();
    GotoService.endGoto();
  });

  var loadedViewHandler = function() {
    $ionicLoading.hide();
    $scope.loadedViewClass = "loaded-view";
    $scope.transitionClass = "";
    $scope.$broadcast('obj.calcContentTop');
  };

  // $scope.$on('objctrl.update', function() {
  // });

  // $scope.$on('obj.viewLeaving', function(event) {
  //   $scope.loadedViewClass = '';
  // });

  // $scope.$on('$ionicView.afterEnter', function (event, toState, toParams, fromState, fromParams) {
  //  $ionicLoading.hide();
  // });

  $scope.$on('$locationChangeSuccess', function (event) {
   
    // Bail if we're not on tabs/quick-edit view
    var path = $location.path();
    if ($state.current.name.indexOf("app.job.obj.tabs.tab") == -1 && path.indexOf('quick-edit') === -1) {
      return;
    }

    var search = $location.search(),
        parentChanged = false,
        childChanged = false,
        tabChanged = false;
    //Check if parent changed
    if ('parentId' in search && search.parentId != $scope.parentId) {
      $scope.parentId = search.parentId;
      parentChanged = true;
    }
    //Check if tab changed
    if ('tab' in search && search.tab != $scope.tab) {
      tabChanged = true;
    }
    //Check if child changed
    if ('objId' in search && search.objId != $scope.objId) {
      childChanged = true;
      $scope.objId = search.objId;
    }

    if (parentChanged) {
      setActiveItem();
      handleRelatedMode();
      setChildChild();
      FieldObjFactory.setParentData($stateParams.jobId, $scope.parentId).then(function() {

        setObjData().then(function() {
          if (tabChanged) {
            $location.path($state.href("app.job.obj.tabs.tab-" + search.tab).replace("#", "")).search({parentId: search.parentId,objId: search.objId, tab: search.tab});
            $scope.tab = search.tab;
          }
          else{
            $scope.$broadcast('objctrl.update');
          }
        });
      });
    }
    else if (childChanged) {
      setActiveItem();
      handleRelatedMode();
      setObjData().then(function() {
        if (tabChanged) {
          $location.path($state.href("app.job.obj.tabs.tab-" + search.tab).replace("#", "")).search({parentId: search.parentId,objId: search.objId, tab: search.tab});
          $scope.tab = search.tab;
        }
        else {
          $scope.$broadcast('objctrl.update');
        }
      });
    }
  });

  function setChildChild() {
    var childFod = FieldObjFactory.getChildFod(JobsFactory.job.data, $scope.objId);
    if (childFod) {
      $scope.childType = FieldObjFactory.getChildType(childFod);
      $scope.childTypes = getPluralStr($scope.childType);
      if (childFod.ScObjChildDefId) {
        var childChildFod = $filter('filter')(JobsFactory.job.data.ScObjDefList.ScObjDef, {ScObjDefId: childFod.ScObjChildDefId
        }, true)[0];
        if (childChildFod) {
          $scope.childChildType = childChildFod.ScObjType;
          $scope.childChildTypes = getPluralStr($scope.childChildType);
        }
        else {
          $scope.childChildType = false
        }
      }
      else {
        $scope.childChildType = false
      }

      $scope.propList = FieldObjFactory.getPropList(childFod);
      $scope.tableViewFieldsMap = FieldObjFactory.getPropListForTableView(childFod);
    }
    else {
      $scope.childType = false;
      $scope.childChildType = false;
      $scope.propList = [];
    }

  }

  function setObjData() {
    var deferred = $q.defer();
    $scope.sortedItemId = FieldObjFactory.getSortedItemId(FieldObjFactory.sortedList, $scope.objId);
    $scope.fieldObj = FieldObjFactory.setFieldObjProp(FieldObjFactory.findObject(JobsFactory.job.data.ScoredObjectsList.ScoredObject, $scope.objId), FieldObjFactory.parentValues);
    $scope.formFieldInfo = FieldObjFactory.getFormFieldInfo($scope.fieldObj);
    $scope.objMeta = FieldObjFactory.getObjectMeta(JobsFactory.job.data, $scope.fieldObj);


    if ('RelatedPlotsList' in $scope.fieldObj) {
      FieldObjFactory.relatedPlots = $scope.fieldObj.RelatedPlotsList.RelatedPlot;
      if (!(Array.isArray(FieldObjFactory.relatedPlots))) {
        FieldObjFactory.relatedPlots = [FieldObjFactory.relatedPlots];
      }


      // $scope.relatedPlots = FieldObjFactory.relatedPlots.length;
      $scope.relatedPlots = FieldObjFactory.relatedPlots.length;
    }
    else {
      FieldObjFactory.relatedPlots = false;
      $scope.relatedPlots = false;
    }

    $scope.disabledBtn = false;
    FieldObjFactory.setFod(JobsFactory.job.data, $scope.objId);
    DbService.getFieldsStatus($scope.objId).then(function(fs) {
      $scope.fieldsStatus = fs;
      FieldObjFactory.getItemStatus($scope.objId, FieldObjFactory.sortedList[$scope.sortedItemId]).then(function(res) {
        $scope.currObj = res;
        FieldObjFactory.setJobFieldChildes($stateParams.jobId, $scope.objId).then(function(res) {
          $scope.objLength = FieldObjFactory.objs.length;
          DbService.getJobImages($stateParams.jobId, $scope.objId).then(function(images){
            $scope.pics = images;
            setNextAndBackState();
            deferred.resolve();
          });
        });
      });
    });
    return deferred.promise;
  }

  // Plural child type
  function getPluralStr(str) {
    // ToDo: English only, this will need fix for other languages.
    // Since the 'childType' comes from the job, the job will need to send the
    // pluralized string as well.
    return str + 's';
  }

  $scope.tableViewFieldsMap = tableViewFieldsMap;

  // Parnet codepropList
  $scope.settings = settings;
  // Obj code
  $scope.parentId = parentId;
  $scope.formFieldInfo = formFieldInfo;

  $scope.sortedItemId = sortedItemId;
  $scope.settings = settings;
  $scope.objLength = objLength;
  $scope.propList = propList;
  $scope.pics = pics;
  $scope.currObj = currObj;
  setChildChild();

  $scope.fieldObj = fieldObj;
  $scope.jobId = $stateParams.jobId;
  $scope.objId = objId;
  $scope.tab = tab;
  $scope.disabledBtn = false;
  $scope.objMeta = FieldObjFactory.getObjectMeta(JobsFactory.job.data, $scope.fieldObj);
  $scope.jobDone = JobsFactory.job.status >= 3 ? true : false;
  setNextAndBackState();
  $scope.limitInfoItems = true;
  $scope.loadedViewClass = '';
  $scope.transitionClass = '';

  setActiveItem();
  handleRelatedMode();

  $scope.$broadcast('objctrl.update');

  $scope.goToParent = function() {
    if ('parentId' in $scope && $scope.parentId != 0) {
      var parent = FieldObjFactory.findObject(JobsFactory.job.data.ScoredObjectsList.ScoredObject, $scope.parentId);
      GotoService.goTo(parent.ParentUniqueId, $scope.parentId, "items");
    }
    else {
      $ionicLoading.show();
      $state.go('app.job.child', {
        jobId: $scope.jobId
      });
    }
  };

  $scope.transitionTo = function(dir) {
    // Move item using css transform.
    $scope.transitionClass = 'transition-to transition-' + dir;
    $scope.loadedViewClass = '';
    $timeout(function() {
      $scope.transitionClass = '';
    }, 400);
    $scope.goTo(dir);
  };

  $scope.goTo = function(dir) {

    var destObj;
    if (dir == 'next') {
      destObj = $scope.sortedItemId + 1;
    }
    else {
      destObj = $scope.sortedItemId - 1;
    }

    var goToId = FieldObjFactory.sortedList[destObj] ? FieldObjFactory.sortedList[destObj].id : false;

    if (goToId) {
      $scope.$broadcast('objctrl.transition');
      $scope.loadedViewClass = '';
      $ionicLoading.show();
      GotoService.goTo($scope.parentId, goToId, $scope.tab, "trans");
    }

  };

  // Quick edit
  $scope.toggleQuickEdit = function() {
    if ($location.path().indexOf('quick-edit') === -1) {
      // Coming from another view, go to quick edit.
      GotoService.goTo($scope.parentId, $scope.objId, 'quick-edit');
    }
    else {
      // Quick edit, go to tabs.
      GotoService.goTo($scope.parentId, $scope.objId, 'edit');
    }
  };

  // Pictures
  $scope.getPics = function() {
    DbService.getJobImages($scope.jobId, $scope.objId).then(function(images) {
      $scope.pics = images;
    });
  };

  $scope.$on('pics.newPic', function() {
    $scope.pics = $scope.getPics();
  });

  // Take picture
  $scope.newPic = function() {
    var options = {
      quality: 100,
      correctOrientation: true,
      saveToPhotoAlbum: true
    };
    $cordovaCamera.getPicture(options).then(function(imageURI) {
      if (imageURI) {
        DbService.insertJobImage($scope.jobId, $scope.objId, 0, Date.now() / 1000, imageURI, $scope.fieldObj);
        $scope.pics = $scope.getPics();
      }
    }, function(err) {
      if (err != "Camera cancelled.") {
        $ionicPopup.alert({
          title: gettextCatalog.getString('Something went wrong'),
          template: err
        });
      }
    });
  };
//  $scope.newPic = function() {
//// capture callback
//    var captureSuccess = function(mediaFiles) {
//      var i, path, len;
//      for (i = 0, len = mediaFiles.length; i < len; i += 1) {
//        path = mediaFiles[i].fullPath;
//        // do something interesting with the file
//      }
//    };
//
//// capture error callback
//    var captureError = function(error) {
//      navigator.notification.alert('Error code: ' + error.code, null, 'Capture Error');
//    };
//
//// start audio capture
//    navigator.device.capture.captureImage(captureSuccess, captureError, {limit:2});
//
////    var options = {
////      quality: 100,
////      correctOrientation: true,
////      saveToPhotoAlbum: true
////    };
////    $cordovaCamera.getPicture(options).then(function(imageURI) {
////      if (imageURI) {
////        DbService.insertJobImage($scope.jobId, $scope.objId, 0, Date.now() / 1000, imageURI, $scope.fieldObj);
////        $scope.pics = $scope.getPics();
////      }
////    }, function(err) {
////      console.err(err);
////      $ionicPopup.alert({
////        title: gettextCatalog.getString('Something went wrong'),
////        template: gettextCatalog.getString('Unable to take picture.')
////      });
////    });
//  };

  // Add new item
  $scope.$on('obj.updateLists', function() {
    FieldObjFactory.setJobFieldChildes($scope.jobId, $scope.objId).then(function(objs) {
      $scope.objLength = FieldObjFactory.objs.length;
      $timeout(function(){
        $ionicLoading.hide();
        $scope.$broadcast('childList.refilter');
      });
    });
  });

  // Item info pane
  $scope.$on('$ionicView.afterEnter', function(event) {
    if (angular.element(document.querySelector('[ib-content-top]:not(.processed-content-top)')).length) {
      FieldObjFactory.getInfoState($scope.jobId).then(function(infoPaneExpanded) {
        $scope.limitInfoItems = infoPaneExpanded ? false : true;
        // $scope.calcInfoPaneHeight();
        $timeout(function() {
          $scope.$broadcast('obj.calcContentTop');
        });
        angular.element(document.querySelector('[ib-content-top]')).addClass('processed-content-top');
      });
    }
  });
  // Moved to directive
  $scope.calcInfoPaneHeight = function() {
    // $timeout(function() {
    //   var headerEl = document.getElementsByTagName('ion-header-bar')[0],
    //       tabsEl = document.getElementsByClassName('tabs')[0]
    //       infoPaneEl = document.getElementById('info-pane'),
    //       contentTop = 0;
    //   contentTop += angular.isDefined(headerEl) ? headerEl.offsetHeight : 0;
    //   contentTop += angular.isDefined(tabsEl) ? tabsEl.offsetHeight : 0;
    //   contentTop += angular.isDefined(infoPaneEl) ? infoPaneEl.offsetHeight : 0;
    //   $scope.contentTop = contentTop + 'px';
    // });
  }
  $scope.toggleShowMore = function(event) {
    event.stopPropagation();
    FieldObjFactory.toggleInfoState($scope.jobId).then(function(infoPaneExpanded) {
      $scope.limitInfoItems = infoPaneExpanded ? false : true;
      $timeout(function() {
        $scope.$broadcast('obj.calcContentTop');
        // $scope.calcInfoPaneHeight();
      });
    });
  }
  $scope.$broadcast('obj.calcContentTop');
  // $scope.contentTop = $scope.calcInfoPaneHeight();

  // Next/Back buttons state
  function setNextAndBackState() {
    if (!FieldObjFactory.sortedList[$scope.sortedItemId + 1]) {
      $scope.disabledNext = true;
    }
    else {
      $scope.disabledNext = false;
    }

    if (!FieldObjFactory.sortedList[$scope.sortedItemId - 1]) {
      $scope.disabledPrev = true;
    }
    else {
      $scope.disabledPrev = false;
    }
  }

  // Active item
  function setActiveItem() {
    FieldObjFactory.setActiveItem($scope.jobId, $scope.parentId, $scope.objId);
  }

  // Favorites
  $scope.toggleFavorite = function() {
    var favState;
    favState = FieldObjFactory.toggleFavorite($scope.jobId, $scope.objId);
    $scope.currObj.fav = favState;
    FieldObjFactory.sortedList[$scope.sortedItemId].fav = favState;
    DbService.updateJobFieldsChildes($scope.jobId, $scope.parentId, FieldObjFactory.orderdlist);
  };

  // History modal
  $scope.historyModal = function(form) {
    $ionicModal.fromTemplateUrl('templates/history-modal.html', function(modal) {
      $scope.gridModal = modal;
      if (form && (form.historyList || form.History)) {
        // ToDo: Unify the structure of the form element & info element
        $scope.historyTitle = form.title || form.Name;
        $scope.historyList = form.historyList || form.History;
        $scope.historyUnit = form.unit || form.Unit;
        $scope.gridModal.show();
      }
    }, {
      scope: $scope,
      animation: 'slide-in-up'
    });
  };
  // close video modal
  $scope.closeHistoryModal = function() {
    $scope.gridModal.hide();
  };

  $scope.$on('$destroy', function() {
    if ($scope.gridModal) {
      $scope.gridModal.remove();
    }
  });

  // Related modal
  $scope.openRelatedList = function(form) {
    $ionicModal.fromTemplateUrl('templates/related-modal.html', function(modal) {
      $scope.relatedModal = modal;
      FieldObjFactory.getRelatedItems($stateParams.jobId).then(function(relatedItems){
        $scope.relatedItems = relatedItems;
        $scope.relatedModal.show();
      })
    }, {
      scope: $scope,
      animation: 'slide-in-up'
    });
  };
  // close video modal
  $scope.closeRelatedList = function() {
    $scope.relatedModal.hide();
  };

  $scope.$on('$destroy', function() {
    if ($scope.relatedModal) {
      $scope.relatedModal.remove();
    }
  });

  // Related plots
  function handleRelatedMode() {
    var rid = FieldObjFactory.getRelatedMode();
    if (rid == $scope.objId) {
      $scope.exitRelatedMode();
      $scope.relatedMode = false;
    }
    else if (rid) {
      $scope.relatedMode = true;
    }
  }
  
  $scope.enterRelatedMode = function () {
    FieldObjFactory.enterRelatedMode($scope.objId);
  }

  $scope.exitRelatedMode = function () {
    FieldObjFactory.exitRelatedMode();
  }

  $scope.backToRelated = function () {
    var rid = FieldObjFactory.getRelatedMode();
    GotoService.goTo(0, rid, 'edit');
  }

});

angular.module('ib.controllers')

.controller('ObjEditCtrl', function($scope, $log, formObject, $location, FieldObjFactory, $stateParams, $ionicLoading, GotoService) {
  $scope.$on('objctrl.update', function() {
    FieldObjFactory.getProcessedForm($stateParams.jobId, 1, $scope.fod).then(function(formObject){
      $scope.formObject = formObject;
      $scope.$emit('childobj.updated');
    });
  });
  $scope.formObject = formObject;
  $log.info("ObjEditCtrl enter time " + Date.now(), {'controllerStartTime' : Date.now()});
})

.controller('DescObjEditCtrl', function($scope, $log, decisionsFormObject, FieldObjFactory, $stateParams, $ionicLoading, GotoService) {
  $scope.$on('objctrl.update', function() {
    FieldObjFactory.getProcessedForm($stateParams.jobId, 2).then(function(decisionsFormObject){
      $scope.decisionsFormObject = decisionsFormObject;
      GotoService.endGoto();
      $scope.$emit('childobj.updated');
    });
  });
  $scope.decisionsFormObject = decisionsFormObject;
  $log.info("DescObjEditCtrl enter time " + Date.now(), {'controllerStartTime' : Date.now()});
})

angular.module('ib.controllers')
.controller('ObjInfoCtrl', function($scope, $log, objInfo, FieldObjFactory, JobsFactory) {
  $scope.$on('objctrl.update', function() {
    $scope.objInfo = FieldObjFactory.getObjectInfo(JobsFactory.job.data);
    $scope.$emit('childobj.updated');
  });
  $scope.objInfo = objInfo;
  $log.info("ObjInfoCtrl enter time " + Date.now(), {'controllerStartTime' : Date.now()});
  $scope.isLongText = function(item) {
    return item.Type == "7";
  };
  $scope.isNotLongText = function(item) {
    return !$scope.isLongText(item);
  };
  $scope.hasNoValue = function(item) {
    return !item.Value && !item.FemaleParentValue && !item.MaleParentValue;
  };
  $scope.hasParentValue = function(item) {
    return item.FemaleParentValue || item.MaleParentValue;
  };

});

angular.module('ib.controllers')
.controller('ObjItemsCtrl', function($scope, $stateParams, $state, $ionicModal, $ionicPopup, $ionicLoading, $cordovaBarcodeScanner, FieldObjFactory, DbService, JobsFactory, $location, GotoService, gettextCatalog) {
  var addItemPopup;
  function resetDelete() {
    // if ($scope.objs2delete) { // WTF - this breaks the flow between tabs
    FieldObjFactory.objs.filter(function( obj ) {
        return obj.toDelete === true;
      }).forEach(function(obj) {
        delete obj.toDelete;
      });
      $scope.objs2delete = 0;
      // Should we trigger this here?
      $scope.$emit('obj.updateLists');
    // }
  }
  $scope.$on('childList.resetDelete', function() {
    resetDelete();
  });

  function initScope() {
    $scope.newItems = {};
    $scope.newItems.number = 1;
    $scope.newItems.preDefinedID = '';
    $scope.validationError = false;
    $scope.objs2delete = 0;
    resetDelete();
  }
  $scope.$on('objctrl.update', function() {
    initScope();
  })
  initScope();

  $scope.initAddPopup = function() {
    // Get preDefinedIDs
    $scope.preDefinedIDs = FieldObjFactory.fod.preDefinedIDs;
  }

  // Open new item popup
  $scope.openNewItemPopup = function(event) {
    $scope.addLocked = false;
    $scope.newItems.preDefinedID = '';
    $scope.resetItems();

    if ($scope.fieldObj.numChildsToAdd && !FieldObjFactory.getUsedNumChildsToAdd($scope.jobId, $scope.objId)) {
      $scope.createItems($scope.fieldObj.numChildsToAdd, $scope.newItems.number);
    }

    $ionicModal.fromTemplateUrl('templates/add-new-item.html', function(modal) {
        addItemPopup = modal;
        addItemPopup.show();
      },
      {
        scope: $scope,
        animation: 'slide-in-up',
        focusFirstInput: true,
        backdropClickToClose: false,
        hardwareBackButtonClose: false
      }
    );
    //Be sure to cleanup the modal by removing it from the DOM
    $scope.$on('$destroy', function() {
      addItemPopup.remove();
    });

    $scope.addItemPopupSubmit = function () {
      if (!$scope.addLocked) {
        $scope.addLocked = true;
        addItems($scope.newItems.items);
      }
    }

    $scope.addItemPopupHide = function () {
      addItemPopup.hide();
    }

    // addItemPopup = $ionicPopup.show({
    //   templateUrl: 'templates/add-new-item.html',
    //   scope: $scope,
    //   buttons: [
    //     {
    //       text: gettextCatalog.getString('ADD'),
    //       type: 'button-balanced',
    //       onTap: function(e) {
    //         e.preventDefault();
    //         if (!$scope.addLocked) {
    //           $scope.addLocked = true;
    //           addItems($scope.newItems.items);
    //         }
    //       }
    //     },
    //     { text: gettextCatalog.getString('CANCEL') }
    //   ]
    // });
  };

  $scope.setPreDefinedID = function() {
    $scope.validationError = false;
    $scope.newItems.items = [{suggestedTitle: $scope.newItems.preDefinedID}];
    $scope.newItems.items
      .forEach(function(item) {
        if (!validateUniqueId(item.suggestedTitle)) {
          item.validationError = true;
          $scope.validationError = true;
        }
      });
    $scope.$broadcast('event:updateItemsLength');
  }

  function getNextAvailableID(addingNewItem) {

    // Initial title is the number of existing items + number of new items + 1 for the current item.
    var num = FieldObjFactory.objs.length;

    // If we have a bunch of newItems with suggestedTitle, count them.
    if ($scope.newItems && $scope.newItems.items) {
      var hasSuggestedTitle = $scope.newItems.items.filter(function ( obj ) {
        return obj.suggestedTitle;
      });
      num = num + hasSuggestedTitle.length;
    }

    // While creating an extra item, count it.
    if (addingNewItem) {
      num = num + 1;
    }
    else if (num == 0) {
      num = 1;
    }

    var found = true,
        title = num;
    // Search if 'title' exist in $scope.objs
    while (angular.isDefined(found)) {
      title = num;

      found = FieldObjFactory.objs.filter(function ( obj ) {
        return obj.title == title;
      })[0];

      // Now search also in the "newItems"
      if (angular.isUndefined(found) && $scope.newItems && $scope.newItems.items && $scope.newItems.items.length) {
        found = $scope.newItems.items.filter(function ( obj ) {
          return obj.id == title ||  obj.suggestedTitle == title;
        })[0];
      }

      num += 1;
    };

    return title;
  }

  $scope.$on('event:updateItemsLength', function() {
    if ($scope.newItems.number) {
      $scope.newItems.number = $scope.newItems.items.length;
    }
  });

  $scope.pushItem = function() {
    if ($scope.newItems.preDefinedID) {
      $scope.newItems.preDefinedID = '';
      $scope.resetItems();
    }
    $scope.newItems.items.push({
      suggestedTitle: getNextAvailableID(true)
    });
    $scope.$broadcast('event:updateItemsLength');
  }

  $scope.popItem = function() {
    if ($scope.newItems.items.length > 1) {
      $scope.newItems.items.pop()
    }
    $scope.$broadcast('event:updateItemsLength');
  }

  $scope.createItems = function(newVal, oldVal) {
    if (isNaN(newVal)) {
      return;
    }
    var oldVal = oldVal || 1;
        diff = newVal - oldVal;
    if (diff > 0) {
      for (var i = 0; i < diff; i++) {
        $scope.pushItem()
      }
    }
    else if (diff < 0) {
      for (var i = 0; i > diff; i--) {
        $scope.popItem();
      }
    }
  }

  $scope.resetItems = function() {
    $scope.validationError = false;
    $scope.newItems.items = [];
    $scope.pushItem();
    $scope.$broadcast('event:updateItemsLength');
  }

  function validateUniqueId(title) {
    for (var i = 0; i < FieldObjFactory.objs.length; i++) {
      if (FieldObjFactory.objs[i].title.toString().trim() === title.trim()) {
        return false;
      }
    }
    var foundInNew = 0;
    for (var i = 0; i < $scope.newItems.items.length; i++) {
      if ($scope.newItems.items[i].hasOwnProperty('id')) {
        if ($scope.newItems.items[i].id == title) {
          foundInNew++;
        }
      }
      else if ($scope.newItems.items[i].hasOwnProperty('suggestedTitle')) {
        if ($scope.newItems.items[i].suggestedTitle == title) {
          foundInNew++;
        }
      }
      if (foundInNew > 1) {
        return false;
      }
    }
    return true; // Unique
  }

  $scope.reValidateNewItems = function() {
    // Remove suggested items
    $scope.newItems.items
      .forEach(function(item) {
        delete item.suggestedTitle;
      });
    // Validate IDs
    $scope.validationError = false;
    $scope.newItems.items
      .filter(function(item) {
        return item.id;
      })
      .forEach(function(item) {
        item.validationError = false;
        if (!validateUniqueId(item.id)) {
          item.validationError = true;
          $scope.validationError = true;
        }
      });
    // Updated suggested items
    $scope.newItems.items
      .filter(function(item) {
        return !item.id;
      })
      .forEach(function(item) {
        item.validationError = false;
        item.suggestedTitle = getNextAvailableID();
      });
  }

  function addItems(items) {
    var titles = [];
    for (var i = 0; i < items.length; i++) {
      var title;
      if (items[i].id) {
        title = items[i].id.toString();
      }
      else {
        title = items[i].suggestedTitle.toString();
      }
      titles.push(title);
    }
    if ($scope.validationError == false) {

      $ionicLoading.show();
      FieldObjFactory.addNewObjects(JobsFactory.job, $scope.objId, titles).then(function(newObjs) {
        addItemPopup.hide();
        $scope.addLocked = false;
        for (var i = 0; i < newObjs.length; i++) {
          FieldObjFactory.objs.push(FieldObjFactory.getObjectsListItemInfo(JobsFactory.job.data, newObjs[i], FieldObjFactory.getChildFod(JobsFactory.job.data, $scope.objId)));
        }
        DbService.updateJobFieldsChildes($scope.jobId, $scope.objId, FieldObjFactory.objs);
        FieldObjFactory.sortedList[$scope.sortedItemId].childObjs = FieldObjFactory.objs.filter(function (obj) { return obj.deleted !== true; }).length;

        DbService.updateJobFieldsChildes($scope.jobId, $scope.parentId, FieldObjFactory.orderdlist);

        if ($scope.newItems.items.length > 1) {
          if ($scope.fieldObj.numChildsToAdd) {
            FieldObjFactory.usedNumChildsToAdd($scope.jobId, $scope.objId);
          }
        }
        else {
          GotoService.goTo(newObjs[0].ParentUniqueId, newObjs[0].ObjectUniqueId, "edit");
        }

        $scope.$emit('obj.updateLists');

        // Cleanup
        $scope.newItems.preDefinedID = '';
        $scope.resetItems();
      });
    }
    else {
      $scope.addLocked = false;
    }
  }

  $scope.toggleItemDeletion = function(objId) {
    var obj = FieldObjFactory.objs.filter(function( obj ) {
      return obj.id == objId;
    })[0];
    if (obj) {
      // Flag it
      obj.toDelete = obj.toDelete ? false : true;
      // Counter
      obj.toDelete ? $scope.objs2delete++ : $scope.objs2delete--;
    }
  };

  $scope.deleteObjs = function() {
    $scope.popup = {
      done: false,
      subTitle: gettextCatalog.getString('THIS ACTION IS IRREVERSIBLE'),
      description: false
    };
    $ionicPopup.show({
      title: gettextCatalog.getString('Delete') + ' ' + ($scope.objs2delete > 1 ? $scope.childTypes : $scope.childType),
      subTitle: $scope.popup.subTitle,
      template: '<ion-checkbox class="checkbox-assertive" ng-model="popup.done">' + gettextCatalog.getString('I understand.') + '</ion-checkbox><div class="assertive" ng-show="popup.description">' + gettextCatalog.getString('Please confirm') + '</div>',
      scope: $scope,
      buttons: [{
        text: gettextCatalog.getString('DELETE'),
        type: 'button-assertive',
        onTap: function(e) {
          if (!$scope.popup.done) {
            e.preventDefault();
            $scope.popup.description = true;
          } else {
            // ToDo: update parent (counters);
            FieldObjFactory.objs.forEach(function(obj, index, objs) {
              if (obj.toDelete) {
                obj.deleted = true;
                DbService.updateJobFieldIsNew(obj.id, 2);
                delete obj.toDelete;
              }
            });
            DbService.updateJobFieldsChildes($scope.jobId, $scope.objId, FieldObjFactory.objs);
            resetDelete();
            FieldObjFactory.sortedList[$scope.sortedItemId].childObjs = FieldObjFactory.objs.filter(function (obj) { return obj.deleted !== true; }).length;
            DbService.updateJobFieldsChildes($scope.jobId, $scope.parentId, FieldObjFactory.orderdlist);
            // DbService.updateJobSyncStatus($scope.jobId, 0).then(function(){
              $scope.$emit('obj.updateLists');
            // });
          }
        }
      }, {
        text: gettextCatalog.getString('CANCEL'),
        type: 'button-default'
      }]
    });
  }

  $scope.scanNewItemQR = function() {
    $cordovaBarcodeScanner
      .scan()
      .then(function(barcodeData) {
        $scope.newItems.items[0].id = barcodeData.text;
      }, function(error) {
        $ionicPopup.alert({
          title: gettextCatalog.getString('Something went wrong'),
          template: gettextCatalog.getString('Unable to scan the QR code.')
        });
      });
  }

})

'use strict';
angular.module('ib.controllers')

.controller('ObjQuickEditCtrl', function($scope, $window, $log, $filter, quickFormObject, FieldObjFactory, $stateParams, GotoService, $timeout, $ionicModal, JobsFactory, $location) {

  $scope.$on('$locationChangeSuccess', function(event, url) {
    if (url.indexOf('/quick-edit') > -1) {
      $scope.activeIndex = 0;
      $scope.horzScroll = '0px';
    }
  });

  $scope.quickEditObjId = $scope.objId;
  $scope.$on('$stateChangeSuccess', function(event, toState) {
    if (toState.url == '/quick-edit' && $scope.quickEditObjId != $scope.objId) {
      $scope.activeIndex = 0;
      $scope.horzScroll = '0px';
      $scope.$emit('objctrl.update');
    }
  });

  $scope.$on('objctrl.update', function() {
    $scope.quickEditFieldsMap = initFieldsMap();
    FieldObjFactory.getProcessedForm($stateParams.jobId, 1).then(function(formObject){
      // COMMENTED TO REMOVE DECISIONS FIELDS FROM QUICK EDIT
      // FieldObjFactory.getProcessedForm($stateParams.jobId, 2).then(function(decisionsFormObject){
      //   quickFormObject = formObject.concat(decisionsFormObject);
        quickFormObject = formObject;
        GotoService.endGoto();
        quickEditBtnsState();
        $scope.quickEditObjId = $scope.objId;
        $scope.$emit('childobj.updated');
        $scope.$emit('quickFormObject.rebuild');
      // });
    });
  });

  $scope.$on('quickFormObject.rebuild', function() {
    // Filter quickFormObject using the values in quickEditFieldsMap
    $scope.quickFormObject = $filter('filter')(quickFormObject, function(value) {
      // Return the right value from $scope.quickEditFieldsMap
      return $filter('filter')($scope.quickEditFieldsMap, {pd: value.pd})[0].showInQuickEdit;
    });
    $scope.quickFormObjectLength = $scope.quickEditFieldsMap.filter(function (el) {
      return el.showInQuickEdit === true;
    }).length;
    $scope.$broadcast('quickFormObject.autoFocus');
  });

  $log.info('ObjQuickEditCtrl enter time ' + Date.now(), {'controllerStartTime' : Date.now()});

  // Generate initial map
  var initFieldsMap = function() {
    // ToDo: we can move this to the sqlite DB / Job cache.
    var fieldsMap = FieldObjFactory.getQuickEditFieldsMap($scope.jobId, $scope.objMeta.Type);
    if (!fieldsMap.length) {
      quickFormObject.forEach(function(field) {
        fieldsMap.push({
          pd: field.pd,
          title: field.title,
          showInQuickEdit: (angular.isUndefined(field.showInQuickEdit) || field.showInQuickEdit === true)
        });
      });
    }
    return fieldsMap;
  };

  $scope.quickEditFieldsMap = initFieldsMap();
  $scope.$emit('quickFormObject.rebuild');

  $scope.activeIndex = 0;
  $scope.horzScroll = '0px';
  quickEditBtnsState();

  $scope.quickEditBtns = function(dir) {
    var lastItem = $scope.quickFormObject.length - 1,
        shouldScroll = false;
    if ($scope.parentHasNextWithChild && dir === 'next') {
			FieldObjFactory.getNextItem(JobsFactory).then(function (res) {
				GotoService.goTo(res.parentId, res.objId, $scope.tab);
			});
    } else if($scope.parentHasPrevWithChild && dir === 'prev') {
			FieldObjFactory.getPrevItem(JobsFactory).then(function (res) {
				GotoService.goTo(res.parentId, res.objId, $scope.tab);
			});
		}

    else if ($scope.activeIndex === lastItem && dir === 'next') {
      $scope.transitionTo('next');
    }
    else if ($scope.activeIndex === 0 && dir === 'prev') {
      $scope.transitionTo('prev');
    }
    else {
      $scope.activeIndex += (dir === 'next') ? 1 : -1;
      shouldScroll = true;
    }

    if (shouldScroll) {
      $scope.horzScroll = ($scope.activeIndex * -100) + '%';
      $timeout(function() {
        var focusable = document
                        .querySelectorAll('.quick-edit-form .form-repeat')[$scope.activeIndex]
                        .querySelector('[tabindex]');
        if (focusable && focusable.focus) {
          $timeout(function() {
            focusable.focus();
            // focusable.click();
            if($window.cordova && $window.cordova.plugins.Keyboard) {
              cordova.plugins.Keyboard.show();
            }
          }, 400);
        }
      });
    }

    quickEditBtnsState();

  };

  // Next/Prev buttons state
  function quickEditBtnsState() {
    $scope.parentHasNextWithChild = false;
    $scope.parentHasPrevWithChild = false;
    $scope.prevSuffix = '';
    $scope.nextSuffix = '';

    $scope.disabledQuickPrev = false;
    $scope.disabledQuickNext = false;

    // if it's the first form element.
    if ($scope.activeIndex < 1) {
      if ($scope.relatedMode) {
        $scope.disabledQuickPrev = true;
      }
      else if (!FieldObjFactory.sortedList[$scope.sortedItemId - 1]) {
        FieldObjFactory.getPrevItem(JobsFactory).then(function (res) {
          $scope.parentHasPrevWithChild = res !== false;
          if ($scope.parentHasPrevWithChild) {
            $scope.prevSuffix = res.parent.type.toUpperCase();
          }
          else {
            $scope.disabledQuickPrev = true;
          }
        })
      }
    }

    // If it's the last form item.
    if ($scope.activeIndex >= $scope.quickFormObjectLength - 1) {
      if ($scope.relatedMode) {
        $scope.disabledQuickNext = true;
      }
			else if (!FieldObjFactory.sortedList[$scope.sortedItemId + 1]) {
				FieldObjFactory.getNextItem(JobsFactory).then(function (res) {
					$scope.parentHasNextWithChild = res !== false;

					if ($scope.parentHasNextWithChild) {
						$scope.nextSuffix = res.parent.type.toUpperCase();
					}
					else {
						$scope.disabledQuickNext = true;
					}
				});
			}
		}
	}

  $scope.quickEditToggleMenu = function() {
    $ionicModal.fromTemplateUrl('templates/quick-edit-menu.html', function(modal) {
      $scope.quickEditMenu = modal;
      $scope.quickEditMenu.show();
    }, {
      scope: $scope,
      animation: 'slide-in-up'
    });
  };
  // close video modal
  $scope.closeQuickEditMenu = function() {
    $scope.quickEditMenu.hide();
  };
  $scope.$on('modal.hidden', function() {
    $scope.$emit('quickFormObject.rebuild');
    FieldObjFactory.setQuickEditFieldsMap($scope.jobId, $scope.objMeta.Type, $scope.quickEditFieldsMap);
  });
  $scope.$on('$destroy', function() {
    if ($scope.quickEditMenu) {
      $scope.quickEditMenu.remove();
    }
  });

  $scope.checkAll = true;
  $scope.quickEditMenuToggleCheckAll = function(checkAll) {
    $scope.checkAll = checkAll;
    var keys = Object.keys($scope.quickEditFieldsMap);
    for (var i = 0; i < keys.length; i++) {
      $scope.quickEditFieldsMap[keys[i]].showInQuickEdit = checkAll;
    }
  };

  $scope.quickEditAutoAdvance = function(formField) {
    if (formField.type && ['select', 'boolean', 'images'].indexOf(formField.type) != -1) {
      $scope.quickEditBtns('next');
    }
  };

});

angular.module('ib.controllers')
.controller('ObjRelatedCtrl', function($scope, GotoService) {
  $scope.goToWithLoading = function (objId, parentUniqueId) {
    $scope.enterRelatedMode();
    $scope.closeRelatedList();
    GotoService.goTo(0, objId, "edit");
  };
})

angular.module('ib.controllers')
.controller('PicsCtrl', function($scope, $ionicModal, $ionicPopup, DbService, gettextCatalog) {

  function initScope() {
    $scope.deleteMode = false;
    $scope.transitionClass = "";
  }
  $scope.$on('objctrl.update', function() {
    initScope();
    $scope.$emit('childobj.updated');
  })
  initScope();
  function toggleDelete(imgUrl) {
    var pic = $scope.pics.filter(function( pic ) {
      return pic.imgUrl == imgUrl;
    })[0];
    pic.toDelete = pic.toDelete ? false : true;
    $scope.pics2delete = $scope.pics.filter(function( pic ) {
      return pic.toDelete == true;
    });
    if ($scope.pics2delete.length == 0) {
      $scope.deleteMode = false;
    }
  }

  // Enter delete mode
  $scope.imgHold = function(imgUrl) {
    $scope.deleteMode = true;
    toggleDelete(imgUrl);
  };

  // Open image modal OR mark for deletion
  $scope.imgTap = function(imgUrl) {
    if ($scope.deleteMode) {
      toggleDelete(imgUrl);
    }
    else {
      $ionicModal.fromTemplateUrl('templates/img-modal.html', function(modal) {
        $scope.gridModal = modal;
        $scope.imgUrl = imgUrl;
        $scope.gridModal.show();
      }, {
        scope: $scope,
        animation: 'slide-in-up'
      });
    }
  };
  // Close image modal
  $scope.closeImgModal = function() {
    $scope.gridModal.hide();
  };
  $scope.$on('$destroy', function() {
    if ($scope.gridModal) {
      $scope.gridModal.remove();
    }
  });

  // Delete pics
  $scope.deletePics = function() {
    $scope.popup = {
      done: false,
      subTitle: gettextCatalog.getString('THIS ACTION IS IRREVERSIBLE'),
      description: false
    };
    $ionicPopup.show({
      title: gettextCatalog.getString('Delete Images'),
      subTitle: $scope.popup.subTitle,
      template: '<ion-checkbox class="checkbox-assertive" ng-model="popup.done">I understand.</ion-checkbox><div class="assertive" ng-show="popup.description">' + gettextCatalog.getString('Please confirm') + '</div>',
      scope: $scope,
      buttons: [{
        text: gettextCatalog.getString('DELETE'),
        type: 'button-assertive',
        onTap: function(e) {
          if (!$scope.popup.done) {
            e.preventDefault();
            $scope.popup.description = true;
          } else {
            DbService.deleteJobImages($scope.pics2delete).then(function() {
              $scope.pics2delete = [];
              $scope.deleteMode = false;
              $scope.$emit('pics.newPic');
            });
          }
        }
      }, {
        text: gettextCatalog.getString('CANCEL'),
        type: 'button-default'
      }]
    });

  };

})

angular.module('ib.controllers')
.controller('SettingsCtrl', function($scope, $ionicPlatform, $state) {

  $scope.saveSettings = function() {
    DbService.storeSettings($scope.settings);
  };

  // Back button functionality.
  var deregister = $ionicPlatform.registerBackButtonAction(
    function() {
      if($state.is('app.settings.menu')) {
        $state.go('app.jobs');
      }
    }, 101
  );
  $scope.$on('$destroy', deregister);
})
