angular.module('ib.directives', [])

.directive('childList', function () {
  return {
    restrict: 'E',
    controller: 'ChildListCtrl',
    templateUrl: 'templates/child-list.html'
  };
})

.directive('relatedList', function () {
  return {
    restrict: 'E',
    controller: 'ObjRelatedCtrl',
    templateUrl: 'templates/related-list.html'
  };
})

.directive('infoPane', function () {
  return {
    restrict: 'E',
    templateUrl: 'templates/info-pane.html'
  };
})

angular.module('ib.directives')
.directive('breadcrumbs', function(FieldObjFactory, JobsFactory, $state, $ionicLoading, $location, GotoService, gettextCatalog) {

  var jobData, objects;

  var getBreadObj = function(objects, objId) {
    var breadObj = FieldObjFactory.findObject(objects, objId);
    if (breadObj) {
      var breadObjFod = FieldObjFactory.getFod(jobData, objId);
      breadObj.type = breadObjFod.ScObjType;
    }
    return breadObj;
  }

  var breadcrumbsHandler = function(scope, element, attrs) {
    scope.$on('objctrl.update', function() {
      setData();
    })
    scope.$on('$stateChangeSuccess', function() {
      setData();
    })
    jobData = JobsFactory.job.data;
    objects = jobData.ScoredObjectsList.ScoredObject;

    function setData() {
      scope.breadcrumbs = [];
      // Current object
      var breadObj = getBreadObj(objects, scope.currObj.id);
      scope.breadcrumbs.unshift(breadObj);
      // Iterate parent objets
      do {
        breadObj = getBreadObj(objects, breadObj.ParentUniqueId);
        if (breadObj) {
          scope.breadcrumbs.unshift(breadObj);
        }
      } while (breadObj);
      // Job element
      var jobCrumb = {
        ObjectName: jobData.JobName,
        type: gettextCatalog.getString("Job"),
        ObjectUniqueId: "job",
        ObjectId: jobData.JobId

      }

      scope.breadcrumbs.unshift(jobCrumb);
      FieldObjFactory.setBC(scope.breadcrumbs);
      // Jobs element
      var jobsCrumb = {
        ObjectName: gettextCatalog.getString("Jobs"),
        ObjectUniqueId: "jobs"
      }
      scope.breadcrumbs.unshift(jobsCrumb);

    }
    setData();
    // Handle navigation
    scope.goToWithLoading = function(objId, parentUniqueId) {

      if (objId == scope.currObj.id) {
        return false;
      }
//      scope.$emit('obj.viewLeaving');
      $ionicLoading.show();
      if (objId == "jobs") {
        $state.go('app.jobs');
      }
      else if (objId == "job") {
        $state.go('app.job.child', {
          jobId: scope.jobId
        });
      }
      else {
        GotoService.goTo(parentUniqueId,objId, "edit");
      }
    }

  }

  return {
    restrict: 'E',
    link: breadcrumbsHandler,
    template: '<ul class="breadcrumb"><li ng-repeat=\'bc in breadcrumbs\' ng-class="{\'active\': {{$last}} }"><a on-tap-hold="goToWithLoading(bc.ObjectUniqueId, bc.ParentUniqueId)">{{bc.type}} {{bc.ObjectName}}</a><span class="divider" ng-show="! $last"> >> </span></li></ul>'
  };
});

'use strict';
angular.module('ib.directives')
.directive('deletable', function($document, $parse, $compile, gettextCatalog) {
  return {
    restrict: 'A',
    link: function(scope, element, attrs) {

      var startX, x, animationInterval, undo, obj;

      // Bind to 'dragstart'.
      element.on('dragstart', drag);
      scope.$on('$destroy', function() {
        element.off('dragstart', drag);
      });

      function drag() {
        // (child list) Prevent on 'Done' jobs.
        if (scope.jobDone) {
          return;
        }
        obj = $parse(attrs.deletable)(scope);
        // (child list) Bail if NOT isNewItem.
        // We use obj.title to differentiate jobs & plots/plants, jobs don't have 'title'
        if (obj.title && !obj.isNewItem) {
          return;
        }
        // (job list) Allow only done jobs
        // We use obj.jobName to differentiate jobs & plots/plants, only jobs have 'jobName'
        if (obj.jobName && obj.status != 3) {
          return;
        }
        // Prevent default dragging if scrolling down.
        if (event.gesture.velocityX < event.gesture.velocityY) {
          return;
        }
        event.stopPropagation();
        event.gesture.preventDefault();
        // Init.
        x = 0;
        // Set starting point.
        startX = event.gesture.center.pageX - x;
        // Move while dragging.
        $document.on('drag', function(event) {
          x = event.gesture.center.pageX - startX;
          // 5px threshold
          if (Math.abs(x) > 10) {
            move(x);
          }
          else if (x !== 0) {
            move(0);
          }
        });
        $document.on('dragend', release);
      }

      // Move item using css transform.
      function move(x) {
        element.css({
          '-webkit-transform':  'translate3d(' + x + 'px, 0, 0)',
          'transform':  'translate3d(' + x + 'px, 0, 0)',
          'opacity': Math.max(1 - Math.abs(x/200), 0),
        });
      }

      // Animate move().
      function animate(n) {
        animationInterval = setInterval(function() {
          x = x + n;
          if (10 > Math.abs(x)) {
            // Item back in center.
            move(0);
            clearInterval(animationInterval);
            return false;
          }
          else if (Math.abs(x) > window.innerWidth) {
            // Item is outside.
            toBeDeleted();
            clearInterval(animationInterval);
            return false;
          }
          else {
            // Keep animating.
            move(x);
          }
        }, 1);
      }

      // Release handler
      function release() {
        // ToDo: CHECK WHY THESE DON'T WORK
        // $document.off('drag', move);
        // $document.off('dragend', release);
        // THESE WORK BUT REMOVE ALL OF THE BINDINGS OF $document FOR 'drag' and 'dragend'
        $document.off('drag');
        $document.off('dragend');

        var diff;
        if (x > 100 || x < -100) {
          // Mark for deletion.
          diff = x > 0 ? 3 : -3;
          animate(diff);
        }
        else {
          // If not passed threshold animate it back.
          diff = x > 0 ? -2 : 2;
          // Limit x so we don't wait for it too much.
          x = Math.min(Math.max(parseInt(x), -100), 100);
          animate(diff);
        }
      }

      // Mark for deletion.
      function toBeDeleted() {
        undo = angular.element('<div ng-click="undoDelete(' + obj.id + ')" class="undo"><span class="ion-ios-undo">' + gettextCatalog.getString('UNDO') + '</span></div>');
        $compile(undo)(scope);
        scope.toggleItemDeletion(obj.id);
        element
          .append(undo)
          .css({
            '-webkit-transform':  'translate3d(0, 0, 0)',
            'transform':  'translate3d(0, 0, 0)',
            'opacity': 1,
          })
          .removeAttr('style');
        scope.$apply();
      }

      function removeUndo() {
        if (undo) {
          undo.remove();
        }
      }

      scope.undoDelete = function(objId) {
        undo.remove();
        scope.toggleItemDeletion(objId);
      };

      scope.$on('obj.updateLists', removeUndo);
      scope.$on('childList.reorder', removeUndo);
    }
  };
});

angular.module('ib.directives')
.directive('historicValue', function($parse) {

  function renderStyledValue(val, style, cls) {
    cls = cls || '';
    return '<span class="' + cls + ' display-' + style + '">' + val + '</span>'
  }

  function historicValueLink(scope, element, attrs) {
    var hasValue = false;
    var output = '';
    var propData = $parse(attrs.propData);
    var prop = propData(scope);

    if (!prop) {
      return;
    }

    if (prop.oldval != "") {
      hasValue = true;
      output += renderStyledValue(prop.oldval, prop.displayStyle);
    }
    else if (prop.femaleParentVal != "" || prop.maleParentVal != "") {
      hasValue = true;
      output += renderStyledValue(prop.femaleParentVal, prop.femaleParentDisplayStyle, 'female-parent ion-female');
      if (prop.femaleParentVal != "" && prop.maleParentVal != "") {
        output += ' / ';
      }
      output += renderStyledValue(prop.maleParentVal, prop.maleParentDisplayStyle, 'male-parent ion-male');
    }

    if (hasValue) {
      element.replaceWith('<span class="oldval">(' + output + ')</span>');
    }

  }

  return {
    link: historicValueLink
  };
});

'use strict';
angular.module('ib.directives')
.directive('ibAutoFocus', function($timeout, $window) {

  return {
    restrict: 'A',
    link: function(scope, element) {

      // Auto focus every time the quick-edit panel opens.
      scope.$on('$stateChangeSuccess', function(event, toState) {
        if (toState.url == '/quick-edit') {
          $timeout(function() {
            autoFocus();
          });
        }
      });

      // Auto focus after the quick-edit object updates.
      scope.$on('quickFormObject.autoFocus', function() {
        $timeout(function() {
          autoFocus();
        }, 1000);
      });

      var autoFocus = function() {
        var focusable = element[0].querySelector('.form-group [tabindex]');
        if (focusable && focusable.focus) {
          focusable.focus();
          // focusable.click();
          if($window.cordova && $window.cordova.plugins.Keyboard) {
            cordova.plugins.Keyboard.show();
          }
        }
      }
    }
  };

});

'use strict';
angular.module('ib.directives')
.directive('ibContentTop', function($timeout) {

  return {
    restrict: 'A',
    link: function(scope, element, attrs) {
      // ToDo - why this (sometimes) runs too early on the quick edit?
      scope.$on('obj.calcContentTop', function () {
      // Tried with a $watch also
      // scope.$watch('limitInfoItems', function() {
        $timeout(function() {

          var headerEl = document.querySelector('ion-header-bar'),
              tabsEl = document.querySelector('.tabs'),
              infoPaneEl = document.querySelector('[name="objContent"] .pane[nav-view="active"] .info-pane'),
              prop = attrs.ibContentTop,
              contentTop = 0;

          contentTop += headerEl ? headerEl.offsetHeight : 0;
          contentTop += tabsEl ? tabsEl.offsetHeight : 0;
          contentTop += infoPaneEl ? infoPaneEl.offsetHeight : 0;

          element.css(prop, contentTop + 'px');

        });
      });
    }
  };

});

angular.module('ib.directives')
.directive('itemFind', function($ionicPopup, gettextCatalog, FieldObjFactory, JobsFactory, $filter, GotoService, $ionicLoading, $cordovaBarcodeScanner) {

  var itemFindController = function($scope) {

    var findItemPopup;

    // Find item popup
    $scope.findItemPopup = function() {

      $scope.popup = {
        itemToFind: ''
      };

      $scope.scanQR = function() {
        $cordovaBarcodeScanner
          .scan()
          .then(function(barcodeData) {

            $scope.popup.itemToFind = barcodeData.text;
            $scope.findItem($scope.popup.itemToFind);

          }, function(error) {
            $ionicPopup.alert({
              title: gettextCatalog.getString('Something went wrong'),
              template: gettextCatalog.getString('Unable to scan the QR code.')
            });
        });
      };

      findItemPopup = $ionicPopup.show({
        title: gettextCatalog.getString('Find Item'),
        template: '<div class="find-input-group"><button ng-click="scanQR()" type="button" ng-hide="popup.itemToFind" class="button scan-btn button-small button-icon icon ion-camera"></button><button ng-click="popup.itemToFind = \'\'" type="button" ng-show="popup.itemToFind" class="button btn-clear-search button-small button-icon ion-android-close"></button><input type="text" placeholder="{{\'Find Item...\'|translate}}" ng-model="popup.itemToFind"><div class="assertive">{{findItemPopupMsg}}</div></div>',
        scope: $scope,
        buttons: [{
          text: gettextCatalog.getString('FIND'),
          type: 'button-balanced',
          onTap: function (e) {
            e.preventDefault();
            $scope.findItem($scope.popup.itemToFind);
          }
        }, {
          text: gettextCatalog.getString('CANCEL'),
          type: 'button-default'
        }]
      });

    };

    // Find Item and scroll to it.
    $scope.findItem = function(item) {
      try {
        var obj;

        var goToObject = function(parent, obj) {
          if (angular.isDefined(obj)) {
            if (parent == $scope.parentId && obj == $scope.objId) {
              findItemPopup.close();
            }
            else {
              findItemPopup.close();
              $scope.$broadcast('objctrl.transition');
              $scope.loadedViewClass = '';
              $ionicLoading.show();
              GotoService.goTo(parent || 0, obj, $scope.tab || 'edit', 'trans');
            }
          }
        };

        var compareStr = function (actual, expected) {
          return angular.equals(actual.toString(), expected)
        }

        var searchTopLevel = function(item) {
          // Search for a top-level item by name.
          var objs = JobsFactory.job.data.ScoredObjectsList.ScoredObject;
          obj = $filter('filter')(objs, {"ObjectName": item}, compareStr)[0];
          if (angular.isDefined(obj)) {
            goToObject(obj.ParentUniqueId || 0, obj.ObjectUniqueId);
          }
          else {
            $scope.findItemPopupMsg = "Could not find " + item;
          }
        }

        var globalSearch = item.split('::');
        // Global search will search (strict) the whole job.
        // Local search will search (fuzzy) only current level.
        if (globalSearch.length > 1) {
          // Global search requires to pass a parent & child to search -
          // Level number is not limited to two.
          // e.g. PARENT::CHILD or #1801::BLK::F123

          // Go over the whole job and look for the specified item.
          var objs = JobsFactory.job.data.ScoredObjectsList.ScoredObject;
          obj = $filter('filter')(objs, {"ObjectName": globalSearch[0]}, compareStr)[0];
          if (angular.isDefined(obj)) {
            var nextObj;
            for (var i = 1; i < globalSearch.length; i++) { // Start from 1!
              if (obj.NestedObjList && obj.NestedObjList.ScoredObject && obj.NestedObjList.ScoredObject.length) {
                nextObj = $filter('filter')(obj.NestedObjList.ScoredObject, {"ObjectName": globalSearch[i]}, compareStr)[0];
                if (nextObj) {
                  if (globalSearch.length == (i + 1)) {
                    // Reached the last element in the query.
                    goToObject(nextObj.ParentUniqueId, nextObj.ObjectUniqueId);
                  }
                  else {
                    // Proceed another level.
                    obj = nextObj;
                  }
                }
                else {
                  // Go to the last item found.
                  goToObject(obj.ParentUniqueId, obj.ObjectUniqueId);
                }
              }
            }
          }
          else {
            $scope.findItemPopupMsg = "Could not find " + globalSearch[0];
          }
        }
        // Search current level items, if not found - seach top-level.
        else if (FieldObjFactory.sortedList) {
          // Item list is already in scope.
          obj = $filter('filter')(FieldObjFactory.sortedList, { title: item })[0];
          if (angular.isDefined(obj)) {
            goToObject($scope.parentId || 0, obj.id);
          }
          else {
            searchTopLevel(item);
          }
        }
        else {
          // Item list missing, create it.
          FieldObjFactory.getSortedList($scope.jobId, $scope.parentId).then(function() {
            obj = $filter('filter')(FieldObjFactory.sortedList, {title: item})[0];
            if (angular.isDefined(obj)) {
              goToObject($scope.parentId || 0, obj.id);
            }
            else {
              searchTopLevel(item);
            }
          });
        }
      } catch (e) {
        console.log(e);
      }
    };

  };

  return {
    restrict: 'E',
    controller: itemFindController,
    template: '<button ng-click="findItemPopup()" class="find-item ion-android-search"></button>'
  };
});

angular.module('ib.directives')
.directive('ibFormElement', ['$parse', '$compile', '$http', '$templateCache', function($parse, $compile, $http, $templateCache) {
  var compiledArr = {
    default: $compile($templateCache.get('templates/forms/default.html')),
    select: $compile($templateCache.get('templates/forms/select.html')),
    selectQuick: $compile($templateCache.get('templates/forms/select-quick.html')),
    checkbox: $compile($templateCache.get('templates/forms/checkbox.html')),
    checkboxQuick: $compile($templateCache.get('templates/forms/checkbox-quick.html')),
    textarea: $compile($templateCache.get('templates/forms/textarea.html')),
    images: $compile($templateCache.get('templates/forms/images.html')),
    imagesQuick: $compile($templateCache.get('templates/forms/images-quick.html'))
  };

  var formElementHandler = function(scope, element, attrs) {
    var attrElement = $parse(attrs.form);
    scope.form = attrElement(scope) || [];

    var tpl;
    if (!scope.form.type) {
      tpl = 'default';
    }
    else {
      switch(scope.form.type){
        case 'text':
        case 'date':
          tpl = 'default';
          break;
        case 'string':
          tpl = 'default';
          break;
        case 'number':
        case 'range':
        case 'tel':
          // Use 'tel' keyboard to overcome the missing '.' bug on samsung devices
          tpl = 'default';
          break;
        case 'boolean':
          tpl = 'checkbox';
          break;
        default:
          tpl = scope.form.type;
          break;
      }
    }

    if (scope.quickFormObject && tpl + 'Quick' in compiledArr) {
      tpl = tpl + 'Quick';
    }

    if (tpl in compiledArr) {
      compiledArr[tpl](scope, function(clonedElement, scope){
//        $scope.$evalAsync(
//          function() {
            element.append(clonedElement);
//          }
//        )
      });
    }


  };

  return {
    restrict: 'E',
    link: formElementHandler
  };
}]);

'use strict';
angular.module('ib.directives')
.directive('ibkeyboard', function ($parse) {

  function keyboardHandler(scope, element, attrs) {

    // Get settings from the 'keyboard' attribute.
    var keySettings = $parse(attrs.keyboard);
    scope.keyboard = keySettings(scope);

    // Click handlers.
    scope.keyPress = function(event) {
      var btn = angular.element(event.srcElement),
          action = btn.attr('data-action'),
          serial = scope.form && scope.form.serialNumber ? scope.form.serialNumber : '01',
          textarea = document.querySelector('[tabindex="' + serial + '"]');

      scope.caretPos = scope.caretPos || textarea.selectionStart;

      if (action == 'insertKey') {
        scope.form = scope.form || {};
        var insert = btn.attr('data-insert') + ' ',
            currVal = scope.form.currVal;

        // If focused update caretPos.
        if (textarea === document.activeElement) {
          scope.caretPos = textarea.selectionStart;
        }

        // If it has value update it.
        if (currVal) {
          scope.form.currVal = textarea.value.substring(0, scope.caretPos) + insert + textarea.value.substring(scope.caretPos);
        }
        else {
          scope.form.currVal = insert;
        }

        scope.caretPos = scope.caretPos + insert.length;

      }
      else if (action == 'clearText') {
        scope.form.currVal = '';
      }

      scope.$emit('form.updated', { modelValue: scope.form.currVal, form: scope.form });

    };
  }

  function keyboardTplHandler(element, attrs) {
    if (attrs.keyboard == 'settings.keyboard') {
      return 'templates/keyboard-binded.html';
    }
    else {
      return 'templates/keyboard.html';
    }
  }

  return {
    templateUrl: keyboardTplHandler,
    link: keyboardHandler
  };
});

angular.module('ib.directives')
.directive('handleLongInfoItems', function($timeout) {
  return function(scope, element, attrs) {
    $timeout(function () {
      if (element[0].scrollWidth > element[0].offsetWidth) {

        var infoPane = angular.element(element).parent().parent();

        // Removed in TK-294
        // var infoItem = angular.element(element).parent();
        // infoPane.append(infoItem.addClass('full-row'));

        // is it the info-pane or just a list item?
        if (infoPane.hasClass('pane-list')) {
          scope.calcInfoPaneHeight();
        }
      }
    });
  };
});

angular.module('ib.directives')
.directive('onTapHold', function() {

  function onTapHold(scope, element, attrs) {
    element.on('tap hold', function(e) {
      scope.$apply(attrs['onTapHold'], element);
    });
  }

  return {
    link: onTapHold
  };
});

'use strict';
angular.module('ib.directives')
.directive('paginator', function factory() {

  function getPagerMaxItems() {
    return Math.floor((window.innerWidth - (65*4)) / 42, 10);
  }

  return {
    restrict:'E',
    controller: function ($scope, Paginator) {
      Paginator.page = 0;
      $scope.paginator = Paginator;
      $scope.paginator.maxItems = getPagerMaxItems();
      if (angular.isDefined($scope.objLength)) {
        $scope.paginator.pageCount = function() {
          return Math.ceil($scope.limitedObjects.length / parseInt($scope.rowsPerPage));
        }
      }

      $scope.$watch(function() {
        return window.innerWidth;
      }, function(winInnerWidth) {
        $scope.paginator.maxItems = getPagerMaxItems();
      });
      $scope.paginator.showEllipsisStart = function(active) {
        var maxItems = $scope.paginator.maxItems,
            pageCount = $scope.paginator.pageCount();
        active++; // Zero based.
        if (pageCount > maxItems && active > Math.floor(maxItems / 2)) {
          return true;
        }
      };
      $scope.paginator.showEllipsisEnd = function(active) {
        var maxItems = $scope.paginator.maxItems,
            pageCount = $scope.paginator.pageCount();
        active++; // Zero based.
        if (pageCount > maxItems && active < pageCount - Math.floor(maxItems / 2)) {
          return true;
        }
      }
      $scope.paginator.showItem = function(current, active) {
        var show = true,
            maxItems = $scope.paginator.maxItems,
            pageCount = $scope.paginator.pageCount();

        if (!maxItems) {
          return show;
        }
        if ($scope.paginator.showEllipsisStart(active)) {
          maxItems--;
        }
        if ($scope.paginator.showEllipsisEnd(active)) {
          maxItems--;
        }

        active++; // Zero based.
        current++; // Zero based.

        if (active <= Math.floor(maxItems / 2) && current <= maxItems) {
          // active is at the begining.
          show = true;
        }
        else if (pageCount - active < Math.floor(maxItems / 2) && pageCount - current < maxItems) {
          // active is at the end.
          show = true;
        }
        else if (active - current > Math.floor(maxItems / 2)) {
          // current is too far low, hide.
          show = false;
        }
        else if (active - current <= maxItems / -2) {
          // current is too far high, hide.
          show = false;
        }
        return show;
      };
    },
    templateUrl: 'templates/paginationControl.html'
  };
});

angular.module('ib.directives')
.directive('quickEditMenu', function() {

  var controller = function($scope) {
  };

  return {
    restrict: 'E',
    controller: controller,
    templateUrl: 'templates/quick-edit-menu.html'
  };
});

'use strict';
angular.module('ib.directives')
.directive('swipeable', function($document) {
  return {
    restrict: 'A',
    link: function(scope, element) {

      var startX, x, animationInterval;

      // Bind to 'dragstart'.
      element.on('dragstart', drag);
      scope.$on('$destroy', function() {
        element.off('dragstart', drag);
      });

      function drag(event) {
        // Prevent default dragging if scrolling down.
        if (event.gesture.velocityX < event.gesture.velocityY) {
          return;
        }
        // Prevent on related mode.
        if (scope.relatedMode) {
          return;
        }
        event.stopPropagation();
        event.gesture.preventDefault();
        // Init.
        x = 0;
        // Set starting point.
        startX = event.gesture.center.pageX - x;
        // Move while dragging.
        $document.on('drag', function(event) {
          x = event.gesture.center.pageX - startX;
          // 5px threshold
          if (Math.abs(x) > 10) {
            move(x);
          }
          else if (x != 0) {
            move(0);
          }
        });
        $document.on('dragend', release);
      };

      // Move item using css transform.
      function move(x) {
        element.css({
          '-webkit-transform':  'translate3d(' + x + 'px, 0, 0)',
          'transform':  'translate3d(' + x + 'px, 0, 0)',
          'opacity': Math.max(1 - Math.abs(x/200), 0),
        });
      }

      // Animate move().
      function animate(n) {
        animationInterval = setInterval(function() {
          x = x + n;
          if (10 > x && x > -10) {
            move(0);
            clearInterval(animationInterval);
            return false;
          }
          else {
            move(x);
          }
        }, 1);
      }

      // Release handler
      function release() {
        // ToDo: CHECK WHY THESE DON'T WORK
        // $document.off('drag', move);
        // $document.off('dragend', release);
        // THESE WORK BUT REMOVE ALL OF THE BINDINGS OF $document FOR 'drag' and 'dragend'
        $document.off('drag');
        $document.off('dragend');

        if (x > 100 && !scope.disabledPrev) {
          // Skip to prev.
          goTo('prev');
        }
        else if (x < -100 && !scope.disabledNext) {
          // Skip to next.
          goTo('next');
        }
        else {
          // If not passed threshold animate it back.
          var diff = x > 0 ? -2 : 2;
          // Limit x so we don't wait for it too much.
          x = Math.min(Math.max(parseInt(x), -100), 100);
          animate(diff);
        }
      }

      // Navigate handler
      function goTo(dir) {
        scope.goTo(dir);
        scope.$apply();
      }

      scope.$on('objctrl.transition', function() {
        element.removeAttr('style');
      });

    }
  }
});
