angular.module('ib.services', ['http-auth-interceptor'])

angular.module('ib.services')
.factory('AuthenticationService', function($rootScope, $http, authService, localStorageService, $log, DbService) {
  return {
    login: function(user) {
      if (user.username && user.password) {
        $http.post($rootScope.endPoint + '/user/login.json', { username: user.username, password: user.password }, { ignoreAuthModule: true, timeout: 300000, warningAfter: 5000})
          .success(function (data, status, headers, config) {
            if ('user' in data && 'field_client_nid_ref' in data.user) {
              localStorageService.set('clientId', data.user.field_client_nid_ref);
              DbService.insertConfValue("clientId", data.user.field_client_nid_ref);
              $rootScope.clientId = data.user.field_client_nid_ref;
            }
            $http.defaults.headers.cookie = data.session_name + '=' + data.sessid;
            $http.post($rootScope.tokenEndPoint , null, { ignoreAuthModule: true, timeout: 300000, warningAfter: 5000 })
              .success(function (data, status, headers, config) {
                localStorageService.set('token', data);
                localStorageService.set('username', user.username);
                localStorageService.set('keystring', user.password);
                DbService.insertConfValue("keystring", user.password);
                DbService.insertConfValue("username", user.username);

                $log.overrideExtraDef({'_userClientId' : $rootScope.clientId, '_userName' : user.username});
                $http.defaults.headers.common['X-CSRF-Token'] = data;
                $http.defaults.headers.put['X-CSRF-Token'] = data;
                // Need to inform the http-auth-interceptor that
                // the user has logged in successfully.  To do this, we pass in a function that
                // will configure the request headers with the authorization token so
                // previously failed requests(aka with status == 401) will be resent with the
                // authorization token placed in the header
                authService.loginConfirmed(data, function(config) {
                  return config;
                });
              })
          })
          .error(function (data, status, headers, config) {
            $rootScope.$broadcast('event:auth-login-failed', status);
          });
      }
      else {

        $rootScope.$broadcast('event:auth-login-failed', 401);
      }
    },
    logout: function() {
      $http.post($rootScope.endPoint + '/user/logout.json', null, {timeout: 300000, warningAfter: 5000}).success(function (data, status, headers, config) {
        $log.overrideExtraDef({'_userClientId' : 0, '_userName' : 'anonymous'});
        localStorageService.remove('token');
        localStorageService.remove('keystring');
        $rootScope.$broadcast('event:auth-logout-complete');
      });
    },
    loginCancelled: function() {
      authService.loginCancelled();
    }
  };
})

angular.module('ib.services')
.service('DbService', function($rootScope, $window, $q, $http, HelperService, localStorageService, $log, $cordovaFile, $timeout) {
  var DB_NAME = "ib";
  var DB_FORM_TYPE_INPUT = 1;
  var DB_FORM_TYPE_DESC = 2;
  var _db;
  this.run = function(needUpdate) {
    return createDb(needUpdate);
  }
  var createDb = function(needUpdate) {
    var deferred = $q.defer();
    try {
      $window.sqlitePlugin.openDatabase({ name: DB_NAME, location: 'default' }, function (db) {
        _db = db;
        deferred.resolve();
        createTables(needUpdate, deferred);
      }, function (err) {
        _db = $window.openDatabase(DB_NAME, '1.0', 'IB', 10 * 3072 * 1024);
        createTables(needUpdate, deferred);
      });
    } catch (e) {
      _db = $window.openDatabase(DB_NAME, '1.0', 'IB', 10 * 3072 * 1024);
      createTables(needUpdate, deferred);
    }

    return deferred.promise;
  }
  this.clearJobsAsync = function() {
    var deferred = $q.defer();
    localStorageService.clearAll("jobData");
    _db.transaction(function(tx) {
      tx.executeSql("drop table jobs_values", []);
      tx.executeSql("drop table jobs_images", []);
      tx.executeSql("drop table jobs", []);
      tx.executeSql("drop table jobs_fields", []);
      tx.executeSql("drop table jobs_fields_childes", []);
      tx.executeSql("drop table jobs_forms", []);
      tx.executeSql("drop table jobs_config", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_config(ckey text primary key, cval text)", []);
      tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_conf_index on jobs_config(ckey)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS ib_log(id integer primary key, logMsg text)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_forms(id integer, odefId integer, type int, form text)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs(id integer primary key, jobDate integer, data text, jobName text, jobId text, jobProperties text, status integer, clientId integer, userId integer, isSync integer)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_fields(odefId integer, fid integer primary key, id integer, pfid integer, level integer, status integer, isNew integer, syncStatus integer, serverId integer, userFid text, pServerId integer)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_fields_childes(fid integer, id integer, objs text)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_values(odefId integer, pfid integer, userFid text, id integer, fid integer, pd integer, inputDate integer, inputValue text, valueType integer)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_images(odefId integer, pfid integer, userFid text, id integer, fid integer, pd integer, inputDate integer, local_url text primary key, s3_url text, status integer, valueType integer)", []);
      tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_index on jobs_values(id, fid, pd)", []);

      tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_index_forms on jobs_forms(id, odefId, type)", []);
      tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_index_childs on jobs_fields_childes(id, fid)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_images_nid_index on jobs_images(id)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_images_fid_index on jobs_images(fid)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_mul_index on jobs_fields(fid, id, pfid)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_fid_index on jobs_fields(fid)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_nid_index on jobs_fields(id)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_childs_nid_index on jobs_fields_childes(id)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_forms_nid_index on jobs_forms(id)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_forms_nid_type_index on jobs_forms(id, type)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_childs_fid_index on jobs_fields_childes(fid)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_status_index on jobs_fields(status)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_values_fid_index on jobs_values(fid)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_status_index on jobs(status)", []);
      deferred.resolve();
    });
    return deferred.promise;
  }



  this.clearJobs = function() {
    localStorageService.clearAll("jobData");
    _db.transaction(function(tx) {
      tx.executeSql("drop table jobs_values", []);
      tx.executeSql("drop table jobs_images", []);
      tx.executeSql("drop table jobs", []);
      tx.executeSql("drop table jobs_fields", []);
      tx.executeSql("drop table jobs_fields_childes", []);
      tx.executeSql("drop table jobs_forms", []);

      createTables();
    });
  }

  var upload = function(imageURI, jobId) {
    var deferred = $q.defer();

    try {
      var fileName = imageURI.split("/").pop(),
          ft = new FileTransfer(),
          options = new FileUploadOptions();

      options.fileKey = "file";
      options.fileName = fileName;
      options.mimeType = "image/jpeg";
      options.chunkedMode = false;
      $http.get($rootScope.endPoint + '/appjob/' + fileName + '/' + jobId + '/' + $rootScope.clientId + '.json', {timeout: 300000, warningAfter: 5000})
        .success(function (data, status, headers, config) {
          if (data.resStatus) {
            options.params = {
              "key": $rootScope.clientId + "/" + jobId + "/" + fileName,
              "AWSAccessKeyId": data.data.awsKey,
              "acl": "public-read",
              "policy": data.data.policy,
              "signature": data.data.signature,
              "Content-Type": "image/jpeg"
            };
            var uploadUrl;

            uploadUrl = "https://" + data.data.bucket + ".s3.amazonaws.com/";
            $log.info("uploading image : " + uploadUrl);
            ft.upload(imageURI, uploadUrl,
              function (e) {
                $log.info("uploading image : success" + uploadUrl);
                updateJobImage(imageURI, jobId + "/" + fileName);
                deferred.resolve(true);
              },
              function (e) {
                try {
                  $log.info("uploading image : fail " + uploadUrl);
                  var longMessage;
                  longMessage = JSON.stringify(e);
                  $log.log("s3upload", {'_exception': e.exception, '_uploadUrl': uploadUrl, '_logGroup': 's3upload'}, longMessage);
                } catch (e) {

                }
                if (e.code == 1 || e.code == 2) {
                  updateJobImageStatus(2);
                }
                deferred.resolve(false);
              }, options
            );

          }
        })
        .error(function (data, status, headers, config) {
          deferred.resolve(false);
        });
    } catch (e) {
      $log.error(e);
      deferred.resolve(false);
    }

    return deferred.promise;
  }


  var createTables = function(needUpdate, deferred) {

      _db.transaction(function(tx) {
        if (needUpdate) {
          tx.executeSql("drop table jobs_values", []);
          tx.executeSql("drop table jobs_images", []);
          tx.executeSql("drop table jobs", []);
          tx.executeSql("drop table jobs_fields", []);
          tx.executeSql("drop table jobs_fields_childes", []);
          tx.executeSql("drop table jobs_forms", []);
        }
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_config(ckey text primary key, cval text)", []);
        tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_conf_index on jobs_config(ckey)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS ib_log(id integer primary key, logMsg text)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_forms(id integer, odefId integer, type int, form text)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs(id integer primary key, jobDate integer, data text, jobName text, jobId text, jobProperties text, status integer, clientId integer, userId integer, isSync integer)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_fields(odefId integer, fid integer primary key, id integer, pfid integer, level integer, status integer, isNew integer, syncStatus integer, serverId integer, userFid text, pServerId integer)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_fields_childes(fid integer, id integer, objs text)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_values(odefId integer, pfid integer, userFid text, id integer, fid integer, pd integer, inputDate integer, inputValue text, valueType integer)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_images(odefId integer, pfid integer, userFid text, id integer, fid integer, pd integer, inputDate integer, local_url text primary key, s3_url text, status integer, valueType integer)", []);
        tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_index on jobs_values(id, fid, pd)", []);
        tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_index_forms on jobs_forms(id, odefId, type)", []);
        tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_index_childs on jobs_fields_childes(id, fid)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_images_nid_index on jobs_images(id)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_images_fid_index on jobs_images(fid)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_mul_index on jobs_fields(fid, id, pfid)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_fid_index on jobs_fields(fid)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_nid_index on jobs_fields(id)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_childs_nid_index on jobs_fields_childes(id)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_forms_nid_index on jobs_forms(id)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_forms_nid_type_index on jobs_forms(id, type)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_childs_fid_index on jobs_fields_childes(fid)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_status_index on jobs_fields(status)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_values_fid_index on jobs_values(fid)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_status_index on jobs(status)", []);
        deferred.resolve();
      })
  }

  this.getJobValues = function(id) {
    var deferred = $q.defer();
    executeSelect("SELECT jv.*, jf.isNew as isNewField, jf.serverId FROM jobs_values as jv JOIN jobs_fields as jf on jv.fid = jf.fid where jv.id = ?", [id], 0, 0, 0).then(function(results){
      deferred.resolve(results);
    });
    return deferred.promise;
  }
  this.getNeedSyncFields = function(id) {
    var deferred = $q.defer();
//    _ib_insert_into_ib_field_objects($row['userFid'], $row['odefId'], $nid, $row['level'], $row['pfid'], '')) {
    executeSelect("SELECT fid, userFid, isNew, odefId, level, serverId, pfid, pServerId FROM jobs_fields where id = ? and syncStatus = 0 order by level", [id], 0, 0, 0).then(function(results){
      deferred.resolve(results);
    });
    return deferred.promise;
  }
  var updateFieldsSyncStatus = function(fields) {
    var deferred = $q.defer();
    var query = "UPDATE jobs_fields set syncStatus = 1 where fid = ?";
    _db.transaction(function(tx) {
      fields.forEach(function (args) {
        tx.executeSql(query, [args.fid]);
      });
      deferred.resolve();
    });
    return deferred.promise;
  }
  this.getNeedValuesFields = function(id) {
    var deferred = $q.defer();
    simpleExecuteSelect("SELECT fid, status FROM jobs_fields where id = ? and status != 2", [id]).then(function(results){
      deferred.resolve(results);
    });
    return deferred.promise;
  }
  this.selectJobsData = function() {
    return simpleExecuteSelect("SELECT data, id FROM jobs");
  }

  this.getFieldsStatus = function(objId) {
    if (objId == null) {
      objId = 0;
    }
    var deferred = $q.defer();
    simpleExecuteSelect("SELECT fid, status FROM jobs_fields where pfid = ?", [objId]).then(function(results){
      var objs = {};
      var len = results.rows.length, i;
      for (i = 0; i < len; i++){
        objs[results.rows.item(i).fid] = results.rows.item(i).status;
      }
      deferred.resolve(objs);
    });
    return deferred.promise;

  }

  this.getFieldStatus = function(objId) {
    var deferred = $q.defer();
    simpleExecuteSelect("SELECT status FROM jobs_fields where fid = ?", [objId]).then(function(results){
      var objs = {};
      var len = results.rows.length, i;

      if (len > 0){
        deferred.resolve(results.rows.item(0).status);
      }
      else deferred.resolve(0);
    });
    return deferred.promise;

  }


  this.getJobSyncedImages = function(nid) {
    var deferred = $q.defer();
    executeSelect("SELECT jv.*, jf.isNew as isNewField, jf.serverId FROM jobs_images as jv JOIN jobs_fields as jf on jv.fid = jf.fid  where jv.id = ? and jv.status = 1", [nid], 0, 0, 0).then(function(results){
      deferred.resolve(results);
    });
    return deferred.promise;
  }

  this.getJobOfflineImages = function(nid) {
    var deferred = $q.defer();
    executeSelect("SELECT jv.*, jf.isNew as isNewField, jf.serverId FROM jobs_images as jv JOIN jobs_fields as jf on jv.fid = jf.fid  where jv.id = ?", [nid], 0, 0, 0).then(function(results){
      deferred.resolve(results);
    });
    return deferred.promise;
  }

  this.getJobImages = function(nid, objId) {
    var deferred = $q.defer();
    var items = [];

    // DEBUG
    // items = [
    //   { imgUrl: 'http://lorempixel.com/600/600/nature/?v=1' },
    //   { imgUrl: 'http://lorempixel.com/600/600/nature/?v=2' },
    //   { imgUrl: 'http://lorempixel.com/600/600/nature/?v=3' },
    //   { imgUrl: 'http://lorempixel.com/600/600/nature/?v=4' },
    //   { imgUrl: 'http://lorempixel.com/600/600/nature/?v=5' }
    // ];
    // deferred.resolve(items);

    simpleExecuteSelect("SELECT local_url FROM jobs_images where id = ? and fid = ?", [nid, objId]).then(function(results){
      var len = results.rows.length, i;
      for (i = 0; i < len; i++){
        results.rows.item(i);
        items.push({imgUrl: results.rows.item(i).local_url});
      }
      deferred.resolve(items);
    });
    return deferred.promise;
  }

  this.getFieldObjectValues = function(objId) {
    var deferred = $q.defer();
    var values = {};
    simpleExecuteSelect("SELECT pd, inputValue FROM jobs_values where fid = ? and inputValue != ''", [objId]).then(function(results){
      var len = results.rows.length, i;
      for (i = 0; i < len; i++){
        results.rows.item(i);
        values[results.rows.item(i).pd] = results.rows.item(i).inputValue;
      }
      deferred.resolve(values);
    });
    return deferred.promise;
  }

  this.isJobsHasImages = function() {
    var deferred = $q.defer();
    var query = "SELECT local_url as id from jobs_images where status = 0";
    var args = [];
    var imageCount = 0;
    executeSelect(query, args, 1, 0, 0).then(function(results){
      imageCount = results.rows.length;
      if (results.rows.length > 0) {
        deferred.resolve(true);
      }
      else {
        deferred.resolve(false);
      }
    });
    return deferred.promise;
  }

  this.sendJobFieldsToServer = function(jobId) {
    var deferred = $q.defer();
    this.getNeedSyncFields(jobId).then(function(jobFields) {
      if (jobFields.rows.length > 0 ) {
        $http.put($rootScope.endPoint + '/appjob/' + jobId + '.json?mode=fields' , {rows:jobFields.rows}, {timeout: 300000, warningAfter: 5000})
          .success(function (data, status, headers, config) {
            var rows = data.data;
            updateJobFieldsServerId(rows).then(function() {
              updateFieldsSyncStatus(rows).then(function() {
                deferred.resolve(true);
              })
            })
          })
          .error(function (data, status, headers, config) {
            $log.warn("error in sendJobFieldsToServer data = '" + data + "' : status = '" + status + "'");
            deferred.resolve(false);
          })
      }
      else {
        deferred.resolve(true);
      }
    });
    return deferred.promise;
  }


  this.processJobImages = function(jobId) {

    var deferred = $q.defer();
    var query = "SELECT local_url as id from jobs_images where id = ? and status = 0";
    var args = [jobId];
    var prom = [];
    var imageCount = 0;
    executeSelect(query, args, 1, 0, 0).then(function(results){
      imageCount = results.rows.length;
      if (results.rows.length > 0) {

        results.rows.forEach(function (id) {
          prom.push(upload(id, jobId));
        });
      }
      else {
        deferred.resolve({status:1, imageCount:imageCount});
      }

      $q.all(prom).then(function () {
        executeSelect(query, args, 1, 0, 0).then(function(results){
          imageCount = results.rows.length;
          if (imageCount > 0) {
            deferred.resolve({status:0, imageCount:imageCount});
          }
          else {
            deferred.resolve({status:1, imageCount:imageCount});
          }
        });
      });
    });


    return deferred.promise;
  }

  var updateJobImageStatus = function(local_url, status) {
    var query = "UPDATE jobs_images set status = ?, s3_url = ? where local_url = ?";
    var args = [status, local_url];
    _db.transaction(function(tx) {
      tx.executeSql(query,args);
    });
  }

  var updateJobImage = function(local_url, s3_url) {
    var query = "UPDATE jobs_images set status = 1, s3_url = ? where local_url = ?";
    var args = [s3_url, local_url];
    _db.transaction(function(tx) {
      tx.executeSql(query,args);
    });
  }


  this.addIsNewField = function() {
    var deferred = $q.defer();
    _db.transaction(function(tx) {
      tx.executeSql("ALTER TABLE jobs_fields ADD COLUMN isNew integer",[], dbSuccess, dbError);
      tx.executeSql("ALTER TABLE jobs_fields ADD COLUMN syncStatus integer",[], dbSuccess, dbError);
      tx.executeSql("ALTER TABLE jobs_fields ADD COLUMN serverId integer",[], dbSuccess, dbError);
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in addIsNewField');
    }
    return deferred.promise;
  }
//  this.
  this.insertJobFields = function(fields, isNew) {
    //isNew logic
    //0 from server
    //1 new
    //2 deleted
    var deferred = $q.defer();
    var syncStatus = 1;
    if (isNew == 1) {
      syncStatus = 0;
    }
    var query = "INSERT INTO jobs_fields (odefId, fid , id, pfid, level, status, userFid, isNew, syncStatus) VALUES (?,?,?,?,?,0,?,?,?)";
    _db.transaction(function(tx) {
      fields.forEach(function (args) {
        args.push(isNew);
        args.push(syncStatus);
        tx.executeSql(query,args, dbSuccess, dbError);
        if (isNew == 1) {
          tx.executeSql("UPDATE jobs_fields set pServerId = (SELECT pjf.ServerId FROM jobs_fields pjf WHERE jobs_fields.pfid = pjf.fid) where jobs_fields.fid = ? and jobs_fields.id = ?",[args[1], args[2]], dbSuccess, dbError);
        }
      });
      deferred.resolve();
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in insertJobFields');
    }
    return deferred.promise;
  }


  this.getJobFormByType = function(jobId, odefId, type) {
    var deferred = $q.defer();
    simpleExecuteSelect("select form from jobs_forms where id = ? and odefId = ? and type = ?", [jobId, odefId, type]).then(function(results){
      var form = false;
      if (results.rows.length > 0) {
        form = JSON.parse(results.rows.item(0).form);
      }
      deferred.resolve(form);
    });
    return deferred.promise;
  }


  this.insertJobForms = function(forms) {
    var query = "INSERT INTO jobs_forms (id, odefId, form, type) VALUES (?,?,?,?)";
    _db.transaction(function(tx) {
      forms.forEach(function (args) {
        tx.executeSql(query,args, dbSuccess, dbError);
      });
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in insertJobForm');
    }
  }

  this.getJobFieldChildes = function(jobId, fid) {
    var deferred = $q.defer();
    var selectFid;
    if (fid == null) {
      selectFid = 0;
    }
    else {
      selectFid = fid;
    }
    simpleExecuteSelect("select objs from jobs_fields_childes where id = ? and fid = ?", [jobId, selectFid]).then(function(results){
      var objs = [];
      if (results.rows.length > 0) {
        objs = JSON.parse(results.rows.item(0).objs);
        objs = objs.filter(function (obj) { return obj.deleted !== true; })
      }
      deferred.resolve(objs);
    });
    return deferred.promise;
  }

  this.updateJobFieldsChildes = function(id, fid, objs) {
    var query = "REPLACE INTO jobs_fields_childes (objs ,fid ,id) VALUES (?,?,?)";
    _db.transaction(function(tx) {
      tx.executeSql(query,[JSON.stringify(objs), fid, id], dbSuccess, dbError);
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in updateJobFieldsChildes');
    }
  }

  this.insertJobFieldsChildes = function(fields) {
    var query = "REPLACE INTO jobs_fields_childes (fid, id, objs) VALUES (?,?,?)";
    _db.transaction(function(tx) {
      fields.forEach(function (args) {
        tx.executeSql(query,args, dbSuccess, dbError);
      });
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in insertJobFieldsChildes');
    }
  }

  this.updateJobField = function(fid, status) {
    var query = "UPDATE jobs_fields set status = ? where fid = ?";
    _db.transaction(function(tx) {
      tx.executeSql(query,[status, fid]);
    });
  }

  var updateJobFieldsServerId = function(fields) {
    var deferred = $q.defer();
    var query = "UPDATE jobs_fields set serverId = ? where fid = ?";
    var pQuery = "UPDATE jobs_fields set pServerId = ? where pfid = ?";
    if (fields.length > 0) {
      _db.transaction(function(tx) {
        fields.forEach(function (args) {
          if (args.fid != "0") {
            tx.executeSql(query, [args.serverId, args.fid]);
            tx.executeSql(pQuery, [args.serverId, args.fid]);
          }
        });
        deferred.resolve();
      });
    }
    else {
      deferred.resolve();
    }
    return deferred.promise;
  }


  this.updateJobFieldIsNew = function(fid, status) {
    var query = "UPDATE jobs_fields set isNew = ?, syncStatus = 0 where fid = ?";
    _db.transaction(function(tx) {
      tx.executeSql(query,[status, fid]);
    });
  }

//  var removefile = function(file){
//    fileSystem.root.getFile(file, {create: false, exclusive: false}, gotRemoveFileEntry, fail);
//  }
//
//  var gotRemoveFileEntry = function(fileEntry){
//    fileEntry.remove(fileRemoveSuccess, fileRemoveFail);
//  }
//
//  var fileRemoveSuccess = function(entry) {
//    console.log("Removal succeeded");
//  }

//  var fileRemoveFail = function(error) {
//    console.log("Error removing file: " + error.code);
//  }
  this.deleteFormsCache = function() {
    var deferred = $q.defer();
    _db.transaction(function(tx) {
      var query;
      query = "DELETE FROM jobs_forms";
      tx.executeSql(query, dbSuccess, dbError);
      deferred.resolve(true);
      var dbSuccess = function(tx, results) {

      }
      var dbError = function(e) {
        $log.log('db error in deleteFormsCache');
      }

    })
    return deferred.promise;
  }
  this.deleteJobData = function(jobs) {
    var deferred = $q.defer();
    var query;

    _db.transaction(function(tx) {
      jobs.forEach(function (job) {
        if ($rootScope.mode == "mobile") {
          $cordovaFile.removeFile(cordova.file.dataDirectory, job + ".json");
        }
        query = "DELETE FROM jobs where id = ?";
        tx.executeSql(query, [job], dbSuccess, dbError);
        query = "DELETE FROM jobs_values where id = ?";
        tx.executeSql(query, [job], dbSuccess, dbError);
        query = "DELETE FROM jobs_fields where id = ?";
        tx.executeSql(query, [job], dbSuccess, dbError);
        query = "DELETE FROM jobs_images where id = ?";
        tx.executeSql(query, [job], dbSuccess, dbError);
        query = "DELETE FROM jobs_forms where id = ?";
        tx.executeSql(query, [job], dbSuccess, dbError);

      });
      deferred.resolve(true);
    });
    var dbSuccess = function(tx, results) {

    }
    var dbError = function(e) {
      $log.log('db error in deleteJobData');
    }
    return deferred.promise;
  }

  this.deleteJobImages = function(files) {
    var deferred = $q.defer();
    var query;
    query = "DELETE FROM jobs_images where local_url = ?";
    _db.transaction(function(tx) {
      files.forEach(function (file) {
        tx.executeSql(query, [file.imgUrl], dbSuccess, dbError);
      });
      deferred.resolve(true);
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in deleteJobImages');
    }
    return deferred.promise;
  }

  this.insertJobImage = function(id, fid, pd, inputDate, local_url, obj) {
    var isNew, pfid, userFid, odefId;
    if ('isNewItem' in obj) {
      pfid = obj.ParentUniqueId;
      userFid = obj.ObjectName;
      odefId = obj.ScObjDefId;
    }
    else {
      pfid = 0;
      userFid = '';
      odefId = 0;
    }

    var query;
    var args;
    args = [id, fid, pd, inputDate, local_url, pfid, userFid, odefId];

    query = "INSERT INTO jobs_images (id, fid, pd, inputDate, local_url, status, valueType, pfid, userFid, odefId) VALUES (?,?,?,?,?,0,2,?,?,?)";
    _db.transaction(function(tx) {
      tx.executeSql(query,args, dbSuccess, dbError);
      query = "update jobs set isSync = 0 where id = ?";
      args = [id];
      tx.executeSql(query,args, dbSuccess, dbError);
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in insertJobImage');
    }


  }



  this.updateJobSyncStatus = function(id, status) {
    var deferred = $q.defer();
    var query;
    var args;
    query = "update jobs set isSync = ? where id = ?";
    args = [status, id];
    _db.transaction(function(tx) {
      tx.executeSql(query,args, dbSuccess, dbError);
    });
    var dbSuccess = function(tx, results) {
      deferred.resolve(true);
    }
    var dbError = function(e) {
      $log.log('db error in updateJobSyncStatus');
      deferred.resolve(false);
    }
    return deferred.promise;
  }


  this.insertConfValue = function(ckey, cval) {
    var query = "REPLACE INTO jobs_config (ckey, cval) VALUES (?,?)";
    _db.transaction(function(tx) {
      tx.executeSql(query, [ckey, cval]);
    });
  }
  this.insertJobValue = function(id, fid, pd, inputDate, inputValue, pfid, userFid, odefId) {
    var deferred = $q.defer();
    var query;
    var args;
    args = [id, fid, pd, inputDate, inputValue, pfid, userFid, odefId];
    query = "REPLACE INTO jobs_values (id, fid, pd, inputDate, inputValue, valueType, pfid, userFid, odefId) VALUES (?,?,?,?,?,1,?,?,?)";
    _db.transaction(function(tx) {
      tx.executeSql(query,args, dbSuccess, dbError);
      query = "update jobs set isSync = 0 where id = ?";
      args = [id];
      tx.executeSql(query,args, dbSuccess, dbError);

    });
    var dbSuccess = function(tx, results) {
      deferred.resolve(true);
    }
    var dbError = function(e) {
      $log.log('db error in insertJobValue');
      deferred.resolve(false);
    }
    return deferred.promise;
  }



  this.insertJob = function(id, jobDate, data, jobName, jobId, jobProperties, status) {
    var query = "INSERT INTO jobs (id, jobDate, data, jobName, jobId, jobProperties, status, isSync) VALUES (?,?,?,?,?,?,?,1)";
    var args = [];
    args.push(id);
    args.push(jobDate);
    args.push(data);
    args.push(jobName);
    args.push(jobId);
    args.push(jobProperties);
    args.push(status);
    _db.transaction(function(tx) {
      tx.executeSql(query,args);
    });
  }


  var simpleExecuteSelect = function(query, args) {
    var deferred = $q.defer();
    _db.transaction(function (tx) {
      tx.executeSql(query, args, dbSuccess, dbError);
    });
    var dbSuccess = function(tx, results) {
      deferred.resolve(results);
    }
    var dbError = function(e) {
      $log.log('db error in simpleExecuteSelect');
      deferred.resolve(false);
    }
    return deferred.promise;
  }

  var executeSelect = function(query, args, returnIds, parseJson, returnRows) {
    var deferred = $q.defer();
    var items = [];
    var itemsO = {};
    _db.transaction(function (tx) {
      tx.executeSql(query, args, dbSuccess, dbError);
    });
    var dbSuccess = function(tx, results) {
      if (returnRows) {
        deferred.resolve(results);
      }
      var len = results.rows.length, i;
      for (i = 0; i < len; i++){
        if (returnIds == 1) {
          items.push(results.rows.item(i).id);
        }
        else {
          if (parseJson == 1) {
            var tempItem = results.rows.item(i);
            var statusClass = HelperService.getStatusClass(tempItem.status);
            var o = {id: tempItem.id, jobDate: tempItem.jobDate, jobName: tempItem.jobName, jobProperties: JSON.parse(tempItem.jobProperties), data: JSON.parse(tempItem.data), status: tempItem.status, statusClass:statusClass};
            itemsO[tempItem.id] = o;
          }
          else {
            items.push(results.rows.item(i));
          }
        }
      }
      if (parseJson == 1) {
        deferred.resolve({rows:itemsO});
      }
      else {
        deferred.resolve({rows:items});
      }
    }
    var dbError = function(e) {
      $log.log('db error in executeSelect');
      deferred.resolve([]);
    }
    return deferred.promise;

  }

  this.updateJobStatus = function(jobId, status) {
    var query = "update jobs set status = ? where id = ?";

    _db.transaction(function(tx) {
      tx.executeSql(query, [status, jobId]);
    });
  }

  this.updateJobData = function(jobId, data) {
    var deferred = $q.defer();

    if ($rootScope.mode == "mobile") {
      $cordovaFile.writeFile(cordova.file.dataDirectory, jobId + ".json", data, true).then(function(result) {
        deferred.resolve(true);
        // Success!
      }, function(err) {
        $log.error(err);
        deferred.resolve(false);
        // An error occured. Show a message to the user
      });
    }
    else {

      var query = "update jobs set data = ? where id = ?";

      _db.transaction(function(tx) {
        tx.executeSql(query, [data, jobId], dbSuccess, dbError);
      });
      var dbSuccess = function(tx, results) {
        deferred.resolve(results);
      }
      var dbError = function(e) {
        $log.log('db error in updateJobData');
        deferred.resolve();
      }

    }

    return deferred.promise;
  }

  this.selectJobs = function() {
    return simpleExecuteSelect("SELECT id ,jobDate, jobName, jobId, jobProperties, status FROM jobs order by jobDate desc", []);
  }

  this.selectJobData = function(id) {
    return simpleExecuteSelect("SELECT status, data FROM jobs where id = ?", [id]);
  }

  this.getJobsIdList = function() {
    return executeSelect("SELECT id FROM jobs", [], 1, 0, 0);
  }

  this.resendJobs = function() {

    var query = "UPDATE jobs set isSync = ?";
    var args = [0];
    _db.transaction(function(tx) {
      tx.executeSql(query,args);
    });
  }
  this.getNeedSyncJobsIdList = function() {
    return executeSelect("SELECT id FROM jobs where isSync = 0", [], 1, 0, 0);
  }

  this.getDoneJobsIdList = function() {
    return executeSelect("SELECT id FROM jobs where status = 3 and isSync = 1", [], 1, 0, 0);
  }

  // App Settings
  this.getSettings = function() {

    var defaultSettings = {
      keyboard: [
        {name: '-', value: 'bad'},
        {name: '--', value: 'very bad'},
        {name: '+', value: 'good'},
        {name: '++', value: 'very good'},
        {name: 'LOD', value: 'lodging'},
        {name: 'EM', value: 'emergence'},
        {name: 'HS', value: 'headsmut'}
      ],
      sync:{
        autoSyncInterval: {name: 'Auto synchronize interval', value: 0},
        imageUploadNotify: {name: 'Upload images only on WiFi', value: true}
      }
    };
    // var defaultSettings = {};

    var deferred = $q.defer();

    deferred.resolve(localStorageService.get('settings') || defaultSettings);
    return deferred.promise;
  }

  this.storeSettings = function(settings) {
    localStorageService.set('settings', settings || {});
    return settings;
  }

  // Last sync date
  this.getLastSync = function() {
    var lastSync = localStorageService.get('lastSync') || false;
    return lastSync;
  }

  this.setLastSync = function(now) {
    now = now || Date.now();
    localStorageService.set('lastSync', now);
    return now;
  }

})

'use strict';
angular.module('ib.services')
  .factory('FieldObjFactory', function ($q, $filter, $rootScope, DbService, HelperService, localStorageService, $log, gettextCatalog, $cordovaFileTransfer, $location, $cordovaFile) {

    var JOB_STATUS_NEW = 0,
      JOB_STATUS_IN_PROG = 1,
      JOB_STATUS_COMPLETE = 2,
      JOB_STATUS_DONE = 3,
      JOB_STATUS_SYNC = 4,
      FIELD_STATUS_PEND = 0,
      FIELD_STATUS_IN_PROG = 1,
      FIELD_STATUS_COMP = 2;
    var service = {};
    var DB_FORM_TYPE_INPUT = 1;
    var DB_FORM_TYPE_DESC = 2;

    // Generic set function
    service.set = function (name, value) {
      service[name] = value;
    };

    var getObjectChildList = function (data, objId) {
      var obj = findObject(data.ScoredObjectsList.ScoredObject, objId);
      if (obj && 'NestedObjList' in obj && 'ScoredObject' in obj.NestedObjList) {
        return [].concat(obj.NestedObjList.ScoredObject);
      }
      return false;
    };

    var getJobRootObjects = function (data) {
      return [].concat(data.ScoredObjectsList.ScoredObject);
    };

    var checkJob = function (id) {
      DbService.getNeedValuesFields(id).then(function (results) {
        if (results.rows.length === 0) {
          DbService.updateJobStatus(id, JOB_STATUS_COMPLETE);
          $rootScope.$broadcast('jobs.jobUpdate', {id: id, status: JOB_STATUS_COMPLETE});
        }
        else {
          DbService.updateJobStatus(id, JOB_STATUS_IN_PROG);
          $rootScope.$broadcast('jobs.jobUpdate', {id: id, status: JOB_STATUS_IN_PROG});
        }
      });
    };

    var findObject = function (objects, objId) {
      for (var i = 0; i < objects.length; i++) {
        if (objects[i].ObjectUniqueId == objId) {
          return objects[i];
        }
        var found;
        if ('NestedObjList' in objects[i]) {
          if (!Array.isArray(objects[i].NestedObjList)) {
            found = findObject([].concat(objects[i].NestedObjList.ScoredObject), objId);
          }
        }
        if (found) {
          return found;
        }
      }
      return false;
    };

    var parseHistoryList = function (pd, prop) {
      var historyList = prop.ValuesList.Value;
      switch (parseInt(pd.fieldType)) {
        case 3: // Select list
        case 5: // Checkbox
          historyList.forEach(function (hItem) {
            hItem.Value = processPropValue(pd, hItem.Value);
          });
          break;
      }
      return historyList;
    };

    var processPropValue = function (pd, val) {

      if (val === '') {
        return val;
      }

      switch (parseInt(pd.fieldType)) {
        case 3: // Select list
        case 4: // Select list
          var option;
          if (pd.selectionValueList) {
            if (angular.isArray(pd.selectionValueList) && pd.selectionValueList.length === 0) {

            }
            else if (angular.isArray(pd.selectionValueList.valueEntry) && pd.selectionValueList.valueEntry.length) {
              option = $filter('filter')(pd.selectionValueList.valueEntry, {key: val}, true)[0];
              if (option) {
                val = option.value;
              }
            }
            else {
              if (pd.selectionValueList.valueEntry.key && pd.selectionValueList.valueEntry.value) {
                if (pd.selectionValueList.valueEntry.key == val) {
                  val = pd.selectionValueList.valueEntry.value;
                }
              }
            }
          }
          break;
        case 5: // Checkbox
          if (parseInt(val) == 1 || val == 'true') {
            val = '-' + gettextCatalog.getString('True') + '-';
          }
          else {
            val = '-' + gettextCatalog.getString('False') + '-';
          }
          break;
        case 8:
          val = $filter('setTimeFormat')(val * 1000, 'date', 'short');
          break;
      }
      return val;
    };

    var getPropValue = function (valuesList) {
      var val = '';
      try {
        if ('Value' in valuesList) {
          if (valuesList.Value) {
            val = valuesList.Value;
          }
        }
        else {
          if (valuesList instanceof Array) {
            if (valuesList[0].Value) {
              val = valuesList[0].Value;
            }
          }
        }
      } catch (e) {
        $log.error(e);
      }
      return val;
    };

    var getParentDisplayStyle = function (prop, sex) {
      var val = '';
      try {
        if (sex == 'male' && prop && prop.MaleParentDisplayStyle) {
          val = prop.MaleParentDisplayStyle;
        }
        else if (sex == 'female' && prop && prop.FemaleParentDisplayStyle) {
          val = prop.FemaleParentDisplayStyle;
        }
      } catch (e) {
        $log.error(e);
      }
      return val;
    };

    var getParentValue = function (prop, sex) {
      var val = '';
      try {
        if (sex == 'male' && prop && prop.MaleParentValue) {
          val = prop.MaleParentValue;
        }
        else if (sex == 'female' && prop && prop.FemaleParentValue) {
          val = prop.FemaleParentValue;
        }
      } catch (e) {
        $log.error(e);
      }
      if (val == '-' || val instanceof Array) {
        val = '';
      }
      return val;
    };

    var getScoreTime = function (valuesList) {
      var time = 0;
      if (valuesList.InputDate) {
        time = valuesList.InputDate;
      }
      else {
        if (valuesList instanceof Array) {
          if (valuesList[0].InputDate) {
            time = valuesList[0].InputDate;
          }
        }
      }
      return time;
    };

    service.findCurrentObj = function (items, id) {
      for (var i = 0; i < items.length; i++) {
        if (items[i].id == id) {
          return items[i];
        }
      }
    };


    service.findObject = function (objects, objId) {
      return findObject(objects, objId);
    };

    service.getObjectParent = function (fieldObj) {
      return {
        ParentUniqueId: fieldObj.ParentUniqueId
      };
    };

    service.getObjectMeta = function (data, fieldObject) {
      var fod = $filter('filter')(data.ScObjDefList.ScObjDef, {ScObjDefId: fieldObject.ScObjDefId}, true)[0];
      return {
        Name: fieldObject.ObjectName || fieldObject.ObjectUniqueId || gettextCatalog.getString('Item'),
        Type: fod.ScObjType || gettextCatalog.getString('Item')
      };
    };


    service.getObjectsInfoForTableView = function(data, objects , pds) {
      var fod;
      if (!angular.isDefined(service.fieldObj)) {
        // Not defined, get fod.
        fod = service.getFod(data, 0);
      }
      else {
        // Get parent fod.
        fod = $filter('filter')(data.ScObjDefList.ScObjDef, {ScObjDefId: service.fieldObj.ScObjDefId}, true)[0];
        // Get child fod.
        fod = $filter('filter')(data.ScObjDefList.ScObjDef, {ScObjDefId: fod.ScObjChildDefId}, true)[0];
      }

      angular.forEach(objects, function(value, key) {
        value.tableValues = {};
        var fieldObj = findObject(data.ScoredObjectsList.ScoredObject, value.id);
        DbService.getFieldObjectValues(value.id).then(function (values) {
          if (!(Array.isArray(fod.PropDefList.PropDef))) {
            fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
          }
          fod.PropDefList.PropDef.forEach(function (pd) {
            if (pds.indexOf(pd.id) == -1) {
              return;
            }
            var val = '';
            var prop = $filter('filter')(fieldObj.PropertiesList.Property, {PropId: pd.id}, true)[0];
            if (angular.isDefined(values[pd.id])) {
              value.tableValues[pd.id] =values[pd.id] ;
              val = processPropValue(pd, values[pd.id]);
              value.tableValues[pd.id] = val;
            }
            else if (fieldObj && fieldObj.PropertiesList && fieldObj.PropertiesList.Property && fieldObj.PropertiesList.Property.length) {
              if (prop) {
                if (prop.ValuesList && prop.ValuesList.Value) {
                  val = processPropValue(pd, getPropValue(prop.ValuesList.Value));
                  value.tableValues[pd.id] =  "(" + val + ")";
                }
              }
            }
          })

        });

      })
    }

    service.getObjectInfo = function (data) {
      // Field object def
      var fod = $filter('filter')(data.ScObjDefList.ScObjDef, {ScObjDefId: service.fieldObj.ScObjDefId}, true)[0];

      var objInfo = [];

      if (service.fieldObj && service.fieldObj.PropertiesList && service.fieldObj.PropertiesList.Property && service.fieldObj.PropertiesList.Property.length) {
        // Iterate property definitions
        if (!(Array.isArray(fod.PropDefList.PropDef))) {
          fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
        }

        fod.PropDefList.PropDef.forEach(function (pd) {
          var val = '';
          var scoreTime;
          var femaleParentDisplayStyle = '';
          var maleParentDisplayStyle = '';
          var maleParentValue = '';
          var femaleParentValue = '';
          var historyList = false;
          var displayStyle = '0';
          // Get object's property values if exist
          var prop = $filter('filter')(service.fieldObj.PropertiesList.Property, {PropId: pd.id}, true)[0];
          if (prop) {
            if (prop.ValuesList && prop.ValuesList.Value) {
              displayStyle = prop.ValuesList.Value.displayStyle || pd.displayStyle || '0';
              if (prop.ValuesList.Value.length > 1) {
                historyList = parseHistoryList(pd, prop);
              }

              val = processPropValue(pd, getPropValue(prop.ValuesList.Value));
              if (val !== '') {
                scoreTime = getScoreTime(prop.ValuesList.Value);
              }
            }
            if (prop.MaleParentValue || prop.FemaleParentValue) {
              femaleParentDisplayStyle = getParentDisplayStyle(prop, 'female');
              maleParentDisplayStyle = getParentDisplayStyle(prop, 'male');
              maleParentValue = processPropValue(pd, getParentValue(prop, 'male'));
              femaleParentValue = processPropValue(pd, getParentValue(prop, 'female'));
            }
          }
          // Hide Empty
          // if (val == '' && !maleParentValue && !femaleParentValue) {
          //   return;
          // }

          var objProp = {
            Name: pd.name,
            Type: pd.fieldType,
            Unit: pd.Units,
            Value: val,
            Time: scoreTime,
            MaleParentValue: maleParentValue,
            FemaleParentValue: femaleParentValue,
            History: historyList,
            displayStyle: displayStyle,
            maleParentDisplayStyle: maleParentDisplayStyle,
            femaleParentDisplayStyle: femaleParentDisplayStyle
          };
          objInfo.push(objProp);
        });
      }

      return objInfo;

    };
    service.getObjectsListItemInfo = function (jobData, entry, fod) {
      return getObjectsListItemInfo(jobData, entry, fod);
    };

    var getObjectsListItemInfo = function (jobData, entry, fod) {
      var obj = {
        title: entry.ObjectName || entry.ObjectUniqueId,
        id: entry.ObjectUniqueId,
        ParentUniqueId: entry.ParentUniqueId,
        enableNew: fod.EnableNew || true,
//        enableNew: fod.EnableNew || false,
        isNewItem: entry.isNewItem || 0,
        fav: false,
        properties: []
      };
      if (entry && entry.PropertiesList && entry.PropertiesList.Property && entry.PropertiesList.Property.length) {
        // get the properties list
        if (!(Array.isArray(fod.PropDefList.PropDef))) {
          fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
        }

        fod.PropDefList.PropDef.forEach(function (pd) {
          try {
            var prop = $filter('filter')(entry.PropertiesList.Property, {PropId: pd.id}, true)[0];
            if (pd && pd.inListView == 1) {
              var val;
              if (prop && prop.ValuesList && prop.ValuesList.Value) {
                val = processPropValue(pd, getPropValue(prop.ValuesList.Value));
              }
              else {
                val = '-';
              }
              var objProp = {
                Name: pd.name,
                Value: val
              };
              obj.properties.push(objProp);
            }
          } catch (e) {
            $log.error(e);
          }
        });
      }
      // Get child info
      var childFod = service.getChildFod(jobData, entry.ObjectUniqueId);
      if (childFod) {
        obj.childType = childFod.ScObjType;
        var childObjs = getObjectChildList(jobData, entry.ObjectUniqueId) || [];
        // ToDo: Run this again after deleting items.
        obj.childObjs = childObjs.filter(function (obj) {
          return obj.deleted !== true;
        }).length;
      }

      return obj;
    };

    service.getItemStatus = function (objId, fieldInfo) {
      var deferred = $q.defer();
      DbService.getFieldStatus(objId).then(function (fieldsStatus) {
        fieldInfo.statusClass = HelperService.getStatusClass(fieldsStatus);
        deferred.resolve(fieldInfo);
      });
      return deferred.promise;
    };

    var getPropList = function (fod) {
      var propList = [];
      if (!(Array.isArray(fod.PropDefList.PropDef))) {
        fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
      }

      var pds = fod.PropDefList.PropDef;
      pds.forEach(function (pd) {
        if (pd && pd.inListView == 1) {
          propList.push(pd.name);
        }
      });
      return propList;
    };

    var getFlatPropListForTableView = function(fod, propList) {

      if (!(Array.isArray(fod.PropDefList.PropDef))) {
        fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
      }

      var pds = fod.PropDefList.PropDef;
      pds.forEach(function (pd) {
        if (pd) {
          var group = false;
          if (pd.inListView == 1 || pd.isDescision == 1 || pd.isInput == 1) {
            group = true;
          }
          }
          if (group !== false) {
            propList.push({
              id: pd.id,
              name: pd.name,
              fieldType: pd.fieldType,
            });
          }
      });
      return propList;
    }


    var getPropListForTableView = function (fod, propList) {

      if (!(Array.isArray(fod.PropDefList.PropDef))) {
        fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
      }

      var pds = fod.PropDefList.PropDef;
      pds.forEach(function (pd) {
        if (pd) {
          var group = false;
          if (pd.inListView == 1) {
            group = 'listView';
          }
          else if (pd.isDescision == 1) {
            group = 'decision'
          }
          else if (pd.isInput == 1) {
            group = 'input';
          }
          if (group !== false) {
            propList[group].push({
              id: pd.id,
              name: pd.name,
              fieldType: pd.fieldType,
            });
          }
        }
      });
      return propList;

    }


    service.getFod = function (data, objId) {
      var obj;
      if (objId) {
        obj = findObject(data.ScoredObjectsList.ScoredObject, objId);
      }
      else {
        obj = [].concat(data.ScoredObjectsList.ScoredObject)[0];
      }
      var fod = $filter('filter')(data.ScObjDefList.ScObjDef, {
        ScObjDefId: obj.ScObjDefId
      }, true)[0];
      // Get child fod
      return fod;
    };
    service.setFod = function (data, objId) {
      var obj;
      if (objId) {
        obj = findObject(data.ScoredObjectsList.ScoredObject, objId);
      }
      else {
        obj = [].concat(data.ScoredObjectsList.ScoredObject)[0];
      }
      var fod = $filter('filter')(data.ScObjDefList.ScObjDef, {ScObjDefId: obj.ScObjDefId}, true)[0];
      // Get child fod
      service.fod = fod;
    };


    service.getChildFod = function (data, objId) {
      var obj;
      if (objId) {
        obj = findObject(data.ScoredObjectsList.ScoredObject, objId);
      }
      else {
        obj = [].concat(data.ScoredObjectsList.ScoredObject)[0];
      }
      var fod = $filter('filter')(data.ScObjDefList.ScObjDef, {
        ScObjDefId: obj.ScObjDefId
      }, true)[0];
      // Get child fod
      if (fod.ScObjChildDefId) {
        fod = $filter('filter')(data.ScObjDefList.ScObjDef, {
          ScObjDefId: fod.ScObjChildDefId
        }, true)[0];
      }
      else {
        fod = false;
      }
      return fod;
    };


    service.getPropList = function (fod) {
      var propList = [];
      if (fod) {
        propList = getPropList(fod);
      }
      return propList;
    };


    service.getPropListForTableView = function (fod) {
      var propList = [];
      if (fod) {
        getFlatPropListForTableView(fod, propList);
      }
      return propList;
    };


    service.getChildType = function (fod) {
      return getChildType(fod);
    };
    var getChildType = function (fod) {

      return fod && fod.ScObjType || false;

    };

    // service.getFieldChildsList = function() {
    //   DbService.getJobFieldChildes();
    // }

    service.setFieldsStatus = function (fields, fieldsStatus) {
      if (fields.length) {
        fields.forEach(function (field) {
          if (fieldsStatus !== null) {
            field.statusClass = HelperService.getStatusClass(fieldsStatus[field.id]);
          }
        });
      }
      return fields;
    };
    service.getObjectsList = function (data, objId, fod, fieldsStatus) {
      var objects = {};
      if (objId === null || objId === 0) {
        objects = getJobRootObjects(data);
      }
      else {
        objects = getObjectChildList(data, objId) || [];
      }

      var objectsList = [];
      if (objects.length) {
        objects.forEach(function (entry) {
          var obj = getObjectsListItemInfo(data, entry, fod);
          if (fieldsStatus !== null) {
            obj.statusClass = HelperService.getStatusClass(fieldsStatus[entry.ObjectUniqueId]);
          }
          obj.ObjectId = entry.ObjectId;
          objectsList.push(obj);
        });
        return objectsList;
      }
      else {
        return objectsList;
      }
    };

    service.setFieldStatus = function (items, id, status) {
      for (var i = 0; i < items.length; i++) {
        if (items[i].id == id) {
          items[i].statusClass = HelperService.getStatusClass(status);
          return;
        }
      }
    };

    service.getSort = function (jobId, objId) {
      var storeSort = localStorageService.get('jobData.sorting') || {};
      storeSort[jobId] = storeSort[jobId] || {};
      return storeSort[jobId][objId];
    };

    service.getSortedItemId = function (orderdlist, objId) {
      for (var i = 0, len = orderdlist.length; i < len; i++) {
        if (orderdlist[i].id == objId) {
          return i;
        }
      }
      //      deferred.resolve(orderdlist[destObj] ? orderdlist[destObj].id : false);

    };
    service.setParentData = function (jobId, objId) {
      var deferred = $q.defer();
      service.getSortedList(jobId, objId).then(function (res) {
        service.getParentValues(objId).then(function (values) {
          deferred.resolve();
        });
      });
      return deferred.promise;
    };

    service.getParentValues = function (objId) {
      var deferred = $q.defer();
      DbService.getFieldObjectValues(objId).then(function (values) {
        service.parentValues = values;
        deferred.resolve(values);
      });
      return deferred.promise;
    };


    /**
     *
     * @param jobId
     * @param parent
     * @param setObj if true set the result of the function in the service to by used by the controller's.
     * @param shouldFilter if true filter the list.
     * @returns {promise}
     */
    service.getSortedList = function (jobId, parent, setObj, shouldFilter) {
      if (!angular.isDefined(setObj)) {
        setObj = true;
      }
      if (!angular.isDefined(shouldFilter)) {
        shouldFilter = true;
      }

      var deferred = $q.defer();
      var storedSort,
        predicate,
        reverse,
        orderdlist;

      if (parent === 0) {
        parent = null;
      }
      DbService.getJobFieldChildes(jobId, parent).then(function (objs) {
        storedSort = service.getSort(jobId, parent || 0) || {};
        predicate = storedSort.predicate || false;
        reverse = storedSort.reverse || false;
        if (predicate) {
          orderdlist = $filter('orderBy')(objs, predicate, reverse);
        }
        else {
          // Keep original order.
          orderdlist = objs;
        }
        if (setObj) {
          service.orderdlist = orderdlist;
        }
        if (shouldFilter && storedSort.titleOnly) {
          orderdlist = $filter('filter')(orderdlist, {title: storedSort.titleOnly});
        }
        else if (shouldFilter && storedSort.filter) {
          orderdlist = $filter('filter')(orderdlist, {$: storedSort.filter});
        }
        if (shouldFilter && storedSort.fav) {
          orderdlist = $filter('filter')(orderdlist, {fav: true}, true);
        }
        if (setObj) {
          service.sortedList = orderdlist;
        }
        deferred.resolve(orderdlist);
      });
      return deferred.promise;
    };


    service.setSort = function (jobId, objId, predicate, reverse, search, titleOnly, showOnlyPending) {
      var storeSort = localStorageService.get('jobData.sorting') || {};
      storeSort[jobId] = storeSort[jobId] || {};
      storeSort[jobId][objId] = {
        predicate: predicate,
        reverse: reverse,
        fav: search.fav,
        filter: search.$,
        titleOnly: titleOnly,
        showOnlyPending: showOnlyPending
      };
      localStorageService.set('jobData.sorting', storeSort);
    };

    // Active item
    service.setActiveItem = function (jobId, parent, objId) {
      var storeActive = localStorageService.get('jobData.activeItems') || {};
      storeActive[jobId] = storeActive[jobId] || {};
      if (objId) {
        storeActive[jobId][parent] = objId;
        localStorageService.set('jobData.activeItems', storeActive);
      }
      return storeActive[jobId][parent];
    };

    service.getActiveItem = function (jobId, parent) {
      return service.setActiveItem(jobId, parent);
    };

    // Favorites
    service.toggleFavorite = function (jobId, objId) {
      var storeFav = localStorageService.get('jobData.favorites') || {};
      storeFav[jobId] = storeFav[jobId] || {};
      storeFav[jobId][objId] = storeFav[jobId][objId] ? false : true;
      localStorageService.set('jobData.favorites', storeFav);
      return storeFav[jobId][objId];
    };

    service.getFavorite = function (jobId, objId) {
      var storeFav = localStorageService.get('jobData.favorites') || {};
      storeFav[jobId] = storeFav[jobId] || {};
      return storeFav[jobId][objId] || false;
    };

    // Info pane state
    service.toggleInfoState = function (jobId) {
      var deferred = $q.defer();
      var storedState = localStorageService.get('jobData.infoPaneExpanded') || {};
      storedState[jobId] = storedState[jobId] === false ? true : false;
      localStorageService.set('jobData.infoPaneExpanded', storedState);
      deferred.resolve(storedState[jobId]);
      return deferred.promise;
    };

    service.getInfoState = function (jobId, objId) {
      var deferred = $q.defer();
      var storedState = localStorageService.get('jobData.infoPaneExpanded') || {};
      deferred.resolve(storedState[jobId] === false ? false : true);
      return deferred.promise;
    };

    // Quick edit form filter
    service.setQuickEditFieldsMap = function (jobId, level, fieldsMap) {
      var quickEdit = localStorageService.get('jobData.quickEdit') || {};
      quickEdit[jobId] = quickEdit[jobId] || {};
      quickEdit[jobId][level] = fieldsMap || [];
      localStorageService.set('jobData.quickEdit', quickEdit);
    };

    service.getQuickEditFieldsMap = function (jobId, level) {
      var quickEdit = localStorageService.get('jobData.quickEdit') || {};
      quickEdit[jobId] = quickEdit[jobId] || {};
      return quickEdit[jobId][level] || [];
    };

    // Table view config
    service.setTableViewConfig = function (jobId, level, config) {
      var storedConfig = localStorageService.get('jobData.tableViewConfig') || {};
      storedConfig[jobId] = storedConfig[jobId] || {};
      storedConfig[jobId][level] = config || [];
      localStorageService.set('jobData.tableViewConfig', storedConfig);
      return storedConfig[jobId][level];
    };

    service.getTableViewConfig = function (jobId, level) {
      var storedConfig = localStorageService.get('jobData.tableViewConfig') || {};
      storedConfig[jobId] = storedConfig[jobId] || {};
      return storedConfig[jobId][level] || [];
    };

    // Table view state (on/off)
    service.setTableViewStatus = function (jobId, level, state) {
      var storedStatus = localStorageService.get('jobData.tableViewStatus') || {};
      storedStatus[jobId] = storedStatus[jobId] || {};
      storedStatus[jobId][level] = state || false;
      localStorageService.set('jobData.tableViewStatus', storedStatus);
      return storedStatus[jobId][level];
    };

    service.getTableViewStatus = function (jobId, level) {
      var storedStatus = localStorageService.get('jobData.tableViewStatus') || {};
      storedStatus[jobId] = storedStatus[jobId] || {};
      return storedStatus[jobId][level] || false;
    };

    // Add new item 'numChildsToAdd' - mark as used
    service.usedNumChildsToAdd = function (jobId, objId) {
      var storeUsed = localStorageService.get('jobData.usedNumChilds') || {};
      storeUsed[jobId] = storeUsed[jobId] || {};
      storeUsed[jobId][objId] = true;
      localStorageService.set('jobData.usedNumChilds', storeUsed);
    };

    service.getUsedNumChildsToAdd = function (jobId, objId) {
      var storeUsed = localStorageService.get('jobData.usedNumChilds') || {};
      storeUsed[jobId] = storeUsed[jobId] || {};
      return storeUsed[jobId][objId] || false;
    };

    var getRequiredIds = function (data, fod) {
      var ids = {};
      if (!(Array.isArray(fod.PropDefList.PropDef))) {
        fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
      }

      $filter('filter')(fod.PropDefList.PropDef, {isRequired: '1'}, true).forEach(function (pd) {
        ids[pd.id] = pd.id;
      });
      return ids;
    };

    service.setFieldObjProp = function (fieldObj, parentValues) {
      if (parentValues && Object.keys(parentValues).length > 0 && fieldObj.PropertiesList && fieldObj.PropertiesList.Property && 'isNewItem' in fieldObj && fieldObj.isNewItem == 1) {
        fieldObj.PropertiesList.Property.forEach(function (parentProp) {
          if ('inheritedValue' in parentProp && parentProp.inheritedValue === true && parentProp.PropId in parentValues) {
            parentProp.ValuesList.Value.Value = parentValues[parentProp.PropId];
            parentProp.ValuesList.Value.displayStyle = 1;
          }
        });
      }
      service.fieldObj = fieldObj;
      return fieldObj;
    };

    service.setJobFieldChildes = function (jobId, fid) {
      var deferred = $q.defer();
      service.getSortedList(jobId, fid, false, false).then(function (objs) {
        service.objs = objs;
        deferred.resolve();
      });
      return deferred.promise;
    };



    service.GetFormToCache = function (fod, jobId, decisions, propList) {
      var objForm = [];
      if (!(Array.isArray(fod.PropDefList.PropDef))) {
        fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
      }

      if (fod.PropDefList.PropDef.length) {
        //  get the properties list
        var serialNumber = 0;
        var filterStr = {};
        filterStr.isInput = '1';
        if (decisions) {
          filterStr.isDescision = '1';
        }
        else {
          filterStr.isDescision = '!1';
        }
        if (!(Array.isArray(fod.PropDefList.PropDef))) {
          fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
        }

        $filter('filter')(fod.PropDefList.PropDef, filterStr, true).forEach(function (pd) {
          try {
            var fieldForm = {};
            fieldForm.required = false;
            fieldForm.odefId = fod.ScObjDefId;
            fieldForm.title = pd.name;
            fieldForm.pd = parseInt(pd.id);
            fieldForm.id = jobId;
            fieldForm.key = pd.id;
            // fieldForm.onChange = 'updated(modelValue,form)';
            // fieldForm.onClick = "clickHandler($event, form)";

            fieldForm.serialNumber = ++serialNumber;

            if ('Units' in pd) {
              fieldForm.unit = pd.Units;
            }
            else {
              fieldForm.unit = '';
            }

            if ('isRequired' in pd) {
              if (parseInt(pd.isRequired) == 1) {
                fieldForm.required = true;
              }
            }

            switch (parseInt(pd.fieldType)) {
              case 1:
                // Range numeric (from to)
                if ('minValue' in pd) {
                  fieldForm.minimum = parseFloat(pd.minValue);
                }
                if ('maxValue' in pd) {
                  fieldForm.maximum = parseFloat(pd.maxValue);
                }
                if ('precision' in pd) {
                  fieldForm.precision = parseFloat(pd.precision);
                }

                fieldForm.type = 'range';
                fieldForm.inputType = 'tel';
                break;
              case 2:
                // Number
                if ('minValue' in pd) {
                  fieldForm.minimum = parseFloat(pd.minValue);
                }
                if ('maxValue' in pd) {
                  fieldForm.maximum = parseFloat(pd.maxValue);
                }
                if ('precision' in pd) {
                  fieldForm.precision = parseFloat(pd.precision);
                }
                fieldForm.type = 'number';
                fieldForm.inputType = 'tel';
                break;
              case 3:
                // Select list
                if (pd.selectionValueList && pd.selectionValueList.valueEntry) {

                  var options = [];
                  // Add empty option as default;
                  if (!fieldForm.required) {
                    var optionNone = {value: '', name: ''};
                    options.unshift(optionNone);
                    fieldForm.currVal = fieldForm.currVal || '';
                  }

                  var option;
                  if (angular.isDefined(pd.selectionValueList) && angular.isArray(pd.selectionValueList.valueEntry) && pd.selectionValueList.valueEntry.length) {
                    pd.selectionValueList.valueEntry.forEach(function (selectVal) {
                      if ('fullValue' in selectVal) {
                        option = {value: selectVal.key, name: selectVal.fullValue, smallValue : selectVal.value };
                      }
                      else {
                        option = {value: selectVal.key, name: selectVal.value};
                      }
                      options.push(option);
                    });
                  }
                  else {
                    if (pd.selectionValueList.valueEntry.key && pd.selectionValueList.valueEntry.value) {
                      if ('fullValue' in pd.selectionValueList.valueEntry) {
                        option = {
                          value: pd.selectionValueList.valueEntry.key,
                          name: pd.selectionValueList.valueEntry.fullValue,
                          smallValue : pd.selectionValueList.valueEntry.value
                        };
                      }
                      else {
                        option = {
                          value: pd.selectionValueList.valueEntry.key,
                          name: pd.selectionValueList.valueEntry.value
                        };
                      }

                      options.push(option);
                    }
                  }
                  fieldForm.type = 'select';
                  fieldForm.titleMap = options;

                }
                break;
              case 4:
                var imgDir;
//                cordova.file.externalApplicationStorageDirectory

                try {
                  imgDir = cordova.file.dataDirectory + "ib/images/" + jobId + "/" + pd.id + "/";
                }
                catch (e) {
                }
//                var imgDir = cordova.file.dataDirectory + "/";
                var imagesOptions = [];
                var imageOption;
                fieldForm.inputType = 'text';
                fieldForm.type = 'images';

//                selectionValueList: ObjectvalueEntry: Array[3]0: Object1: Objectkey: "3"pictureName: "fig-interior-G.jpg"value: "int. G"

                if (angular.isArray(pd.selectionValueList.valueEntry) && pd.selectionValueList.valueEntry.length) {


                  var rootDir;
                  if ('picturesFolderURL' in pd) {
                    rootDir = pd.picturesFolderURL;
                  }
                  else {
                    rootDir = propList.picturesFolderURL;
                  }
                  if (rootDir.substr(-1) !== "/") {
                    rootDir = rootDir + "/";


                  }


                  pd.selectionValueList.valueEntry.forEach(function (selectVal) {
                    try {
                      if (angular.isDefined($cordovaFileTransfer)) {
                        $cordovaFileTransfer.download(encodeURI(rootDir + selectVal.pictureName), imgDir + selectVal.pictureName, function (file) {
                        }, function (error) {
                          console.log(error);
                        });
                        imageOption = {
                          value: selectVal.key,
                          webPath: rootDir + selectVal.pictureName,
                          path: imgDir + selectVal.pictureName,
                          title: selectVal.value
                        };
                      }
                      else {
                        imageOption = {
                          value: selectVal.key,
                          webPath: rootDir + selectVal.pictureName,
                          path: false,
                          title: selectVal.value
                        };
                      }
                    }
                    catch (e) {
                      imageOption = {
                        value: selectVal.key,
                        webPath: rootDir + selectVal.pictureName,
                        path: false,
                        title: selectVal.value
                      };

                      $log.error(e);

                    }

                    imagesOptions.push(imageOption);

                  });
                }
                else {
                  if (pd.imagesList.valueEntry.key && pd.imagesList.valueEntry.value) {
                    try {
                      $cordovaFileTransfer.download(pd.PictureFolder + pd.imagesList.valueEntry.value, imgDir + pd.imagesList.valueEntry.value);
                      imageOption = {
                        value: pd.imagesList.valueEntry.key,
                        webPath: pd.PictureFolder + pd.imagesList.valueEntry.value,
                        path: imgDir + pd.imagesList.valueEntry.value,
                        title: pd.imagesList.valueEntry.title
                      };
                    }
                    catch (e) {
                      imageOption = {
                        value: pd.imagesList.valueEntry.key,
                        webPath: pd.PictureFolder + pd.imagesList.valueEntry.value,
                        path: false,
                        title: pd.imagesList.valueEntry.title
                      };
                      $log.error(e);
                    }
                    imagesOptions.push(imageOption);
                  }
                }
                fieldForm.imagesList = imagesOptions;
                break;
              case 5:
                //Checkbox
                fieldForm.type = 'boolean';
                if (angular.isDefined(fieldForm.default)) {
                  if (fieldForm.default.toLowerCase() === 'true' || fieldForm.default.toLowerCase() === 'yes') {
                    fieldForm.default = true;
                  }
                }
                break;
              case 6:
                // One line text
                fieldForm.inputType = 'string';
                fieldForm.type = 'string';
                fieldForm.width = 'half';
                if ('maxValue' in pd) {
                  fieldForm.maxLength = pd.maxValue;
                }
                if ('scanBarcode' in pd) {
                  if (pd.scanBarcode == "yes" || pd.scanBarcode == true || pd.scanBarcode == "true") {
                    pd.scanBarcode = true;
                  }
                  else {
                    pd.scanBarcode = false;
                  }
                  fieldForm.scanBarcode = pd.scanBarcode;
                }
                else {
                  fieldForm.scanBarcode = false;
                }
                break;
              case 7:
                // Long text
                if ('maxValue' in pd) {
                  fieldForm.maxLength = pd.maxValue;
                }

                fieldForm.type = 'string';
                fieldForm.placeholder = gettextCatalog.getString('Make a comment');
                fieldForm.type = 'textarea';
                break;
              case 8:
                fieldForm.type = 'date';
                break;
            }
            objForm.push(fieldForm);
          } catch (e) {
            $log.error(e);
          }
        });
      }

      return objForm;
    };

    service.getFormFieldInfo = function (fieldObj) {
      var formFieldInfo = {};
      if ('isNewItem' in fieldObj && fieldObj.isNewItem == 1) {
        formFieldInfo.isNew = fieldObj.isNewItem;
        formFieldInfo.pfid = fieldObj.ParentUniqueId;
        formFieldInfo.userFid = fieldObj.ObjectName;
        formFieldInfo.odefId = fieldObj.ScObjDefId;
      }
      else {
        formFieldInfo.isNew = 0;
        formFieldInfo.pfid = 0;
        formFieldInfo.userFid = '';
        formFieldInfo.odefId = 0;
      }
      formFieldInfo.fid = fieldObj.ObjectUniqueId;
      return formFieldInfo;
    };

    service.getProcessedForm = function (jobId, type) {
      var deferred = $q.defer();
      DbService.getJobFormByType(jobId, service.fieldObj.ScObjDefId, type).then(function (form) {
        DbService.getFieldObjectValues(service.fieldObj.ObjectUniqueId).then(function (values) {
          var prop, val;
          form.forEach(function (formField) {
            try {
              formField.oldval = '';
              formField.uniqueIndex = service.fieldObj.ObjectUniqueId.toString() + formField.key;
              if (!(Array.isArray(service.fod.PropDefList.PropDef))) {
                service.fod.PropDefList.PropDef = [service.fod.PropDefList.PropDef];
              }

              var pd = $filter('filter')(service.fod.PropDefList.PropDef, {id: formField.key}, true)[0];
              if (service.fieldObj && service.fieldObj.PropertiesList) {
                var items = $filter('filter')(service.fieldObj.PropertiesList.Property, {PropId: formField.key}, true);
                if (items) {
                  prop = $filter('filter')(service.fieldObj.PropertiesList.Property, {PropId: formField.key}, true)[0];
                }
              }
              // History value/value-list
              if (prop && prop.ValuesList && prop.ValuesList.Value) {
                val = getPropValue(prop.ValuesList.Value);

                formField.oldval = processPropValue(pd, val);
                // History
                formField.historyList = false;
                if (prop.ValuesList.Value.length > 1) {
                  formField.historyList = parseHistoryList(pd, prop);
                }
                // displayStyle
                if ('displayStyle' in prop.ValuesList.Value) {
                  formField.displayStyle = prop.ValuesList.Value.displayStyle;
                }
              }
              // M/F Parents values.
              if (formField.oldval === '' || formField.oldval === '-') {
                formField.maleParentVal = processPropValue(pd, getParentValue(prop, 'male'));
                formField.femaleParentVal = processPropValue(pd, getParentValue(prop, 'female'));
                formField.femaleParentDisplayStyle = getParentDisplayStyle(prop, 'female');
                formField.maleParentDisplayStyle = getParentDisplayStyle(prop, 'male');
              }

              if (formField.key in values) {
                formField.currVal = values[formField.key];

              }

              if (pd) {
                switch (parseInt(pd.fieldType)) {
                  case 2:
                    //Float
                    if (pd.id in values) {
                      formField.currVal = parseFloat(values[pd.id]);
                    }
                    if (pd.precision) {
                      formField.precision = parseInt(pd.precision, 10);
                    }
                    break;
                  case 3:
                    formField.displayValue = formField.titleMap.find(function(item){return item.value == formField.currVal}).name
                    break;
                  case 4:
                    // Select
                    for (var i = 0; i < formField.imagesList.length; i++) {
                      if (formField.imagesList[i].value == formField.currVal) {
                        formField.currVal = formField.imagesList[i].title;
                        break;
                      }
                    }

                    break;
                  case 5:
                    //Checkbox
                    if (formField.currVal === 'true' || formField.currVal === 'yes') {
                      formField.currVal = true;
                    }
                    break;
                  case 8:
                    // Date
                    if (pd.id in values) {
                      formField.currVal = new Date(values[pd.id] * 1000);
                    }
                    break;
                }
              }

            } catch (e) {
              $log.error(e);
            }
          });
          deferred.resolve(form);
        });
      });
      return deferred.promise;
    };

    service.checkField = function (job, fid, fod) {
      var deferred = $q.defer();
      var ids = getRequiredIds(job.data, fod);
      var idsKeys = Object.keys(ids);
      var status;
      DbService.getFieldObjectValues(fid).then(function (values) {
        var valuesKeys = Object.keys(values);
        if (idsKeys.length > 0) {

          var diff = idsKeys.diff(valuesKeys);
          if (diff.length > 0) {
            if (valuesKeys.length > 0) {
              status = FIELD_STATUS_IN_PROG;
            }
            else {
              status = FIELD_STATUS_PEND;
            }
          }
          else {
            status = FIELD_STATUS_COMP;
          }

        }
        else {
          if (valuesKeys.length > 0) {
            status = FIELD_STATUS_COMP;
          }
          else {
            status = FIELD_STATUS_PEND;
          }
        }
        DbService.updateJobField(fid, status);
        checkJob(job.id);
        deferred.resolve(status);
      });
      return deferred.promise;
    };

		/**
		 * If this is the last item, get the
		 * first one from the next tree.
		 */
		service.getNextItem = function(JobsFactory) {
			var deferred = $q.defer();
			var search = $location.search();
			var parent = service.findObject(JobsFactory.job.data.ScoredObjectsList.ScoredObject, search.parentId)
			var sParrent = service.getObjectParent(parent);
			service.getSortedList(JobsFactory.job.id, sParrent.ParentUniqueId, false).then(function (res) {
				for(var i = 0; i < res.length ; i++) {
					if(res[i].id == parent.ObjectUniqueId) {
						if(i <= res.length - 2) {
							var nextUncle = res[i+1];
							service.getSortedList(JobsFactory.job.id, nextUncle.id, false).then(function (res) {
								var result = false;
								if(res.length > 0) {
									result = {
										parentId: res[0].ParentUniqueId,
										objId: res[0].id,
										parent: parent,
									};
								}
								deferred.resolve(result);
							})
						}
					}
				}
			});
			return deferred.promise;
		}

		/**
		 * If this is the last item, get the
		 * first one from the next tree.
		 */
		service.getPrevItem = function(JobsFactory) {
			var deferred = $q.defer();
			var search = $location.search();
			var parent = service.findObject(JobsFactory.job.data.ScoredObjectsList.ScoredObject, search.parentId)
			var sParrent = service.getObjectParent(parent);
			service.getSortedList(JobsFactory.job.id, sParrent.ParentUniqueId, false).then(function (res) {
				for(var i = 0; i < res.length ; i++) {
					if(res[i].id == parent.ObjectUniqueId) {
						if(i != 0) {
							var prevUncle = res[i-1];
							service.getSortedList(JobsFactory.job.id, prevUncle.id, false).then(function (res) {
								var result = false;
								if(res.length > 0) {
									result = {
										parentId: res[res.length - 1].ParentUniqueId ,
										objId: res[res.length - 1].id,
										parent: parent,
									};
								}
								deferred.resolve(result);
							})
						}
					}
				}
			});
			return deferred.promise;
		}

    function getNextAvailableID(id, parent) {
      var found = true,
        uniqueId;
      id = isNaN(id) ? parseInt(id) : id;
      if (!parent || !parent.NestedObjList || !parent.NestedObjList.ScoredObject) {
        return id;
      }
      // Search if 'available' exist in 'parent'
      while (angular.isDefined(found)) {
        uniqueId = id;
        found = parent.NestedObjList.ScoredObject.filter(function (obj) {
          return obj.ObjectUniqueId == uniqueId;
        })[0];
        id += 1;
      }
      return uniqueId;
    }

    var inheritParentValues = function (childFod, fod, parent) {
      var propList = [];

      if (!('PropertiesList' in parent)) {
        return propList;
      }

      if (!(Array.isArray(childFod.PropDefList.PropDef))) {
        childFod.PropDefList.PropDef = [childFod.PropDefList.PropDef];
      }

      childFod.PropDefList.PropDef.forEach(function (childProp) {
        if (!(Array.isArray(fod.PropDefList.PropDef))) {
          fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
        }

        fod.PropDefList.PropDef.forEach(function (parentProp) {
          // Parent-child prop match
          if (childProp.id == parentProp.id) {
            var propValue = '';
            var parentFieldProp = $filter('filter')(parent.PropertiesList.Property, {PropId: childProp.id}, true)[0];

            if (parentFieldProp && parentFieldProp.ValuesList && parentFieldProp.ValuesList.Value) {
              propValue = getPropValue(parentFieldProp.ValuesList.Value);
            }

            var MaleParentValue = getParentValue(parentFieldProp, 'male');
            var FemaleParentValue = getParentValue(parentFieldProp, 'female');
            var FemaleParentDisplayStyle = getParentDisplayStyle(parentFieldProp, 'female');
            var MaleParentDisplayStyle = getParentDisplayStyle(parentFieldProp, 'male');

            var newProp = {
              'PropId': childProp.id,
              'MaleParentDisplayStyle': MaleParentDisplayStyle,
              'FemaleParentDisplayStyle': FemaleParentDisplayStyle,
              'MaleParentValue': MaleParentValue,
              'FemaleParentValue': FemaleParentValue,
              'inheritedValue': true,
              'ValuesList': {
                'Value': {
                  'InputDate': '0',
                  'displayStyle': '0',
                  'Value': propValue
                }
              }
            };
            propList.push(newProp);
          }
        });
      });
      return propList;
    };

    service.getRelatedItems = function(jobId) {
      var self = this;
      var deferred = $q.defer();
      if (self.relatedPlots != false) {
        self.getSortedList(jobId, 0, false, false).then(function (list) {
          list = $filter('filter')(list , function(value, index, array) {
            var item = $filter('filter')(self.relatedPlots, {RelatedPlotId : value.ObjectId})
            if (item.length) {
              return true;
            }
          });
          deferred.resolve(list);
        });
      }
      else {
        deferred.resolve(false);
      }
      return deferred.promise;
    }

    service.enterRelatedMode = function (id) {
      service.relatedMode = id;
    }

    service.getRelatedMode = function () {
      return service.relatedMode;
    }

    service.exitRelatedMode = function () {
      service.relatedMode = false;
    }

    service.setBC = function (bc) {
      var bcArr = []
      bc.forEach(function(b) {
        bcArr.push(b.ObjectId.trim());
      })
      service.bc = bcArr;
    }
    service.getBC = function() {
      return service.bc;
    }


    service.addSubDirs = function(path, dirs, dirA, cb) {
      if (dirs.length) {
        dirA.push(dirs.shift());
        service.checkDir(path, dirA.join("/")).then(function() {
          service.addSubDirs(path, dirs, dirA, cb)
        })
      }
      else {
        cb(dirA.join("/"));
      }
    }

    service.checkDirs = function (path, dirs) {
      var deferred = $q.defer();
      service.addSubDirs(path, dirs, [], function(finalPath) {
        deferred.resolve(finalPath);
      })
      return deferred.promise;
    }

    service.checkDir = function(path, dir) {
      var deferred = $q.defer();
      $cordovaFile.checkDir(path, dir).then(function (success) {
        deferred.resolve(true);
      }, function(err) {
        $cordovaFile.createDir(path, dir).then(function(res) {
          deferred.resolve(true);
        })
      });;
      return deferred.promise;
    }

    service.addNewObjects = function (job, parentID, ObjectUniqueIds) {
      var deferred = $q.defer();
      var parent = findObject(job.data.ScoredObjectsList.ScoredObject, parentID);
      var fod = $filter('filter')(job.data.ScObjDefList.ScObjDef, {ScObjDefId: parent.ScObjDefId}, true)[0];
      var childFod = $filter('filter')(job.data.ScObjDefList.ScObjDef, {ScObjDefId: fod.ScObjChildDefId}, true)[0];
      var fields = [];
      var newObjs = [];
      var parentValues = [];
      if (childFod) {
        parentValues = inheritParentValues(childFod, fod, parent);
      }
      for (var i = 0; i < ObjectUniqueIds.length; i++) {
        var newObj = {
          ObjectUniqueId: getNextAvailableID(parseInt(Date.now() / 1000), parent),
          ParentUniqueId: parentID,
          ObjectName: ObjectUniqueIds[i],
          ObjectId: ObjectUniqueIds[i],
          isNewItem: 1,
          ScObjDefId: fod.ScObjChildDefId,
          Level: parseInt(parent.Level) + 1,
          PropertiesList: {}
        };

        // Inherit value from parent
        newObj.PropertiesList.Property = parentValues;
        // Default value
        if (!(Array.isArray(childFod.PropDefList.PropDef))) {
          childFod.PropDefList.PropDef = [childFod.PropDefList.PropDef];
        }

        childFod.PropDefList.PropDef.forEach(function (childProp) {
          if ('defaultValue' in childProp) {
            DbService.insertJobValue(job.id, newObj.ObjectUniqueId, childProp.id, Date.now() / 1000, childProp.defaultValue, parentID, ObjectUniqueIds[i], fod.ScObjChildDefId);
          }
        });

        if (parent.NestedObjList) {
          if (Array.isArray(parent.NestedObjList)) {
            parent.NestedObjList = {};
          }
        }
        else {
          parent.NestedObjList = {};
        }
        parent.NestedObjList.ScoredObject = parent.NestedObjList.ScoredObject || [];
        parent.NestedObjList.ScoredObject.push(newObj);
        fields.push([fod.ScObjChildDefId, newObj.ObjectUniqueId, job.id, parentID, parseInt(parent.Level) + 1, newObj.ObjectName]);
        newObjs.push(newObj);
      }
      DbService.updateJobData(job.id, JSON.stringify(job.data)).then(function () {
        DbService.insertJobFields(fields, 1);
        DbService.updateJobSyncStatus(job.id, 0).then(function () {
          deferred.resolve(newObjs);
        });
      });
      return deferred.promise;
    };

    return service;
  });

'use strict';
angular.module('ib.services')
  .service('GotoService', function($location, $state, $ionicLoading, $log) {
    var startTime;
    var endTime;

    this.goTo = function (parentId, objId, tab, showLoading) {
      // ToDo handle goTo to current item freeze app.
      this.logStart();
      tab = tab || 'edit';
      if (tab === 'quick-edit') {
        this.goToQuickEdit(parentId, objId, showLoading);
      }
      else {
        this.goToTabs(parentId, objId, tab, showLoading);
      }
      this.ionicLoading(showLoading);
    };

    this.goToTabs = function(parentId, objId, tab) {
      if ($state.current.name.indexOf('app.job.obj.tab') == -1) {
        // Coming from another view (plot-list/quick-edit).
        $location.path($state.href('app.job.obj.tabs.tab-' + tab).replace('#', '')).search({parentId: parentId,objId: objId, tab:tab});
      }
      else {
        // Already in a tab view.
        $location.search({parentId: parentId, objId: objId, tab:tab});
      }
    };

    this.goToQuickEdit = function(parentId, objId, tab) {
      if ($location.path().indexOf('quick-edit') === -1) {
        // Coming from another view
        $location.path($state.href('app.job.obj.quick-edit').replace('#', '')).search({parentId: parentId,objId: objId, tab:tab});
      }
      else {
        // Coming from quick edit, different object, change search only.
        $location.search({parentId: parentId, objId: objId});
      }
    };

    this.ionicLoading = function(showLoading) {
      this.showLoading = showLoading || 'load';
      if (this.showLoading === 'load') {
        $ionicLoading.show();
        this.showLoading = true;
      }
    };

    this.logStart = function() {
      startTime = Date.now();
      $log.info('goto enter time ' + startTime, {'controllerStartTime' : startTime});
    };

    this.endGoto = function() {
      endTime = Date.now();
      $log.info('goto end time ' + Date.now(), {'controllerEndTime' : Date.now()});
      $log.info('goto total ' + startTime - endTime, {'totalTime' : startTime - endTime});
      if (this.showLoading) {
        $ionicLoading.hide();
      }
    };
});

angular.module('ib.services')


  .service('HelperService', function($ionicPlatform, $cordovaNetwork, $q, $filter, $log, localStorageService) {
    var ok = function(value) {

    }
    var error = function() {

    }
    this.setLocalValue = function(key, val) {
      try {
        var prefs = plugins.appPreferences;
        prefs.fetch (ok, fail, 'key');
      } catch (e) {
        return localStorageService.get('username');
      }
    }
    this.getLocalValue = function(key, val) {

    }

    var getDateFormatsFallback = function() {
      return {'date short': 'd/M/yy', 'date medium': 'd MMMM yyyy', 'date and time short': 'H:mm - d/M/yy', 'date default': 'yyyy-MM-dd'};
    };

    this.getDateFormats = function() {
      var dateMap = {};

      var deferred = $q.defer();
      try {
        var options;
        var self = this;
        var successCallbackDateShort = function (date) {
          dateMap['date short'] = date.pattern;
          dateMap['date default'] = 'yyyy-MM-dd';
          self.dateMap = dateMap;
          deferred.resolve(true);
        }

        var successCallbackDateMedium = function (date) {
          dateMap['date medium'] = date.pattern;
          options = {formatLength: 'short', selector: 'date'};
          navigator.globalization.getDatePattern(successCallbackDateShort, errorCallback, options);
        }

        var successCallbackDefault = function (date) {
          dateMap['date and time short'] = date.pattern;
          options = {formatLength: 'medium', selector: 'date'};
          navigator.globalization.getDatePattern(successCallbackDateMedium, errorCallback, options);
        }

        var errorCallback = function (err) {
          $log.error(err);
        }

        options = {formatLength: 'short', selector: 'date'};

        navigator.globalization.getDatePattern(successCallbackDefault, errorCallback);
      } catch (e) {
        this.dateMap = getDateFormatsFallback();

        deferred.resolve(true);
      }

      return deferred.promise;
    }
    this.formatDate = function(date, selector, formatLength) {
      var formatStr = '';
      try {
        formatStr = $filter('date')(date, this.dateMap[selector + ' ' + formatLength]);
        return formatStr;
      } catch (e) {
        return '';
      }
    }

    this.isOnlineCheck = function() {
      var isOnline;
      try {
        isOnline = $cordovaNetwork.isOnline();
      } catch (e) {
        isOnline = navigator.onLine;
      }
      return isOnline;
    };

    this.checkConnectionMode = function() {
      var connection;
      $ionicPlatform.ready(function() {
        try {
          var connectionType = $cordovaNetwork.getNetwork();
          if (connectionType == Connection.WIFI) {
            connection = 1;
          }
          else if (connectionType == Connection.NONE) {
            connection = -1;
          }
          else {
            connection = 0;
          }
        } catch (e) {
          connection = navigator.onLine ? 1 : -1;
        };
      });
      return connection;
    }

    this.getStatusClass = function(status) {
      status = parseInt(status);
      var statusClass;
      switch (status) {
        case 0:
          statusClass = "pending";
          break;
        case 1:
          statusClass = "inProgress";
          break;
        case 2:
          statusClass = "completed";
          break;
        case 3:
          statusClass = "done";
          break;
        case 4:
          statusClass = "sync";
          break;
      }
      return statusClass;
    };
})

// http://blog.altoros.com/speed-up-i18n-in-angularjs.html
// angular.module('ib.services')
// .factory('i18nInterceptor', ['$q', '$translate', function($q, $translate) {
//     return {
//         'request': function(config) {
//             return config;
//         },

//         'response': function(response) {
//             _.templateSettings = {
//                 'interpolate': /\[\[([\s\S]+?)\]\]/g
//             };
//             if (typeof response.data === 'string') {
//                 response.data = _.template(response.data, {t: $translate});
//             }
//             return response;
//         }
//     };
// }])

angular.module('ib.services')
.service('InitService', function($rootScope, $q, DbService, UpdateService, HelperService, RestService, $log, localStorageService) {
  var deferred = $q.defer();
  var _init = false;
  this.getInit = function() {
    return _init;
  }
  $rootScope.$on('app.run', function() {
    $rootScope.clientId = localStorageService.get('clientId' || 0);
    $log.setExtraDef($rootScope.ibVersion, $rootScope.ibPackage);
    $log.overrideExtraDef({'_userClientId' : $rootScope.clientId, '_userName' : localStorageService.get('username') || 'anonymous'});
    var needUpdate = false;
    localStorageService.set('appVerU', $rootScope.ibVersion);
    // LogDbService.run(needUpdate);
    DbService.run(needUpdate).then(function() {
      // LogSelectDbService.run();
      RestService.setHttp();
      HelperService.getDateFormats().then(function(res){
        UpdateService.run().then(function() {
          _init = true;
          deferred.resolve();
        });
      });
    });

  });
  this.run = function() {
    return deferred.promise;
  };
})

angular.module('ib.services')
.factory('JobsFactory', function($q, DbService, RestService, HelperService, $cordovaFile, $rootScope) {
    var service = {};
    service.job = {};
    service.init = false;
    //var getJobData = function(id) {
    //  return DbService.selectJobData(id);
    //}
    var getJobData = function(id) {
      var deferred = $q.defer();
      var res = {data: ''};
      DbService.selectJobData(id).then(function(results) {
        res.status = results.rows.item(0).status;
        if ($rootScope.mode == "mobile") {
          $cordovaFile.readAsText(cordova.file.dataDirectory, id + ".json")
            .then(function (file) {
              res.data = file;
              deferred.resolve(res);
            }, function (error) {
              deferred.resolve(res);
            });
        }
        else {
          res.data = results.rows.item(0).data;
          deferred.resolve(res);
        }
      });
        return deferred.promise;
    }

    var getJobs = function(deferred) {
      DbService.selectJobs().then(function(results) {
        var len = results.rows.length, i;
        var items = [];
        for (i = 0; i < len; i++){
          var tempItem = results.rows.item(i);
          var statusClass = HelperService.getStatusClass(tempItem.status);
          var o = {id: tempItem.id, jobDate: tempItem.jobDate, jobName: tempItem.jobName, jobProperties: JSON.parse(tempItem.jobProperties), status: tempItem.status, statusClass:statusClass};
          items.push(o);
        }
        deferred.resolve(items);
      });

    }
    service.setActiveJob = function(id) {
      var deferred = $q.defer();
      service.job.id = id;
      getJobData(id).then(function(result) {
        service.job.data = JSON.parse(result.data);
        service.job.data.JobUniqueId = id;
        service.job.status = result.status;
        deferred.resolve(service.job);
      });
      return deferred.promise;
    }

    service.getJobs = function() {
      var deferred = $q.defer();
      getJobs(deferred);
      return deferred.promise;
    };
    service.refreshJobs = function() {
      service.init = true;
      return RestService.getJobsFromServer();
    };

    return service;
})

'use strict';
angular.module('ib.services')
  .service('LogSelectDbService', function($window, $q, LogDbService) {
    var LOG_DB_NAME = "iblog";
    var _db;
    this.run = function() {
      createDb();
    }
    var createDb = function() {
      try {
        _db = $window.sqlitePlugin.openDatabase({ name: LOG_DB_NAME});
      } catch (e) {
        _db = $window.openDatabase(LOG_DB_NAME, '1.0', 'IB', 10 * 3072 * 1024);
      }
    }
    this.deleteLogMsg = function(id) {
      var query = "DELETE FROM ib_log where id = ?";
      // LogDbService.db.transaction(function(tx) {
      //   tx.executeSql(query,[id], dbSuccess, dbError);
      // });
      var dbSuccess = function(tx, results) {
      }
      var dbError = function(e) {
        console.log('error in create db');
      }
    }
    this.getLogs = function() {
      var deferred = $q.defer();
      var query = "SELECT id, logMsg FROM ib_log";
      // LogDbService.db.transaction(function (tx) {
      //   tx.executeSql(query, [], dbSuccess, dbError);
      // });
      var dbSuccess = function(tx, results) {
        var len = results.rows.length, i;
        var items = [];
        for (i = 0; i < len; i++){
          items.push({'id' : results.rows.item(i).id, 'logMsg' : JSON.parse(results.rows.item(i).logMsg)});
        }
        deferred.resolve(items);
      }
      var dbError = function(e) {
        console.log('error in getLogs');
        deferred.resolve([]);
      }
      return deferred.promise;
    }
  })
  .service('LogDbService', function($window) {
    var LOG_DB_NAME = "iblog";
    this.run = function(needUpdate) {
      this.createDb(needUpdate);
    }
    this.createDb = function(needUpdate) {
      try {
        this.db = $window.sqlitePlugin.openDatabase({ name: LOG_DB_NAME});
      } catch (e) {
        this.db = $window.openDatabase(LOG_DB_NAME, '1.0', 'IB', 10 * 3072 * 1024);
      }
      this.createTables(needUpdate);
    }
    this.createTables = function(needUpdate) {
      this.db.transaction(function(tx) {
        if (needUpdate) {
          tx.executeSql("drop table ib_log", []);
        }
        tx.executeSql("CREATE TABLE IF NOT EXISTS ib_log(id integer primary key, logMsg text)", []);
      })
    }
    this.insertLogMsg = function(logMsg) {
      var query = "INSERT INTO ib_log (logMsg) VALUES (?)";
      this.db.transaction(function(tx) {
        tx.executeSql(query,[logMsg], dbSuccess, dbError);
      });
      var dbSuccess = function(tx, results) {
      }
      var dbError = function(e) {
        console.log('error in insertLogMsg');
      }
    }
  })
  .service('logService', function($rootScope, $http, $q, LogSelectDbService) {
    this.sync = function() {
      var deferred = $q.defer();
      var syncProm = [];
      LogSelectDbService.getLogs().then(function(results) {
        if (results.length > 0) {
          results.forEach(function (logO) {
            syncProm.push(_sync(logO));
          });
        }
        $q.all(syncProm).then(function () {
          deferred.resolve(true);
        });

      });
      return deferred.promise;
    }
    var _sync = function(logO) {
      var deferred = $q.defer();
      sendLM(logO).then(function() {
        deferred.resolve(true);
      });
      return deferred.promise;
    }
    var sendLM = function(logO) {
      var deferred = $q.defer();
      $http.post($rootScope.graylogEndPoint , logO.logMsg,
                {timeout: 300000, warningAfter: 5000, ignoreAuthModule: true ,headers: {'Content-Type' : 'text/plain', 'Access-Control-Request-Headers' : undefined, 'X-CSRF-Token': undefined}, 'withCredentials' : false})
        .success(function (data, status, headers, config) {
          if (status == 202) {
            LogSelectDbService.deleteLogMsg(logO.id);
          }
          deferred.resolve();
        }).error(function (data, status, headers, config) {
          deferred.resolve();
        });
      return deferred.promise;
    }
  }).service('$log', ['$window', 'LogDbService', function($window, LogDbService) {
    var logToConsole = true;
    var logToServer = true;
    var logAll = true;
    var logLevels = {log : 6, debug : 7, warn : 4, info : 1, error : 3, custom : 6};
    var extraDef = {};

    this.setExtraDef = function(ibVersion, ibPackage) {
      extraDef.version = '1.1';
      extraDef.host = "BreedIT Mobile";
      extraDef._appVersion = ibVersion;
      extraDef._appPackage =  ibPackage;
      if ('device' in $window) {
        extraDef._deviceModel = $window.device.model;
        extraDef._devicePlatform = $window.device.platform;
        extraDef._deviceUUID = $window.device.uuid;
        extraDef._deviceVersion = $window.device.version;
      }
      else {
        extraDef._deviceModel = $window.navigator.userAgent;
        extraDef._devicePlatform = $window.navigator.platform;
        //@TODO deviceData.uuid = set once on localStorage from timestamp;
        extraDef._deviceUUID = 'yuval-pc';
        extraDef._deviceVersion = $window.navigator.appVersion;
      }
    }
    this.overrideExtraDef = function(data) {
      angular.forEach(data, function(value, key) {
        this[key] = value;
      }, extraDef);
    }
    var createLogMessage = function(msg, type, extra, longMessage) {
      var gelfMsg;
      try {
        extra = extra || {};
        longMessage = longMessage || false;
        var level = logLevels[type] || 1;
        var timestamp = Date.now() / 1000;
        gelfMsg = {'timestamp' : timestamp, 'version' : extraDef.version, 'host' : extraDef.host, 'short_message' : msg, 'level' : level};
        if (longMessage) {
          gelfMsg.full_message = longMessage;
        }
        angular.forEach(extraDef, function(value, key) {
          this[key] = value;
        }, gelfMsg);
        angular.forEach(extra, function(value, key) {
          this[key] = value;
        }, gelfMsg);
      }
      catch (e) {
        console.log(e);
        return false;
      }
      return gelfMsg;
    }
    var writeLogMessage = function(msg, type, extra, longMessage) {
      try {
        if (logToConsole) {
          if (type == 'error') {
            console.log(msg);
          }
        }
        if (logToServer) {
          if (logAll || type in logLevels) {
            var gelfMsg = createLogMessage(msg, type, extra, longMessage);
            if (gelfMsg) {
              // LogDbService.insertLogMsg(JSON.stringify(gelfMsg));
            }
          }
        }
      }
      catch (e) {
        console.log(e);
      }
    }
    this.log = function(msg, extra, longMessage) {
      writeLogMessage(msg, 'log', extra, longMessage);
    };
    this.debug = function(msg, extra, longMessage) {
      writeLogMessage(msg, 'debug', extra, longMessage);
    };
    this.info = function(msg, extra, longMessage) {
      writeLogMessage(msg, 'info', extra, longMessage);
    };
    this.warn = function(msg, extra, longMessage) {
      writeLogMessage(msg, 'warn', extra, longMessage);
    };
    this.error = function(e, extra) {
      console.log(e.stack);
      writeLogMessage(e.message, 'error', extra);
    };
    this.custom = function(msg, type, extra, longMessage) {
      type = type || 'custom';
      writeLogMessage(msg, type, extra, longMessage);
    }
  }]).factory('logToServerInterceptor', ['$q', '$log', '$rootScope', function ($q, $log, $rootScope) {
    return {
      'request': function (config) {
        if (config.url.indexOf('http') == -1 || config.url.indexOf($rootScope.graylogEndPoint) != -1) {
          return config;
        }
        config.msBeforeAjaxCall = new Date().getTime();

        var longMessageO = {};
        var longMessage;
        longMessageO.config = config;
        longMessage = JSON.stringify(longMessageO);
        $log.log("httpReq", {'_method' : config.method, '_url' : config.url, '_logGroup' : 'http'}, longMessage);
        return config;
      },
      'response': function (response) {
        try {
          if (response.config.url.indexOf('http') == -1 || response.config.url.indexOf($rootScope.graylogEndPoint) != -1) {
            return response;
          }
          var msAfterAjaxCall = new Date().getTime();
          var timeTakenInMs =
            msAfterAjaxCall - response.config.msBeforeAjaxCall;
          $log.log('httpReq success', {'_method' : response.config.method, '_url' : response.config.url, '_timeTakenInMs': timeTakenInMs, '_logGroup' : 'http'});
          if (response.config.warningAfter) {
            if (timeTakenInMs > response.config.warningAfter) {
              $log.warn('httpReq long request warn', {'_method' : response.config.method, '_url' : response.config.url, '_timeTakenInMs': timeTakenInMs, '_logGroup' : 'http'});
            }
          }
        } catch (e) {
          $log.error(e);
        }

        return response;
      },
      'responseError': function (rejection) {
        try {
          if (rejection.config.url.indexOf('http') == -1 || rejection.config.url.indexOf($rootScope.graylogEndPoint) != -1) {
            return $q.reject(rejection);
          }
          var msAfterAjaxCall = new Date().getTime();
          var timeTakenInMs =
            msAfterAjaxCall - rejection.config.msBeforeAjaxCall;
          var extra = {'_method' : rejection.config.method, '_url' : rejection.config.url, '_timeTakenInMs': timeTakenInMs, '_logGroup' : 'http'};
          var msg = 'http request error status ' + rejection.status;
          extra._status = status;
          if ('statusText' in rejection) {
            extra._statusText = rejection.statusText;
            msg += ' : ' + rejection.statusText;
          }
          var longMessageO = {};
          var longMessage;
          longMessageO.config = rejection.config;
          if ('data' in rejection && Array.isArray(rejection.data)) {
            longMessageO.data = rejection.data;
          }
          longMessage = JSON.stringify(longMessageO);
          if (rejection.status == 0) {
            extra._timeout = true;
          }
          $log.custom(msg, 'httpError', extra, longMessage);
        } catch (e) {
          $log.error(e);
        }
        return $q.reject(rejection);
      }
    };
  }]).factory('$exceptionHandler', ['$log', function($log) {
    return function(exception, cause) {
      try {
        if (!exception.processed || exception.processed == false) {
          exception.message += ' (caused by "' + cause + '")';
          exception.processed = true;
          $log.error(exception);
        }

      } catch (e) {
        console.log(e);
      }
      throw exception;
    };
  }]);

"use strict";
angular.module('ib.services')
  .service('RestService', function ($rootScope, $http, $filter, $q, $log, $cordovaNetwork, localStorageService, DbService, FieldObjFactory, $cordovaFile) {


    var JOB_STATUS_NEW = 0,
      JOB_STATUS_IN_PROG = 1,
      JOB_STATUS_COMPLETE = 2,
      JOB_STATUS_DONE = 3,
      JOB_STATUS_SYNC = 4,
      FIELD_STATUS_PEND = 0,
      FIELD_STATUS_IN_PROG = 1,
      FIELD_STATUS_COMP = 2;

    this.setHttp = function () {
      var token = localStorageService.get('token');
      if (token) {
        $http.defaults.headers.common['X-CSRF-Token'] = token;  // Step 1
        $http.defaults.headers.put['X-CSRF-Token'] = token;  // Step 1
      }
    }
    this.getFieldsList = function (id, fieldsList, data) {
      return getFieldsList(id, fieldsList, data);
    }


    var getFieldsList = function (id, fieldsList, data) {
      var processedList = [];
      fieldsList.forEach(function (fid) {
        var fod;
        if (fid == 0) {
          fod = FieldObjFactory.getFod(data, fid);
        }
        else {
          fod = FieldObjFactory.getChildFod(data, fid);
        }
        processedList.push([fid, id, JSON.stringify(FieldObjFactory.getObjectsList(data, fid, fod, null))]);
      });
      return processedList;
    }


    var getJobDataFromServer = function (url) {
      var deferred = $q.defer();

      try {
        var status = false;
        $http.get(url, {timeout: 300000, warningAfter: 5000, 'withCredentials': false, handleError: true})
          .success(function (data, status, headers, config) {
            var dataStr = JSON.stringify(data);
            console.log(data);
            console.log(dataStr);

            if ($rootScope.mode == "mobile") {
              $cordovaFile.writeFile(cordova.file.dataDirectory, data.jobNid + ".json", dataStr, true).then(function (result) {
                console.log(result);
                //Get file name
                insertJobData(deferred, data, "");
                // Success!
              }, function (err) {
                $log.error(err);
                deferred.resolve(false);
                // An error occured. Show a message to the user            });
              });
            }
            else {
              insertJobData(deferred, data, dataStr);
            }
          }).error(function (data, status, headers, config) {
          $log.warn("error in getJobDataFromServer data = '" + data + "' : status = '" + status + "'");
          deferred.resolve(false);
        })
      } catch (e) {
        deferred.resolve(true);
      }
      return deferred.promise;
    }


    var insertJobData = function (deferred, data, dataStr) {
      DbService.insertJob(data.jobNid, data.Date, dataStr, data.JobName, data.JobId,
        JSON.stringify(data.JobProperties), JOB_STATUS_NEW);
      var pFields = [];
      var parentsList = [];
      processJobFields(data.jobNid, data.ScoredObjectsList.ScoredObject, pFields, parentsList);
      DbService.insertJobFields(pFields, 0).then(function () {
        DbService.insertJobFieldsChildes(getFieldsList(data.jobNid, parentsList, data));
        DbService.insertJobForms(getFormsList(data.jobNid, data.ScObjDefList.ScObjDef, data.JobProperties.JobProperty));
        deferred.resolve(true);
      });

    }

    var getFormsList = function (id, fodList, propList) {
      var processedList = [];

      fodList.forEach(function (fod) {
        processedList.push([id, fod.ScObjDefId, JSON.stringify(FieldObjFactory.GetFormToCache(fod, id, false, propList)), 1]);
        processedList.push([id, fod.ScObjDefId, JSON.stringify(FieldObjFactory.GetFormToCache(fod, id, true, propList)), 2]);
      });
      return processedList;

    }

    this.getFormsList = function (id, fodList, propList) {
      return getFormsList(id, fodList, propList);
    }
    this.processJobFields = function (id, fields, pFields, parentsList) {
      return processJobFields(id, fields, pFields, parentsList);
    }
    var processJobFields = function (id, fields, pFields, parentsList) {
      if (fields) {
        for (var i = 0; i < fields.length; i++) {
          pFields.push([fields[i].ScObjDefId, fields[i].ObjectUniqueId, id, fields[i].ParentUniqueId, fields[i].Level, '']);
          if (parentsList.indexOf(fields[i].ParentUniqueId) == -1) {
            parentsList.push(fields[i].ParentUniqueId);
          }
          if ("NestedObjList" in fields[i]) {
            processJobFields([].concat(id), fields[i].NestedObjList.ScoredObject, pFields, parentsList);
          }
        }
      }
    }


    this.sendJobValuesToServer = function (nid) {
      var deferred = $q.defer();
      try {
        DbService.getJobValues(nid).then(function (jobValues) {
          DbService.getJobSyncedImages(nid).then(function (picValues) {
            var values = jobValues.rows.concat(picValues.rows);
            if (values.length > 0) {
              $http.put($rootScope.endPoint + '/appjob/' + nid + '.json', {rows: values}, {
                timeout: 300000,
                warningAfter: 5000
              })
                .success(function (data, status, headers, config) {
                  deferred.resolve(true);
                })
                .error(function (data, status, headers, config) {
                  $log.warn("error in sendJobValuesToServer data = '" + data + "' : status = '" + status + "'");
                  deferred.resolve(false);
                })
            }
            else {
              deferred.resolve(true);
            }
          });
        });
      } catch (e) {
        $log.error(e);
        deferred.resolve(false);
      }
      return deferred.promise;
    }

    this.getJobsFromServer = function () {
      var deferred = $q.defer();
      $http.get($rootScope.endPoint + '/appjob.json', {timeout: 300000, warningAfter: 5000})
        .success(function (data, status, headers, config) {
          var isUpdate = false;
          var hasData = false;
          var hasNeedToDelete = false;
          var urls;
          if (data.resStatus) {
            if (data.count > 0) {
              hasData = true;
              urls = data.urls;
            }
            if (data.deleteJobsCount > 0) {
              hasNeedToDelete = true;
            }

            DbService.getJobsIdList().then(function (results) {
              var updatedData = null;
              var prom = [];

              if (results.rows.length > 0) {
                if (hasData) {
                  //We have data on the server and on the client.
                  //Get the jobs that we have only on the server.
                  var diff = data.data.diff(results.rows);
                  if (diff.length > 0) {
                    updatedData = diff;
                  }
                }
              }
              else {
                if (hasData) {
                  //We have data only on the server
                  updatedData = data.data;
                }
              }
              if (updatedData != null) {
                isUpdate = true;
                updatedData.forEach(function (nid) {
                  prom.push(getJobDataFromServer(urls[nid]));
                });
              }

              if (hasNeedToDelete) {
                isUpdate = true;
                prom.push(DbService.deleteJobData(data.deleteJobs));
              }

              $q.all(prom).then(function () {
                deferred.resolve(isUpdate);
              });
            });
          }
          else {
            deferred.resolve(isUpdate);
          }
        })
        .error(function (data, status, headers, config) {
          $log.warn("error in getJobsFromServer data = '" + data + "' : status = '" + status + "'");
        });
      return deferred.promise;
    }

  })

angular.module('ib.services')
.service('SyncService', function($q, DbService, RestService, $http, $rootScope, $log, JobsFactory) {
    this.checkSyncLogic = function(connectionType, settings) {
      var deferred = $q.defer();
      if (connectionType == 1) { // WiFi
        // May sync
        deferred.resolve(1);
      }
      if (connectionType == -1) { // no connection
        // Do nothing
        //Off line sync
        deferred.resolve(-1);

      }
      else if (connectionType == 0) { // Connected, not WiFi
        DbService.isJobsHasImages().then(function(hasImages) {
          if (hasImages == true) {
            if (settings.sync.imageUploadNotify.value) {
              // Prompt the user about the connection type.
              deferred.resolve(0);
            }
            else {
              // No need for popup - may sync
              deferred.resolve(1);
            }
          }
          else { // No images to sync
            // May sync
            deferred.resolve(1);
          }
        });
      }
      return deferred.promise;
    }
    this.sync = function() {

      var deferred = $q.defer();
      var syncProm = [];
      var doneProm = [];

      DbService.getNeedSyncJobsIdList().then(function(results) {
        if (results.rows.length > 0) {
          results.rows.forEach(function (id) {
            syncProm.push(_sync(id));
          });
        }
        $q.all(syncProm).then(function () {
          DbService.getDoneJobsIdList().then(function(results) {
            if (results.rows.length > 0) {
              results.rows.forEach(function (id) {
                doneProm.push(_markJobAsDone(id));
              });
            }
            $q.all(doneProm).then(function () {
              deferred.resolve();
            });
          });
        });
      });
      return deferred.promise;
    }

    var _markJobAsDone = function(nid) {
      var deferred = $q.defer();
      $http.put($rootScope.endPoint + '/appjob/' + nid + '.json' , {status:3})
        .success(function (data, status, headers, config) {

          DbService.updateJobStatus(nid, 4);
          if (JobsFactory.job.id == nid) {
            JobsFactory.job.status = 4;
            $rootScope.$broadcast('jobs.jobUpdate');
          }
          deferred.resolve(true);
        })
        .error(function (data, status, headers, config) {
          $log.warn("error in _markJobAsDone data = '" + data + "' : status = '" + status + "'");
          deferred.resolve(false);
        });
      return deferred.promise;

    }

    var _sync = function(id) {
      var deferred = $q.defer();
      DbService.sendJobFieldsToServer(id).then(function(status) {
        if (status == true) {
          DbService.processJobImages(id).then(function(data){
            if (data.status == true) {
              RestService.sendJobValuesToServer(id).then(function(status) {
                if (status == true) {
                  DbService.updateJobSyncStatus(id, 1).then(function(status) {
                    deferred.resolve(status);
                  })
                }
                else {
                  deferred.resolve(false);
                }
              });
            }
            else {
              deferred.resolve(false);
            }
          });
        }
        else {
          deferred.resolve(false);
        }
      });
      return deferred.promise;
    }
})


'use strict';
angular.module('ib.services')
.service('UpdateService', function($rootScope, localStorageService, DbService, RestService, $q, $log, $cordovaFile) {
  var moveDataToFiles = function () {
    var deferred = $q.defer();
    if ($rootScope.mode != "mobile") {
      deferred.resolve();
    }
    var prom = [];

    DbService.selectJobsData().then(function(results){
      if (results.rows.length > 0) {
        var len = results.rows.length, i;
        for (i = 0; i < len; i++){
          prom.push($cordovaFile.writeFile(cordova.file.dataDirectory, results.rows.item(i).id + ".json", results.rows.item(i).data, true))
        }
      }
      $q.all(prom).then(function () {
        deferred.resolve();
        return;
      })
    })
    return deferred.promise;
  };
  var deleteCache = function () {
    var deferred = $q.defer();
    DbService.deleteFormsCache().then(function(){
      DbService.selectJobsData().then(function(results){
        if (results.rows.length > 0) {
          var len = results.rows.length, i;
          for (i = 0; i < len; i++){
            var data = JSON.parse(results.rows.item(i).data);
            var formsList = RestService.getFormsList(results.rows.item(i).id, data.ScObjDefList.ScObjDef, data.JobProperties.JobProperty);
            DbService.insertJobForms(formsList);

            var pFields = [];
            var parentsList = [];
            RestService.processJobFields(results.rows.item(i).id, data.ScoredObjectsList.ScoredObject, pFields, parentsList);
            var fieldsList = RestService.getFieldsList(results.rows.item(i).id, parentsList, data);
            DbService.insertJobFieldsChildes(fieldsList);
          }
        }

        deferred.resolve();
      });
    });
    return deferred.promise;
  };

  // Updates list.
  var Updates = {
    1: function() {},
    2: function() {},
    10: function() {
      return DbService.clearJobsAsync();
    },
    20: function() {

      return moveDataToFiles();
    }
  };

  this.run = function() {
    var updating = false;
    var deferred = $q.defer();
    var lastKey = localStorageService.get('update', 0),
        nextKey = false,
        prom = [];

    if (lastKey === 0) {
      // Install mode
      Object.keys(Updates).forEach(function(key){
        nextKey = Math.max(nextKey || 0, key);
        localStorageService.set('update', nextKey);
      });
    }
    else {
      // Update mode
      Object.keys(Updates).forEach(function(key){
        if (lastKey < key) {
          if (updating === false) {
            updating = true;
//            prom.push(deleteCache());
          }
          nextKey = Math.max(nextKey || 0, key);
          prom.push(Updates[key]());
          $log.log('run app update function number ' + key.toString(), {'upgrade number' : key.toString()});
        }
      });
    }
    if (nextKey) {
      localStorageService.set('update', nextKey);
    }
    $q.all(prom).then(function () {
      deferred.resolve();
    });

    return deferred.promise;
  };

});
