angular.module('ib.controllers')
  .controller('AppCtrl', function($scope, $ionicPlatform, $ionicModal, $ionicPopup, $filter, $interval, HelperService, DbService, SyncService, AuthenticationService, localStorageService, settings, $rootScope, $log, $templateCache, $http, $compile, FieldObjFactory, JobsFactory, gettextCatalog, $timeout, $cordovaFile) {
    $rootScope.$on('$stateChangeStart',
      function(event, toState, toParams, fromState, fromParams){
        $log.info("state changed enter time " + Date.now(), {'stateChangedStartTime' : Date.now(), fromState : fromState, toState: toState});
      });
    $scope.$on('$ionicView.afterEnter', function(){
      $log.info("ionicView.afterEnter " + Date.now(), {'startTime' : Date.now()});
    });
    $scope.$on('$ionicView.beforeLeave', function(){
      $log.info("ionicView.beforeLeave " + Date.now(), {'startTime' : Date.now()});
    });
    $scope.settings = settings;
    $ionicModal.fromTemplateUrl('templates/login.html', function(modal) {
        $scope.loginModal = modal;
      },
      {
        scope: $scope,
        animation: 'slide-in-up',
        focusFirstInput: true,
        backdropClickToClose: false,
        hardwareBackButtonClose: false
      }
    );
    //Be sure to cleanup the modal by removing it from the DOM
    $scope.$on('$destroy', function() {
      $scope.loginModal.remove();
    });

    $scope.$on('event:auth-logout-complete', function() {
      $scope.loginModal.show();
    });

    $scope.displayname = localStorageService.get('username');
    $scope.$on('event:auth-loginConfirmed', function() {
      $scope.displayname = localStorageService.get('username');
    });

    $scope.logout = function() {
      if ($scope.connectionType < 0) {
        $ionicPopup.alert({
          title: gettextCatalog.getString('You are offline.'),
          template: gettextCatalog.getString("You must be connected in order to log out."),
          okText: gettextCatalog.getString('OK'),
          okType: 'button-balanced'
        });
      }
      else {
        AuthenticationService.logout();
      }
    };

    $scope.login = function() {
      if (localStorageService.get('token')) {
        AuthenticationService.logout();
      }
      else {
        $scope.loginModal.show();
      }
    };

    $scope.clearJobs = function() {
      var confirmPopup = $ionicPopup.confirm({
        title: gettextCatalog.getString('Are you sure?'),
        template: gettextCatalog.getString("You are about to delete all data from the device.</br>Are you sure you want to proceed?"),
        okText: gettextCatalog.getString('CLEAR DATA'),
        okType: 'button-assertive'
      });
      confirmPopup.then(function (ok) {
        if (ok) {
          DbService.clearJobs();
          // LogDbService.createTables(true);

          $scope.$broadcast('data.clear');
        }
        else {
          // Do nothing
          return false;
        }
      });

    };

    // Sync
    $scope.isSyncing = false;
    $scope.isSyncingLog = false;
    var lastSyncDate = DbService.getLastSync();
    $scope.lastSync = lastSyncDate ? $filter('setTimeFormat')(lastSyncDate, 'date and time', 'short') : 'Never'; // 15:13 - 1/1/15

    var offLineSync = function() {


      try {
        DbService.getNeedSyncJobsIdList().then(function(results) {
          if (results.rows.length > 0) {
            results.rows.forEach(function (id) {
              DbService.getJobValues(id).then(function (jobValues) {
                DbService.getJobOfflineImages(id).then(function (picValues) {
                  $cordovaFile.writeFile(cordova.file.externalDataDirectory, id + ".json", JSON.stringify(jobValues.rows.concat(picValues.rows)), true).then(function(result) {
                  // Success!
                  }, function(err) {
                    $log.error(err);
                    // An error occured. Show a message to the user
                  });
                });
              });
            });
          }
        });
      }
      catch (e) {
        $log.error(e);
      }
    }

    var syncNow = function () {
      return;
      //@TODO delete the code since we sync in the BG...

      if ($scope.isSyncing == false) {
        $scope.isSyncing = true;
        SyncService.sync().then(function() {
          $scope.isSyncing = false;
          $scope.lastSync = $filter('setTimeFormat')(DbService.setLastSync(), 'date and time', 'short'); // 15:13 - 1/1/15
        });
      }
    };

    function updateOnlineStatus() {
      $scope.connectionType = HelperService.checkConnectionMode();
      if ($scope.connectionType > -1) {
        $scope.enableSync = true;
      }
      else {
        $scope.enableSync = false;
      }
    }
    // Watch for changes
    updateOnlineStatus();
    window.addEventListener('online',  updateOnlineStatus);
    window.addEventListener('offline', updateOnlineStatus);
    $scope.syncLog = function() {
      if ($scope.isSyncingLog == false && HelperService.checkConnectionMode() != -1) {
        // $scope.isSyncingLog = true;
        // logService.sync().then(function() {
        //   $scope.isSyncingLog = false;
        // });
      }
    }
      $scope.syncData = function(mode) {
      $scope.connectionType = HelperService.checkConnectionMode();
      if ($scope.isSyncing) {
        return;
      }
      mode = mode || false;
      SyncService.checkSyncLogic($scope.connectionType, settings).then(function(res) {
        if (res == -1) {
          offLineSync();
          return false;
        }
        else if(res == 1) {
          syncNow();
        }
        else if(res == 0) {
          if (mode == 'manual') {
            var confirmPopup = $ionicPopup.confirm({
              title: gettextCatalog.getString('No WiFi Connection'),
              template: gettextCatalog.getString("You are about to upload images.</br>Are you sure you want to sync?"),
              okText: gettextCatalog.getString('SYNC'),
              okType: 'button-balanced'
            });
            confirmPopup.then(function (ok) {
              if (ok) {
                // May sync
                syncNow();
              }
              else {
                // Do nothing
                return false;
              }
            });
          }
          return false;
        }
      })
    };

    // Auto-Sync
    if (settings.sync && settings.sync.autoSyncInterval && settings.sync.autoSyncInterval.value > 0) {
      var interval = (settings.sync.autoSyncInterval.value * 1000 * 60);
      $interval($scope.syncData, interval);
    };

    //@TODO wrap with settings check
    $interval($scope.syncLog, 1000 * 60);
  })
