angular.module('ib.controllers').controller('ChildListCtrl', function($scope, $state, $stateParams, $filter, $window, $ionicPopup, $ionicLoading, $cordovaBarcodeScanner, FieldObjFactory, $location, $ionicTabsDelegate, $ionicHistory, $ionicScrollDelegate, $ionicModal, GotoService, gettextCatalog, DbService, JobsFactory, Paginator, $timeout, $ionicPosition) {

  $scope.rowsPerPage = 20;
  // 1. Filter
  $scope.$on('childList.refilter', function() {
    // Reset
    FieldObjFactory.setFieldsStatus(FieldObjFactory.objs, $scope.fieldsStatus);
    FieldObjFactory.set('filteredObjects', FieldObjFactory.objs);
    // Filter to show only pending
    var predicate;
    if ($scope.showOnlyPending) {
      predicate = {statusClass: 'pending'};
      FieldObjFactory.set('filteredObjects', $filter('filter')(
        FieldObjFactory.filteredObjects, predicate, true
      ));
    }
    // Filter by title only / by any field-status
    if ($scope.searchByTitleOnly) {
      predicate = {title: $scope.search.title};
      FieldObjFactory.set('filteredObjects', $filter('filter')(
        FieldObjFactory.filteredObjects, predicate
      ));
    }
    else {
      FieldObjFactory.set('filteredObjects', $filter('filter')(
        FieldObjFactory.filteredObjects, $scope.search
      ));
    }
    $scope.$broadcast('childList.reorder');
  });

  // 2. Order by...
  $scope.$on('childList.reorder', function() {
    if ($scope.predicate) {
      FieldObjFactory.set('orderedObjects', $filter('orderBy')(FieldObjFactory.filteredObjects, $scope.predicate, $scope.reverse));
    }
    else {
      FieldObjFactory.set('orderedObjects', FieldObjFactory.filteredObjects);
    }
    $scope.limitedObjects = FieldObjFactory.filteredObjects;
    $scope.$broadcast('childList.finishedOrdering');
  });

  // 3. Statuses
  $scope.$on('childList.finishedOrdering', function() {
    $scope.limitedObjects = FieldObjFactory.setFieldsStatus(FieldObjFactory.orderedObjects, $scope.fieldsStatus);

    if ($scope.tableView.show) {
      $scope.tableViewHeaderUpdate();
    }
  });

  // Update cached view
  $scope.$on('$ionicView.beforeEnter', function(scopes, states) {
    if (states.fromCache) {
      DbService.getFieldsStatus($scope.objId).then(function(fs) {
        $scope.fieldsStatus = fs;
        $scope.$broadcast('childList.refilter');
      });
    }
  });

  $scope.tableViewHeaderUpdate = function() {
    var pds = [];

    $scope.tableViewFieldsMap.forEach(function (item) {
      if (item.showInTableView) {
        pds.push(item.id);
      }
    });
    FieldObjFactory.getObjectsInfoForTableView(JobsFactory.job.data, $scope.limitedObjects, pds);

    // Save to localStorage.
    var level = $scope.objMeta ? $scope.objMeta.Type : 'job';
    FieldObjFactory.setTableViewConfig($scope.jobId, level, pds);

  };

  function initTableView() {

    var level = $scope.objMeta ? $scope.objMeta.Type : 'job';
    var pds = FieldObjFactory.getTableViewConfig($scope.jobId, level);
    $scope.tableView = $scope.tableView || {};
    $scope.tableView.show = FieldObjFactory.getTableViewStatus($scope.jobId, level);
    $scope.tableView.maxHeight = ($window.innerHeight - 300) + 'px';

    if (pds != []) {
      $scope.tableViewFieldsMap.forEach(function (item) {
        if (pds.indexOf(item.id) >= 0) {
          item.showInTableView = true;
        }
      });
    }

  }

  var viewId;

  function showActiveItem(newViewId) {

    // Polyfill Array.prototype.findIndex()
    if (!Array.prototype.findIndex) {
      Array.prototype.findIndex = function(predicate) {
        'use strict';
        if (this == null) {
          throw new TypeError('Array.prototype.findIndex called on null or undefined');
        }
        if (typeof predicate !== 'function') {
          throw new TypeError('predicate must be a function');
        }
        var list = Object(this);
        var length = list.length >>> 0;
        var thisArg = arguments[1];
        var value;

        for (var i = 0; i < length; i++) {
          value = list[i];
          if (predicate.call(thisArg, value, i, list)) {
            return i;
          }
        }
        return -1;
      };
    }

    var scrollToActiveItem = function() {
      var activeElement = document.querySelector('.view-id-' + viewId + ' .active-item');
      if (activeElement) {
        var activePosition = $ionicPosition.position(angular.element(activeElement));
        var scrollHandle = $scope.tableView.show ? 'tableViewScroll' : 'childListScroll';
        if ($scope.tableView.show) {
          // Table-view
          $ionicScrollDelegate.$getByHandle('tableViewScroll').scrollTo(0, activePosition.top, true);
          // Scroll wrapping view to top
          $ionicScrollDelegate.$getByHandle('childListScroll').scrollTop(true);
        }
        else {
          // List view
          $ionicScrollDelegate.$getByHandle('childListScroll').scrollTo(0, activePosition.top, true);
        }
      }
    };

    // Use last viewId when got nothing.
    viewId = newViewId ? newViewId : viewId;

    // Find active item's page
    var activeIndex = $scope.limitedObjects.findIndex(function(item) {
      return item.id == $scope.activeItem;
    });
    if (activeIndex > -1) {
      var activePage = Math.floor(activeIndex / Paginator.rowsPerPage);
      if (activePage > -1) {
        // Wait for the pager to load
        $timeout(function() {
          Paginator.setPage(activePage);
          // Wait for the right page to load
          $timeout(function() {
            scrollToActiveItem();
          });
        });
      }
      else {
        // Wait for the page to load
        $timeout(function() {
          scrollToActiveItem();
        });
      }
    }
  }

  function initActiveItem() {
    $scope.activeItem = FieldObjFactory.getActiveItem($scope.jobId, $scope.objId || 0);
    if ($scope.activeItem) {
      $scope.$on('$ionicView.enter', function(event, data) {
        if(event.targetScope !== $scope) {
          return;
        }
        showActiveItem(event.targetScope.$id);
      });
    }
  }

  function initScope() {
    //$scope.buffer = 12;
    //$scope.limit = $scope.buffer;

    $scope.sortSelect = [
      {value: 'none', name: gettextCatalog.getString('Sort By...')},
      {value: 'title', name: gettextCatalog.getString('Title')}
    ];
    $scope.propList.forEach(function(prop, index) {
      $scope.sortSelect.push({value: 'properties[' + index + '].Value', name: prop});
    });
    $scope.storedSort = FieldObjFactory.getSort($scope.jobId, $scope.objId || 0) || {};
    $scope.predicate = $scope.storedSort.predicate || '';
    $scope.reverse = $scope.storedSort.reverse || false;
    $scope.sort = $scope.predicate || 'none';
    $scope.search = {};
    $scope.search.$ = $scope.storedSort.filter || '';
    if ($scope.storedSort.fav) {
      $scope.search.fav = true;
    }
    $scope.search.title = $scope.storedSort.titleOnly || '';
    $scope.searchByTitleOnly = !!$scope.storedSort.titleOnly || false;
    $scope.showOnlyPending = $scope.storedSort.showOnlyPending || false;
    $scope.childListFilters = {show: false};
    initTableView();
    initActiveItem();
    // Enable load more.
    //$scope.moreDataCanBeLoaded = true;
    //if ($scope.limit >= $scope.objLength  ) {
    //  $scope.moreDataCanBeLoaded = false;
    //}
    // Filter
    $scope.$broadcast('childList.refilter');

  }

  initScope();

  $scope.$on('objctrl.update', function() {
    initScope();
    $scope.$emit('childobj.updated');
  });

  // Scroll to top when clicking bottom pager.
  $scope.bottomPagerScrollTop = function($event) {
    var el = $event.srcElement,
        enabled = false,
        aEl;
    // Iterate over element & parents (a-la jQuery.closest())
    // Check if it's an enabled pager item.
    while (el !== null) {
      aEl = angular.element(el);
      if (aEl.hasClass('bottom-pager')) {
        // Reached pager wrapper and didn't find active element.
        break;
      }
      else if (aEl.attr('ng-click') && !aEl.hasClass('disabled') && !aEl.hasClass('active')) {
        // Found active element. break.
        enabled = true;
        break;
      }
      el = el.parentElement;
    }
    // Scroll only if the clicked item is enabled
    if (enabled) {
      $ionicScrollDelegate.scrollTop(true);
    }
  };

  $scope.goToWithLoading = function(objId, parentUniqueId) {
    GotoService.goTo(parentUniqueId, objId, "edit");
  };

  // $scope.scanQR = function() {
  //   $cordovaBarcodeScanner
  //     .scan()
  //     .then(function(barcodeData) {
  //       var res = $filter('filter')(FieldObjFactory.objs, barcodeData.text);
  //       var keys = Object.keys(res);
  //       if (keys.length == 1) {
  //         GotoService.goTo($scope.objId || 0, res[0].id, "edit");
  //       }
  //
  //       $scope.search.$ = barcodeData.text;
  //
  //     }, function(error) {
  //       $ionicPopup.alert({
  //         title: gettextCatalog.getString('Something went wrong'),
  //         template: gettextCatalog.getString('Unable to scan the QR code.')
  //       });
  //   });
  // };

  // Search/Favorite filter
  $scope.filterList = function(prop) {
    if ($scope.objs2delete) {
      $scope.$broadcast('childList.resetDelete');
    }
    // Favorite (toggle)
    if (prop == 'fav') {
      if ($scope.search.fav) {
        delete $scope.search.fav;
      }
      else {
        $scope.search.fav = true;
      }
    }
    // Filter
    else if (prop == 'filter') {
      // Nothing special to do here, just save.
    }
    FieldObjFactory.setSort($scope.jobId, $scope.objId || 0, $scope.predicate, $scope.reverse, $scope.search, $scope.search.title, $scope.showOnlyPending);
    $scope.$broadcast('childList.refilter');
    $timeout(function() {
      showActiveItem();
    });
  };

  // Reset search
  $scope.resetSearch = function() {
    $scope.search.$ = '';
    $scope.search.title = '';
    $scope.childListFilters.show = false;
    $scope.filterList('filter');
  };

  // Sort
  $scope.sortList = function (prop) {
    if ($scope.objs2delete) {
      $scope.$broadcast('childList.resetDelete');
    }
    if (prop == 'reverse') {
      $scope.reverse = !$scope.reverse;
    }
    // Sort term (select box)
    else if (prop == 'none') {
      $scope.predicate = '';
    }
    else {
      $scope.predicate = prop;
    }
    FieldObjFactory.setSort($scope.jobId, $scope.objId || 0, $scope.predicate, $scope.reverse, $scope.search, $scope.search.title);
    $scope.$broadcast('childList.reorder');
  };

  $scope.getItemStatus = function(status) {
    var statusStr;
    switch (status) {
      case "pending":
        statusStr = gettextCatalog.getString('PENDING');
        break;
      case "inProgress":
        statusStr = gettextCatalog.getString('IN PROGRESS');
        break;
      case "completed":
        statusStr = gettextCatalog.getString('COMPLETED');
        break;
      case "done":
      case "sync":
        statusStr = gettextCatalog.getString('DONE');
        break;
      default:
        statusStr = gettextCatalog.getString('PENDING');
    }
    return statusStr;
  };

  // Table view
  $scope.tableViewToggleState = function() {
    $scope.childListFilters.show = false;
    if ($scope.tableView.show === true) {
      $scope.tableViewShowMenu();
      $scope.tableViewHeaderUpdate();
    }

    // Save to localStorage.
    var level = $scope.objMeta ? $scope.objMeta.Type : 'job';
    FieldObjFactory.setTableViewStatus($scope.jobId, level, $scope.tableView.show);

    showActiveItem();
  };

  $scope.tableViewShowMenu = function() {
    $ionicModal.fromTemplateUrl('templates/table-view-menu.html', function(modal) {
      $scope.tableViewMenu = modal;
      $scope.tableViewMenu.show();
    }, {
      scope: $scope,
      animation: 'slide-in-up',
      backdropClickToClose: false
    });
  };
  // close video modal
  $scope.tableViewCloseMenu = function() {
    $scope.tableViewMenu.hide();
  };
  $scope.$on('modal.hidden', function() {
  });
  $scope.$on('$destroy', function() {
    if ($scope.tableViewMenu) {
      $scope.tableViewMenu.remove();
    }
  });

  $scope.checkAll = false;
  $scope.tableViewMenuToggleCheckAll = function(checkAll) {
    $scope.checkAll = checkAll;

    $scope.tableViewFieldsMap.forEach(function (item) {
      item.showInTableView = checkAll;
    });
    $scope.tableViewFieldsMap.showInTableView = checkAll;

    $scope.tableViewHeaderUpdate();
  };

});
