angular.module('ib.controllers')
.controller('FormController', function($stateParams, $scope, DbService, FieldObjFactory, HelperService, $log, JobsFactory, $ionicModal, $ionicPopup, $ionicPosition, $ionicScrollDelegate, $cordovaBarcodeScanner, gettextCatalog) {
  $log.info("FormController enter time " + Date.now(), {'controllerStartTime' : Date.now()});

  // Handling form ng-changes and JS changes via event 'form.updated'
  $scope.updated = function(modelValue, form) {

    if (angular.isDefined($scope.quickEditAutoAdvance)) {
      $scope.quickEditAutoAdvance(form);
    }
    if (angular.isUndefined(modelValue)) {
      return;
    }
    if ($scope.jobDone) {
      return;
    }
    form.feedback = validateValue(modelValue, form);
    if (form.feedback && !form.allowSave) {
      return;
    }
    var insertValue = modelValue;
    if (form.type && form.type == 'date') {
      insertValue = Date.parse(modelValue) / 1000;
    }
    else if (form.type && form.type == 'select') {
      form.displayValue = form.titleMap.find(function(item){return item.value == modelValue}).name
    }
    DbService.insertJobValue($scope.jobId, $scope.formFieldInfo.fid, form.pd, Date.now() / 1000, insertValue, $scope.formFieldInfo.pfid, $scope.formFieldInfo.userFid, $scope.formFieldInfo.odefId).then(function() {

      FieldObjFactory.checkField(JobsFactory.job, $scope.formFieldInfo.fid, FieldObjFactory.getFod(JobsFactory.job.data, $scope.objId)).then(function(status) {
        $scope.currObj.statusClass = HelperService.getStatusClass(status);
      });
    });
  };

  $scope.$on('form.updated', function(event, args) {
    $scope.updated(args.modelValue, args.form);
  });

  // Validate form elements
  function validateValue(val, form) {

    // Required
    if (form.required && !val && val !== 0) {
      form.allowSave = true; // Save invalid required value (Feature #550).
      return gettextCatalog.getString("This field is required");
    }

    switch(form.type){
      case "number":
      case "range":
      case "tel":

        // Type
        if (isNaN(val)) {
          return gettextCatalog.getString("Please enter a valid number");
        }

        // Precision (float/integer)
        var precisionRegex;
        if (form.precision) { // Float decimal precision
          precisionRegex = new RegExp("^\\s*-?(\\d+\\.?\\d{0," + form.precision + "})?\\s*$", "g");
          if (!precisionRegex.test(val)) {
            return gettextCatalog.getString("Please enter up to {{precision}} decimal places", { precision: form.precision });
          }
        }
        else { // Integer
          precisionRegex = new RegExp("^\\s*-?\\d*\\s*$", "g");
          if (!precisionRegex.test(val)) {
            return gettextCatalog.getString("Integers only please");
          }
        }

        // Minimum value
        if (!isNaN(form.minimum) && val < form.minimum) {
          return gettextCatalog.getString("The minimum value is {{minimum}}", { minimum: form.minimum });
        }

        // Maximum value
        if (!isNaN(form.maximum) && val > form.maximum) {
          return gettextCatalog.getString("The maximum value is {{maximum}}", { maximum: form.maximum });
        }
        break;
      case "string":
        // Max length
        if (!isNaN(form.maxLength) && val.length > form.maxLength) {
          return gettextCatalog.getString("Only {{maxLength}} characters are allowed", { maxLength: form.maxLength });
        }

        break;
    }

    return false; // Valid!
  }

  $scope.toggleKeyboard = function(event, form) {
    // Hide other keyboards.
    $scope.$emit('keyboard.hideOthers', {id: form.pd});
    // Register once, hide keyboard when other one opens.
    if (!form.showKeyboard && !form.keyboardRegisterEvent) {
      $scope.$on('keyboard.hideOthers', function(event, args) {
        if (args.id !== form.pd) {
          form.showKeyboard = false;
        }
        form.keyboardRegisterEvent = true;
      });
    }
    // Toggle
    form.showKeyboard = form.showKeyboard ? false : true;

    // Scroll to show keyboard if not visible.
    if (form.showKeyboard) {
      var formItem = angular.element(event.srcElement.parentNode),
          offset = $ionicPosition.offset(formItem);

      if (offset.top + offset.height + 50 > window.innerHeight) {
        var scrollAmount = offset.top + offset.height - window.innerHeight + 60;
        $ionicScrollDelegate.$getByHandle('formContent').scrollBy(0, scrollAmount, true);
      }
    }
  }

  $scope.hasHistoryBtn = function(form) {
    // True if there's a history or an oldVal (not empty).
    return form.historyList || form.oldval;
  }

  $scope.insertOldVal = function(form) {
    if ($scope.jobDone) {
      return false;
    }
    switch(form.type){
      case "select":
        for (var i = 0; i < form.titleMap.length; i++) {
          if (form.titleMap[i].name == form.oldval) {
            form.currVal = form.titleMap[i].value;
            break;
          }
        }

        break;
      case "boolean":
        form.currVal = form.oldval == '-' + gettextCatalog.getString('True') + '-' ? true : false;
        break;
      default:
        form.currVal = form.oldval;
        break;
    }
    $scope.$emit('form.updated', { modelValue: form.currVal, form: form });
  }

  // Image select modal
  $scope.imageSelectModal = function(form) {
    $ionicModal.fromTemplateUrl('templates/image-select-modal.html', function(modal) {
      $scope.gridModal = modal;
      if (form && form.imagesList) {
        // ToDo: Unify the structure of the form element & info element
        $scope.imagesTitle = form.title;
        $scope.imagesList = form.imagesList;
        $scope.gridModal.show();
      }
    }, {
      scope: $scope,
      animation: 'slide-in-up'
    });

    $scope.imageSelect = function(image) {
      form.currVal = image.title;
      // ToDo: validate and save.
      $scope.$emit('form.updated', { modelValue: image.value, form: form });
    };

  };
  // close video modal
  $scope.closeImagesModal = function() {
    $scope.gridModal.hide();
  };

  $scope.$on('$destroy', function() {
    if ($scope.gridModal) {
      $scope.gridModal.remove();
    }
  });

  // Image select - quick edit
  $scope.quickEditImageSelect = function(image, form) {
    form.currVal = image.title;
    // ToDo: validate and save.
    $scope.$emit('form.updated', { modelValue: image.value, form: form });
  };

  // Barcode scanner
  $scope.scanQR = function(form) {
    $cordovaBarcodeScanner
      .scan()
      .then(function(barcodeData) {
        form.currVal = barcodeData.text;
        $scope.$emit('form.updated', { modelValue: barcodeData.text, form: form });
      }, function(error) {
        $ionicPopup.alert({
          title: gettextCatalog.getString('Something went wrong'),
          template: gettextCatalog.getString('Unable to scan the code.')
        });
    });
  };

});
