angular.module('ib.controllers')
.controller('JobsCtrl', function($scope, $rootScope, $ionicPlatform, $timeout, $state, $ionicPopup, $ionicLoading, JobsFactory, HelperService, jobs, $filter, gettextCatalog, DbService) {
  $scope.jobs = jobs;
  $scope.jobsCount = Object.keys($scope.jobs).length || 0;
  $scope.jobs2delete = 0;

  $scope.$on('$stateChangeSuccess', function() {
    $ionicLoading.hide();
  });

  $scope.goToWithLoading = function(jobId) {
    $ionicLoading.show();
    $state.go('app.job.child', {
      jobId: jobId
    });
  }

  $scope.getJobStatus = function(status) {
    var statusStr;
    switch (status) {
      case "pending":
        statusStr = gettextCatalog.getString('PENDING');
        break;
      case "inProgress":
        statusStr = gettextCatalog.getString('IN PROGRESS');
        break;
      case "completed":
        statusStr = gettextCatalog.getString('COMPLETED');
        break;
      case "done":
      case "sync":
        statusStr = gettextCatalog.getString('DONE');
        break;
      default:
        statusStr = gettextCatalog.getString('PENDING');
    }
    return statusStr;
  }

  $scope.formatTime = function(timestamp) {
    var date = new Date(timestamp * 1000);
    return date.toDateString();
  };

  $scope.toggleItemDeletion = function(jobId) {
    var job = $scope.jobs.filter(function( job ) {
      return job.id == jobId;
    })[0];
    if (job) {
      // Flag it
      job.toDelete = job.toDelete ? false : true;
      // Counter
      job.toDelete ? $scope.jobs2delete++ : $scope.jobs2delete--;
    }
  };

  function resetDelete() {
    $scope.jobs.filter(function( job ) {
      return job.toDelete === true;
    }).forEach(function(job) {
      delete job.toDelete;
    });
    $scope.jobs2delete = 0;
  }
  // $scope.$on('job.resetDelete', function() {
  //   resetDelete();
  // });

  $scope.deleteJobs = function() {
    $scope.popup = {
      done: false,
      subTitle: gettextCatalog.getString('THIS ACTION IS IRREVERSIBLE'),
      description: false
    };
    $ionicPopup.show({
      title: gettextCatalog.getString('Delete') + ' ' + ($scope.jobs2delete > 1 ? gettextCatalog.getString('Jobs') : gettextCatalog.getString('Job')),
      subTitle: $scope.popup.subTitle,
      template: '<ion-checkbox class="checkbox-assertive" ng-model="popup.done">' + gettextCatalog.getString('I understand.') + '</ion-checkbox><div class="assertive" ng-show="popup.description">' + gettextCatalog.getString('Please confirm') + '</div>',
      scope: $scope,
      buttons: [{
        text: gettextCatalog.getString('DELETE'),
        type: 'button-assertive',
        onTap: function(e) {
          if (!$scope.popup.done) {
            e.preventDefault();
            $scope.popup.description = true;
          } else {
            $scope.jobs.filter(function( job ) {
              return job.toDelete === true;
            }).forEach(function(job) {
              DbService.deleteJobData([job.id]);
              job.deleted = true;
              delete job.toDelete;
            });
            resetDelete();
          }
        }
      }, {
        text: gettextCatalog.getString('CANCEL'),
        type: 'button-default'
      }]
    });
  };

  $scope.doRefresh  = function() {
    if (HelperService.isOnlineCheck()) {
      JobsFactory.refreshJobs().then(function(result) {
        if (result == true) {
          JobsFactory.getJobs().then(function(results) {
            $scope.jobs = results;
            $scope.jobsCount = Object.keys($scope.jobs).length || 0;
            $scope.$broadcast('scroll.refreshComplete');
          });
        }
        else {
          $scope.$broadcast('scroll.refreshComplete');
        }
      });
    }
    else {
      //@TODO: alert the user.
      $scope.$broadcast('scroll.refreshComplete');
      $ionicPopup.alert({
        title: gettextCatalog.getString('No internet connection'),
        okType: 'button-balanced'
      });
    }
  }
  if (JobsFactory.init == false) {
    if (HelperService.isOnlineCheck()) {
      $scope.doRefresh();
    }
  }
  $scope.$on('data.clear', function() {
    $scope.jobs = {};
    $scope.jobsCount = 0;
  });

  // Back button functionality.
  var deregister = $ionicPlatform.registerBackButtonAction(
    function(e) {
      if($state.is('app.jobs')) {
        var confirmPopup = $ionicPopup.confirm({
          title: gettextCatalog.getString('Confirm Exit'),
          template: gettextCatalog.getString("Are you sure you want to quit?"),
          okText: gettextCatalog.getString('QUIT'),
          okType: 'button-balanced'
        });
        confirmPopup.then(function (close) {
          if (close) {
            // Close the app.
            ionic.Platform.exitApp();
          }
          else {
            // Do nothing
            e.preventDefault();
            return false;
          }
        });
      }
    }, 101
  );
  $scope.$on('$destroy', deregister);

})
