'use strict';
angular.module('ib.directives')
.directive('deletable', function($document, $parse, $compile, gettextCatalog) {
  return {
    restrict: 'A',
    link: function(scope, element, attrs) {

      var startX, x, animationInterval, undo, obj;

      // Bind to 'dragstart'.
      element.on('dragstart', drag);
      scope.$on('$destroy', function() {
        element.off('dragstart', drag);
      });

      function drag() {
        // (child list) Prevent on 'Done' jobs.
        if (scope.jobDone) {
          return;
        }
        obj = $parse(attrs.deletable)(scope);
        // (child list) Bail if NOT isNewItem.
        // We use obj.title to differentiate jobs & plots/plants, jobs don't have 'title'
        if (obj.title && !obj.isNewItem) {
          return;
        }
        // (job list) Allow only done jobs
        // We use obj.jobName to differentiate jobs & plots/plants, only jobs have 'jobName'
        if (obj.jobName && obj.status != 3) {
          return;
        }
        // Prevent default dragging if scrolling down.
        if (event.gesture.velocityX < event.gesture.velocityY) {
          return;
        }
        event.stopPropagation();
        event.gesture.preventDefault();
        // Init.
        x = 0;
        // Set starting point.
        startX = event.gesture.center.pageX - x;
        // Move while dragging.
        $document.on('drag', function(event) {
          x = event.gesture.center.pageX - startX;
          // 5px threshold
          if (Math.abs(x) > 10) {
            move(x);
          }
          else if (x !== 0) {
            move(0);
          }
        });
        $document.on('dragend', release);
      }

      // Move item using css transform.
      function move(x) {
        element.css({
          '-webkit-transform':  'translate3d(' + x + 'px, 0, 0)',
          'transform':  'translate3d(' + x + 'px, 0, 0)',
          'opacity': Math.max(1 - Math.abs(x/200), 0),
        });
      }

      // Animate move().
      function animate(n) {
        animationInterval = setInterval(function() {
          x = x + n;
          if (10 > Math.abs(x)) {
            // Item back in center.
            move(0);
            clearInterval(animationInterval);
            return false;
          }
          else if (Math.abs(x) > window.innerWidth) {
            // Item is outside.
            toBeDeleted();
            clearInterval(animationInterval);
            return false;
          }
          else {
            // Keep animating.
            move(x);
          }
        }, 1);
      }

      // Release handler
      function release() {
        // ToDo: CHECK WHY THESE DON'T WORK
        // $document.off('drag', move);
        // $document.off('dragend', release);
        // THESE WORK BUT REMOVE ALL OF THE BINDINGS OF $document FOR 'drag' and 'dragend'
        $document.off('drag');
        $document.off('dragend');

        var diff;
        if (x > 100 || x < -100) {
          // Mark for deletion.
          diff = x > 0 ? 3 : -3;
          animate(diff);
        }
        else {
          // If not passed threshold animate it back.
          diff = x > 0 ? -2 : 2;
          // Limit x so we don't wait for it too much.
          x = Math.min(Math.max(parseInt(x), -100), 100);
          animate(diff);
        }
      }

      // Mark for deletion.
      function toBeDeleted() {
        undo = angular.element('<div ng-click="undoDelete(' + obj.id + ')" class="undo"><span class="ion-ios-undo">' + gettextCatalog.getString('UNDO') + '</span></div>');
        $compile(undo)(scope);
        scope.toggleItemDeletion(obj.id);
        element
          .append(undo)
          .css({
            '-webkit-transform':  'translate3d(0, 0, 0)',
            'transform':  'translate3d(0, 0, 0)',
            'opacity': 1,
          })
          .removeAttr('style');
        scope.$apply();
      }

      function removeUndo() {
        if (undo) {
          undo.remove();
        }
      }

      scope.undoDelete = function(objId) {
        undo.remove();
        scope.toggleItemDeletion(objId);
      };

      scope.$on('obj.updateLists', removeUndo);
      scope.$on('childList.reorder', removeUndo);
    }
  };
});
