angular.module('ib.directives')
.directive('itemFind', function($ionicPopup, gettextCatalog, FieldObjFactory, JobsFactory, $filter, GotoService, $ionicLoading, $cordovaBarcodeScanner) {

  var itemFindController = function($scope) {

    var findItemPopup;

    // Find item popup
    $scope.findItemPopup = function() {

      $scope.popup = {
        itemToFind: ''
      };

      $scope.scanQR = function() {
        $cordovaBarcodeScanner
          .scan()
          .then(function(barcodeData) {

            $scope.popup.itemToFind = barcodeData.text;
            $scope.findItem($scope.popup.itemToFind);

          }, function(error) {
            $ionicPopup.alert({
              title: gettextCatalog.getString('Something went wrong'),
              template: gettextCatalog.getString('Unable to scan the QR code.')
            });
        });
      };

      findItemPopup = $ionicPopup.show({
        title: gettextCatalog.getString('Find Item'),
        template: '<div class="find-input-group"><button ng-click="scanQR()" type="button" ng-hide="popup.itemToFind" class="button scan-btn button-small button-icon icon ion-camera"></button><button ng-click="popup.itemToFind = \'\'" type="button" ng-show="popup.itemToFind" class="button btn-clear-search button-small button-icon ion-android-close"></button><input type="text" placeholder="{{\'Find Item...\'|translate}}" ng-model="popup.itemToFind"><div class="assertive">{{findItemPopupMsg}}</div></div>',
        scope: $scope,
        buttons: [{
          text: gettextCatalog.getString('FIND'),
          type: 'button-balanced',
          onTap: function (e) {
            e.preventDefault();
            $scope.findItem($scope.popup.itemToFind);
          }
        }, {
          text: gettextCatalog.getString('CANCEL'),
          type: 'button-default'
        }]
      });

    };

    // Find Item and scroll to it.
    $scope.findItem = function(item) {
      try {
        var obj;

        var goToObject = function(parent, obj) {
          if (angular.isDefined(obj)) {
            if (parent == $scope.parentId && obj == $scope.objId) {
              findItemPopup.close();
            }
            else {
              findItemPopup.close();
              $scope.$broadcast('objctrl.transition');
              $scope.loadedViewClass = '';
              $ionicLoading.show();
              GotoService.goTo(parent || 0, obj, $scope.tab || 'edit', 'trans');
            }
          }
        };

        var compareStr = function (actual, expected) {
          return angular.equals(actual.toString(), expected)
        }

        var searchTopLevel = function(item) {
          // Search for a top-level item by name.
          var objs = JobsFactory.job.data.ScoredObjectsList.ScoredObject;
          obj = $filter('filter')(objs, {"ObjectName": item}, compareStr)[0];
          if (angular.isDefined(obj)) {
            goToObject(obj.ParentUniqueId || 0, obj.ObjectUniqueId);
          }
          else {
            $scope.findItemPopupMsg = "Could not find " + item;
          }
        }

        var globalSearch = item.split('::');
        // Global search will search (strict) the whole job.
        // Local search will search (fuzzy) only current level.
        if (globalSearch.length > 1) {
          // Global search requires to pass a parent & child to search -
          // Level number is not limited to two.
          // e.g. PARENT::CHILD or #1801::BLK::F123

          // Go over the whole job and look for the specified item.
          var objs = JobsFactory.job.data.ScoredObjectsList.ScoredObject;
          obj = $filter('filter')(objs, {"ObjectName": globalSearch[0]}, compareStr)[0];
          if (angular.isDefined(obj)) {
            var nextObj;
            for (var i = 1; i < globalSearch.length; i++) { // Start from 1!
              if (obj.NestedObjList && obj.NestedObjList.ScoredObject && obj.NestedObjList.ScoredObject.length) {
                nextObj = $filter('filter')(obj.NestedObjList.ScoredObject, {"ObjectName": globalSearch[i]}, compareStr)[0];
                if (nextObj) {
                  if (globalSearch.length == (i + 1)) {
                    // Reached the last element in the query.
                    goToObject(nextObj.ParentUniqueId, nextObj.ObjectUniqueId);
                  }
                  else {
                    // Proceed another level.
                    obj = nextObj;
                  }
                }
                else {
                  // Go to the last item found.
                  goToObject(obj.ParentUniqueId, obj.ObjectUniqueId);
                }
              }
            }
          }
          else {
            $scope.findItemPopupMsg = "Could not find " + globalSearch[0];
          }
        }
        // Search current level items, if not found - seach top-level.
        else if (FieldObjFactory.sortedList) {
          // Item list is already in scope.
          obj = $filter('filter')(FieldObjFactory.sortedList, { title: item })[0];
          if (angular.isDefined(obj)) {
            goToObject($scope.parentId || 0, obj.id);
          }
          else {
            searchTopLevel(item);
          }
        }
        else {
          // Item list missing, create it.
          FieldObjFactory.getSortedList($scope.jobId, $scope.parentId).then(function() {
            obj = $filter('filter')(FieldObjFactory.sortedList, {title: item})[0];
            if (angular.isDefined(obj)) {
              goToObject($scope.parentId || 0, obj.id);
            }
            else {
              searchTopLevel(item);
            }
          });
        }
      } catch (e) {
        console.log(e);
      }
    };

  };

  return {
    restrict: 'E',
    controller: itemFindController,
    template: '<button ng-click="findItemPopup()" class="find-item ion-android-search"></button>'
  };
});
