'use strict';
angular.module('ib.directives')
.directive('paginator', function factory() {

  function getPagerMaxItems() {
    return Math.floor((window.innerWidth - (65*4)) / 42, 10);
  }

  return {
    restrict:'E',
    controller: function ($scope, Paginator) {
      Paginator.page = 0;
      $scope.paginator = Paginator;
      $scope.paginator.maxItems = getPagerMaxItems();
      if (angular.isDefined($scope.objLength)) {
        $scope.paginator.pageCount = function() {
          return Math.ceil($scope.limitedObjects.length / parseInt($scope.rowsPerPage));
        }
      }

      $scope.$watch(function() {
        return window.innerWidth;
      }, function(winInnerWidth) {
        $scope.paginator.maxItems = getPagerMaxItems();
      });
      $scope.paginator.showEllipsisStart = function(active) {
        var maxItems = $scope.paginator.maxItems,
            pageCount = $scope.paginator.pageCount();
        active++; // Zero based.
        if (pageCount > maxItems && active > Math.floor(maxItems / 2)) {
          return true;
        }
      };
      $scope.paginator.showEllipsisEnd = function(active) {
        var maxItems = $scope.paginator.maxItems,
            pageCount = $scope.paginator.pageCount();
        active++; // Zero based.
        if (pageCount > maxItems && active < pageCount - Math.floor(maxItems / 2)) {
          return true;
        }
      }
      $scope.paginator.showItem = function(current, active) {
        var show = true,
            maxItems = $scope.paginator.maxItems,
            pageCount = $scope.paginator.pageCount();

        if (!maxItems) {
          return show;
        }
        if ($scope.paginator.showEllipsisStart(active)) {
          maxItems--;
        }
        if ($scope.paginator.showEllipsisEnd(active)) {
          maxItems--;
        }

        active++; // Zero based.
        current++; // Zero based.

        if (active <= Math.floor(maxItems / 2) && current <= maxItems) {
          // active is at the begining.
          show = true;
        }
        else if (pageCount - active < Math.floor(maxItems / 2) && pageCount - current < maxItems) {
          // active is at the end.
          show = true;
        }
        else if (active - current > Math.floor(maxItems / 2)) {
          // current is too far low, hide.
          show = false;
        }
        else if (active - current <= maxItems / -2) {
          // current is too far high, hide.
          show = false;
        }
        return show;
      };
    },
    templateUrl: 'templates/paginationControl.html'
  };
});
