'use strict';
angular.module('ib.directives')
.directive('swipeable', function($document) {
  return {
    restrict: 'A',
    link: function(scope, element) {

      var startX, x, animationInterval;

      // Bind to 'dragstart'.
      element.on('dragstart', drag);
      scope.$on('$destroy', function() {
        element.off('dragstart', drag);
      });

      function drag(event) {
        // Prevent default dragging if scrolling down.
        if (event.gesture.velocityX < event.gesture.velocityY) {
          return;
        }
        // Prevent on related mode.
        if (scope.relatedMode) {
          return;
        }
        event.stopPropagation();
        event.gesture.preventDefault();
        // Init.
        x = 0;
        // Set starting point.
        startX = event.gesture.center.pageX - x;
        // Move while dragging.
        $document.on('drag', function(event) {
          x = event.gesture.center.pageX - startX;
          // 5px threshold
          if (Math.abs(x) > 10) {
            move(x);
          }
          else if (x != 0) {
            move(0);
          }
        });
        $document.on('dragend', release);
      };

      // Move item using css transform.
      function move(x) {
        element.css({
          '-webkit-transform':  'translate3d(' + x + 'px, 0, 0)',
          'transform':  'translate3d(' + x + 'px, 0, 0)',
          'opacity': Math.max(1 - Math.abs(x/200), 0),
        });
      }

      // Animate move().
      function animate(n) {
        animationInterval = setInterval(function() {
          x = x + n;
          if (10 > x && x > -10) {
            move(0);
            clearInterval(animationInterval);
            return false;
          }
          else {
            move(x);
          }
        }, 1);
      }

      // Release handler
      function release() {
        // ToDo: CHECK WHY THESE DON'T WORK
        // $document.off('drag', move);
        // $document.off('dragend', release);
        // THESE WORK BUT REMOVE ALL OF THE BINDINGS OF $document FOR 'drag' and 'dragend'
        $document.off('drag');
        $document.off('dragend');

        if (x > 100 && !scope.disabledPrev) {
          // Skip to prev.
          goTo('prev');
        }
        else if (x < -100 && !scope.disabledNext) {
          // Skip to next.
          goTo('next');
        }
        else {
          // If not passed threshold animate it back.
          var diff = x > 0 ? -2 : 2;
          // Limit x so we don't wait for it too much.
          x = Math.min(Math.max(parseInt(x), -100), 100);
          animate(diff);
        }
      }

      // Navigate handler
      function goTo(dir) {
        scope.goTo(dir);
        scope.$apply();
      }

      scope.$on('objctrl.transition', function() {
        element.removeAttr('style');
      });

    }
  }
});
