angular.module('ib.services')
.service('DbService', function($rootScope, $window, $q, $http, HelperService, localStorageService, $log, $cordovaFile, $timeout) {
  var DB_NAME = "ib";
  var DB_FORM_TYPE_INPUT = 1;
  var DB_FORM_TYPE_DESC = 2;
  var _db;
  this.run = function(needUpdate) {
    return createDb(needUpdate);
  }
  var createDb = function(needUpdate) {
    var deferred = $q.defer();
    try {
      $window.sqlitePlugin.openDatabase({ name: DB_NAME, location: 'default' }, function (db) {
        _db = db;
        deferred.resolve();
        createTables(needUpdate, deferred);
      }, function (err) {
        _db = $window.openDatabase(DB_NAME, '1.0', 'IB', 10 * 3072 * 1024);
        createTables(needUpdate, deferred);
      });
    } catch (e) {
      _db = $window.openDatabase(DB_NAME, '1.0', 'IB', 10 * 3072 * 1024);
      createTables(needUpdate, deferred);
    }

    return deferred.promise;
  }
  this.clearJobsAsync = function() {
    var deferred = $q.defer();
    localStorageService.clearAll("jobData");
    _db.transaction(function(tx) {
      tx.executeSql("drop table jobs_values", []);
      tx.executeSql("drop table jobs_images", []);
      tx.executeSql("drop table jobs", []);
      tx.executeSql("drop table jobs_fields", []);
      tx.executeSql("drop table jobs_fields_childes", []);
      tx.executeSql("drop table jobs_forms", []);
      tx.executeSql("drop table jobs_config", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_config(ckey text primary key, cval text)", []);
      tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_conf_index on jobs_config(ckey)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS ib_log(id integer primary key, logMsg text)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_forms(id integer, odefId integer, type int, form text)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs(id integer primary key, jobDate integer, data text, jobName text, jobId text, jobProperties text, status integer, clientId integer, userId integer, isSync integer)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_fields(odefId integer, fid integer primary key, id integer, pfid integer, level integer, status integer, isNew integer, syncStatus integer, serverId integer, userFid text, pServerId integer)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_fields_childes(fid integer, id integer, objs text)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_values(odefId integer, pfid integer, userFid text, id integer, fid integer, pd integer, inputDate integer, inputValue text, valueType integer)", []);
      tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_images(odefId integer, pfid integer, userFid text, id integer, fid integer, pd integer, inputDate integer, local_url text primary key, s3_url text, status integer, valueType integer)", []);
      tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_index on jobs_values(id, fid, pd)", []);

      tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_index_forms on jobs_forms(id, odefId, type)", []);
      tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_index_childs on jobs_fields_childes(id, fid)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_images_nid_index on jobs_images(id)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_images_fid_index on jobs_images(fid)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_mul_index on jobs_fields(fid, id, pfid)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_fid_index on jobs_fields(fid)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_nid_index on jobs_fields(id)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_childs_nid_index on jobs_fields_childes(id)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_forms_nid_index on jobs_forms(id)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_forms_nid_type_index on jobs_forms(id, type)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_childs_fid_index on jobs_fields_childes(fid)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS fields_status_index on jobs_fields(status)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_values_fid_index on jobs_values(fid)", []);
      tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_status_index on jobs(status)", []);
      deferred.resolve();
    });
    return deferred.promise;
  }



  this.clearJobs = function() {
    localStorageService.clearAll("jobData");
    _db.transaction(function(tx) {
      tx.executeSql("drop table jobs_values", []);
      tx.executeSql("drop table jobs_images", []);
      tx.executeSql("drop table jobs", []);
      tx.executeSql("drop table jobs_fields", []);
      tx.executeSql("drop table jobs_fields_childes", []);
      tx.executeSql("drop table jobs_forms", []);

      createTables();
    });
  }

  var upload = function(imageURI, jobId) {
    var deferred = $q.defer();

    try {
      var fileName = imageURI.split("/").pop(),
          ft = new FileTransfer(),
          options = new FileUploadOptions();

      options.fileKey = "file";
      options.fileName = fileName;
      options.mimeType = "image/jpeg";
      options.chunkedMode = false;
      $http.get($rootScope.endPoint + '/appjob/' + fileName + '/' + jobId + '/' + $rootScope.clientId + '.json', {timeout: 300000, warningAfter: 5000})
        .success(function (data, status, headers, config) {
          if (data.resStatus) {
            options.params = {
              "key": $rootScope.clientId + "/" + jobId + "/" + fileName,
              "AWSAccessKeyId": data.data.awsKey,
              "acl": "public-read",
              "policy": data.data.policy,
              "signature": data.data.signature,
              "Content-Type": "image/jpeg"
            };
            var uploadUrl;

            uploadUrl = "https://" + data.data.bucket + ".s3.amazonaws.com/";
            $log.info("uploading image : " + uploadUrl);
            ft.upload(imageURI, uploadUrl,
              function (e) {
                $log.info("uploading image : success" + uploadUrl);
                updateJobImage(imageURI, jobId + "/" + fileName);
                deferred.resolve(true);
              },
              function (e) {
                try {
                  $log.info("uploading image : fail " + uploadUrl);
                  var longMessage;
                  longMessage = JSON.stringify(e);
                  $log.log("s3upload", {'_exception': e.exception, '_uploadUrl': uploadUrl, '_logGroup': 's3upload'}, longMessage);
                } catch (e) {

                }
                if (e.code == 1 || e.code == 2) {
                  updateJobImageStatus(2);
                }
                deferred.resolve(false);
              }, options
            );

          }
        })
        .error(function (data, status, headers, config) {
          deferred.resolve(false);
        });
    } catch (e) {
      $log.error(e);
      deferred.resolve(false);
    }

    return deferred.promise;
  }


  var createTables = function(needUpdate, deferred) {

      _db.transaction(function(tx) {
        if (needUpdate) {
          tx.executeSql("drop table jobs_values", []);
          tx.executeSql("drop table jobs_images", []);
          tx.executeSql("drop table jobs", []);
          tx.executeSql("drop table jobs_fields", []);
          tx.executeSql("drop table jobs_fields_childes", []);
          tx.executeSql("drop table jobs_forms", []);
        }
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_config(ckey text primary key, cval text)", []);
        tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_conf_index on jobs_config(ckey)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS ib_log(id integer primary key, logMsg text)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_forms(id integer, odefId integer, type int, form text)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs(id integer primary key, jobDate integer, data text, jobName text, jobId text, jobProperties text, status integer, clientId integer, userId integer, isSync integer)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_fields(odefId integer, fid integer primary key, id integer, pfid integer, level integer, status integer, isNew integer, syncStatus integer, serverId integer, userFid text, pServerId integer)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_fields_childes(fid integer, id integer, objs text)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_values(odefId integer, pfid integer, userFid text, id integer, fid integer, pd integer, inputDate integer, inputValue text, valueType integer)", []);
        tx.executeSql("CREATE TABLE IF NOT EXISTS jobs_images(odefId integer, pfid integer, userFid text, id integer, fid integer, pd integer, inputDate integer, local_url text primary key, s3_url text, status integer, valueType integer)", []);
        tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_index on jobs_values(id, fid, pd)", []);
        tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_index_forms on jobs_forms(id, odefId, type)", []);
        tx.executeSql("CREATE UNIQUE INDEX IF NOT EXISTS val_index_childs on jobs_fields_childes(id, fid)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_images_nid_index on jobs_images(id)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_images_fid_index on jobs_images(fid)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_mul_index on jobs_fields(fid, id, pfid)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_fid_index on jobs_fields(fid)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_nid_index on jobs_fields(id)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_childs_nid_index on jobs_fields_childes(id)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_forms_nid_index on jobs_forms(id)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_forms_nid_type_index on jobs_forms(id, type)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_childs_fid_index on jobs_fields_childes(fid)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS fields_status_index on jobs_fields(status)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_values_fid_index on jobs_values(fid)", []);
        tx.executeSql("CREATE INDEX IF NOT EXISTS jobs_status_index on jobs(status)", []);
        deferred.resolve();
      })
  }

  this.getJobValues = function(id) {
    var deferred = $q.defer();
    executeSelect("SELECT jv.*, jf.isNew as isNewField, jf.serverId FROM jobs_values as jv JOIN jobs_fields as jf on jv.fid = jf.fid where jv.id = ?", [id], 0, 0, 0).then(function(results){
      deferred.resolve(results);
    });
    return deferred.promise;
  }
  this.getNeedSyncFields = function(id) {
    var deferred = $q.defer();
//    _ib_insert_into_ib_field_objects($row['userFid'], $row['odefId'], $nid, $row['level'], $row['pfid'], '')) {
    executeSelect("SELECT fid, userFid, isNew, odefId, level, serverId, pfid, pServerId FROM jobs_fields where id = ? and syncStatus = 0 order by level", [id], 0, 0, 0).then(function(results){
      deferred.resolve(results);
    });
    return deferred.promise;
  }
  var updateFieldsSyncStatus = function(fields) {
    var deferred = $q.defer();
    var query = "UPDATE jobs_fields set syncStatus = 1 where fid = ?";
    _db.transaction(function(tx) {
      fields.forEach(function (args) {
        tx.executeSql(query, [args.fid]);
      });
      deferred.resolve();
    });
    return deferred.promise;
  }
  this.getNeedValuesFields = function(id) {
    var deferred = $q.defer();
    simpleExecuteSelect("SELECT fid, status FROM jobs_fields where id = ? and status != 2", [id]).then(function(results){
      deferred.resolve(results);
    });
    return deferred.promise;
  }
  this.selectJobsData = function() {
    return simpleExecuteSelect("SELECT data, id FROM jobs");
  }

  this.getFieldsStatus = function(objId) {
    if (objId == null) {
      objId = 0;
    }
    var deferred = $q.defer();
    simpleExecuteSelect("SELECT fid, status FROM jobs_fields where pfid = ?", [objId]).then(function(results){
      var objs = {};
      var len = results.rows.length, i;
      for (i = 0; i < len; i++){
        objs[results.rows.item(i).fid] = results.rows.item(i).status;
      }
      deferred.resolve(objs);
    });
    return deferred.promise;

  }

  this.getFieldStatus = function(objId) {
    var deferred = $q.defer();
    simpleExecuteSelect("SELECT status FROM jobs_fields where fid = ?", [objId]).then(function(results){
      var objs = {};
      var len = results.rows.length, i;

      if (len > 0){
        deferred.resolve(results.rows.item(0).status);
      }
      else deferred.resolve(0);
    });
    return deferred.promise;

  }


  this.getJobSyncedImages = function(nid) {
    var deferred = $q.defer();
    executeSelect("SELECT jv.*, jf.isNew as isNewField, jf.serverId FROM jobs_images as jv JOIN jobs_fields as jf on jv.fid = jf.fid  where jv.id = ? and jv.status = 1", [nid], 0, 0, 0).then(function(results){
      deferred.resolve(results);
    });
    return deferred.promise;
  }

  this.getJobOfflineImages = function(nid) {
    var deferred = $q.defer();
    executeSelect("SELECT jv.*, jf.isNew as isNewField, jf.serverId FROM jobs_images as jv JOIN jobs_fields as jf on jv.fid = jf.fid  where jv.id = ?", [nid], 0, 0, 0).then(function(results){
      deferred.resolve(results);
    });
    return deferred.promise;
  }

  this.getJobImages = function(nid, objId) {
    var deferred = $q.defer();
    var items = [];

    // DEBUG
    // items = [
    //   { imgUrl: 'http://lorempixel.com/600/600/nature/?v=1' },
    //   { imgUrl: 'http://lorempixel.com/600/600/nature/?v=2' },
    //   { imgUrl: 'http://lorempixel.com/600/600/nature/?v=3' },
    //   { imgUrl: 'http://lorempixel.com/600/600/nature/?v=4' },
    //   { imgUrl: 'http://lorempixel.com/600/600/nature/?v=5' }
    // ];
    // deferred.resolve(items);

    simpleExecuteSelect("SELECT local_url FROM jobs_images where id = ? and fid = ?", [nid, objId]).then(function(results){
      var len = results.rows.length, i;
      for (i = 0; i < len; i++){
        results.rows.item(i);
        items.push({imgUrl: results.rows.item(i).local_url});
      }
      deferred.resolve(items);
    });
    return deferred.promise;
  }

  this.getFieldObjectValues = function(objId) {
    var deferred = $q.defer();
    var values = {};
    simpleExecuteSelect("SELECT pd, inputValue FROM jobs_values where fid = ? and inputValue != ''", [objId]).then(function(results){
      var len = results.rows.length, i;
      for (i = 0; i < len; i++){
        results.rows.item(i);
        values[results.rows.item(i).pd] = results.rows.item(i).inputValue;
      }
      deferred.resolve(values);
    });
    return deferred.promise;
  }

  this.isJobsHasImages = function() {
    var deferred = $q.defer();
    var query = "SELECT local_url as id from jobs_images where status = 0";
    var args = [];
    var imageCount = 0;
    executeSelect(query, args, 1, 0, 0).then(function(results){
      imageCount = results.rows.length;
      if (results.rows.length > 0) {
        deferred.resolve(true);
      }
      else {
        deferred.resolve(false);
      }
    });
    return deferred.promise;
  }

  this.sendJobFieldsToServer = function(jobId) {
    var deferred = $q.defer();
    this.getNeedSyncFields(jobId).then(function(jobFields) {
      if (jobFields.rows.length > 0 ) {
        $http.put($rootScope.endPoint + '/appjob/' + jobId + '.json?mode=fields' , {rows:jobFields.rows}, {timeout: 300000, warningAfter: 5000})
          .success(function (data, status, headers, config) {
            var rows = data.data;
            updateJobFieldsServerId(rows).then(function() {
              updateFieldsSyncStatus(rows).then(function() {
                deferred.resolve(true);
              })
            })
          })
          .error(function (data, status, headers, config) {
            $log.warn("error in sendJobFieldsToServer data = '" + data + "' : status = '" + status + "'");
            deferred.resolve(false);
          })
      }
      else {
        deferred.resolve(true);
      }
    });
    return deferred.promise;
  }


  this.processJobImages = function(jobId) {

    var deferred = $q.defer();
    var query = "SELECT local_url as id from jobs_images where id = ? and status = 0";
    var args = [jobId];
    var prom = [];
    var imageCount = 0;
    executeSelect(query, args, 1, 0, 0).then(function(results){
      imageCount = results.rows.length;
      if (results.rows.length > 0) {

        results.rows.forEach(function (id) {
          prom.push(upload(id, jobId));
        });
      }
      else {
        deferred.resolve({status:1, imageCount:imageCount});
      }

      $q.all(prom).then(function () {
        executeSelect(query, args, 1, 0, 0).then(function(results){
          imageCount = results.rows.length;
          if (imageCount > 0) {
            deferred.resolve({status:0, imageCount:imageCount});
          }
          else {
            deferred.resolve({status:1, imageCount:imageCount});
          }
        });
      });
    });


    return deferred.promise;
  }

  var updateJobImageStatus = function(local_url, status) {
    var query = "UPDATE jobs_images set status = ?, s3_url = ? where local_url = ?";
    var args = [status, local_url];
    _db.transaction(function(tx) {
      tx.executeSql(query,args);
    });
  }

  var updateJobImage = function(local_url, s3_url) {
    var query = "UPDATE jobs_images set status = 1, s3_url = ? where local_url = ?";
    var args = [s3_url, local_url];
    _db.transaction(function(tx) {
      tx.executeSql(query,args);
    });
  }


  this.addIsNewField = function() {
    var deferred = $q.defer();
    _db.transaction(function(tx) {
      tx.executeSql("ALTER TABLE jobs_fields ADD COLUMN isNew integer",[], dbSuccess, dbError);
      tx.executeSql("ALTER TABLE jobs_fields ADD COLUMN syncStatus integer",[], dbSuccess, dbError);
      tx.executeSql("ALTER TABLE jobs_fields ADD COLUMN serverId integer",[], dbSuccess, dbError);
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in addIsNewField');
    }
    return deferred.promise;
  }
//  this.
  this.insertJobFields = function(fields, isNew) {
    //isNew logic
    //0 from server
    //1 new
    //2 deleted
    var deferred = $q.defer();
    var syncStatus = 1;
    if (isNew == 1) {
      syncStatus = 0;
    }
    var query = "INSERT INTO jobs_fields (odefId, fid , id, pfid, level, status, userFid, isNew, syncStatus) VALUES (?,?,?,?,?,0,?,?,?)";
    _db.transaction(function(tx) {
      fields.forEach(function (args) {
        args.push(isNew);
        args.push(syncStatus);
        tx.executeSql(query,args, dbSuccess, dbError);
        if (isNew == 1) {
          tx.executeSql("UPDATE jobs_fields set pServerId = (SELECT pjf.ServerId FROM jobs_fields pjf WHERE jobs_fields.pfid = pjf.fid) where jobs_fields.fid = ? and jobs_fields.id = ?",[args[1], args[2]], dbSuccess, dbError);
        }
      });
      deferred.resolve();
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in insertJobFields');
    }
    return deferred.promise;
  }


  this.getJobFormByType = function(jobId, odefId, type) {
    var deferred = $q.defer();
    simpleExecuteSelect("select form from jobs_forms where id = ? and odefId = ? and type = ?", [jobId, odefId, type]).then(function(results){
      var form = false;
      if (results.rows.length > 0) {
        form = JSON.parse(results.rows.item(0).form);
      }
      deferred.resolve(form);
    });
    return deferred.promise;
  }


  this.insertJobForms = function(forms) {
    var query = "INSERT INTO jobs_forms (id, odefId, form, type) VALUES (?,?,?,?)";
    _db.transaction(function(tx) {
      forms.forEach(function (args) {
        tx.executeSql(query,args, dbSuccess, dbError);
      });
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in insertJobForm');
    }
  }

  this.getJobFieldChildes = function(jobId, fid) {
    var deferred = $q.defer();
    var selectFid;
    if (fid == null) {
      selectFid = 0;
    }
    else {
      selectFid = fid;
    }
    simpleExecuteSelect("select objs from jobs_fields_childes where id = ? and fid = ?", [jobId, selectFid]).then(function(results){
      var objs = [];
      if (results.rows.length > 0) {
        objs = JSON.parse(results.rows.item(0).objs);
        objs = objs.filter(function (obj) { return obj.deleted !== true; })
      }
      deferred.resolve(objs);
    });
    return deferred.promise;
  }

  this.updateJobFieldsChildes = function(id, fid, objs) {
    var query = "REPLACE INTO jobs_fields_childes (objs ,fid ,id) VALUES (?,?,?)";
    _db.transaction(function(tx) {
      tx.executeSql(query,[JSON.stringify(objs), fid, id], dbSuccess, dbError);
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in updateJobFieldsChildes');
    }
  }

  this.insertJobFieldsChildes = function(fields) {
    var query = "REPLACE INTO jobs_fields_childes (fid, id, objs) VALUES (?,?,?)";
    _db.transaction(function(tx) {
      fields.forEach(function (args) {
        tx.executeSql(query,args, dbSuccess, dbError);
      });
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in insertJobFieldsChildes');
    }
  }

  this.updateJobField = function(fid, status) {
    var query = "UPDATE jobs_fields set status = ? where fid = ?";
    _db.transaction(function(tx) {
      tx.executeSql(query,[status, fid]);
    });
  }

  var updateJobFieldsServerId = function(fields) {
    var deferred = $q.defer();
    var query = "UPDATE jobs_fields set serverId = ? where fid = ?";
    var pQuery = "UPDATE jobs_fields set pServerId = ? where pfid = ?";
    if (fields.length > 0) {
      _db.transaction(function(tx) {
        fields.forEach(function (args) {
          if (args.fid != "0") {
            tx.executeSql(query, [args.serverId, args.fid]);
            tx.executeSql(pQuery, [args.serverId, args.fid]);
          }
        });
        deferred.resolve();
      });
    }
    else {
      deferred.resolve();
    }
    return deferred.promise;
  }


  this.updateJobFieldIsNew = function(fid, status) {
    var query = "UPDATE jobs_fields set isNew = ?, syncStatus = 0 where fid = ?";
    _db.transaction(function(tx) {
      tx.executeSql(query,[status, fid]);
    });
  }

//  var removefile = function(file){
//    fileSystem.root.getFile(file, {create: false, exclusive: false}, gotRemoveFileEntry, fail);
//  }
//
//  var gotRemoveFileEntry = function(fileEntry){
//    fileEntry.remove(fileRemoveSuccess, fileRemoveFail);
//  }
//
//  var fileRemoveSuccess = function(entry) {
//    console.log("Removal succeeded");
//  }

//  var fileRemoveFail = function(error) {
//    console.log("Error removing file: " + error.code);
//  }
  this.deleteFormsCache = function() {
    var deferred = $q.defer();
    _db.transaction(function(tx) {
      var query;
      query = "DELETE FROM jobs_forms";
      tx.executeSql(query, dbSuccess, dbError);
      deferred.resolve(true);
      var dbSuccess = function(tx, results) {

      }
      var dbError = function(e) {
        $log.log('db error in deleteFormsCache');
      }

    })
    return deferred.promise;
  }
  this.deleteJobData = function(jobs) {
    var deferred = $q.defer();
    var query;

    _db.transaction(function(tx) {
      jobs.forEach(function (job) {
        if ($rootScope.mode == "mobile") {
          $cordovaFile.removeFile(cordova.file.dataDirectory, job + ".json");
        }
        query = "DELETE FROM jobs where id = ?";
        tx.executeSql(query, [job], dbSuccess, dbError);
        query = "DELETE FROM jobs_values where id = ?";
        tx.executeSql(query, [job], dbSuccess, dbError);
        query = "DELETE FROM jobs_fields where id = ?";
        tx.executeSql(query, [job], dbSuccess, dbError);
        query = "DELETE FROM jobs_images where id = ?";
        tx.executeSql(query, [job], dbSuccess, dbError);
        query = "DELETE FROM jobs_forms where id = ?";
        tx.executeSql(query, [job], dbSuccess, dbError);

      });
      deferred.resolve(true);
    });
    var dbSuccess = function(tx, results) {

    }
    var dbError = function(e) {
      $log.log('db error in deleteJobData');
    }
    return deferred.promise;
  }

  this.deleteJobImages = function(files) {
    var deferred = $q.defer();
    var query;
    query = "DELETE FROM jobs_images where local_url = ?";
    _db.transaction(function(tx) {
      files.forEach(function (file) {
        tx.executeSql(query, [file.imgUrl], dbSuccess, dbError);
      });
      deferred.resolve(true);
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in deleteJobImages');
    }
    return deferred.promise;
  }

  this.insertJobImage = function(id, fid, pd, inputDate, local_url, obj) {
    var isNew, pfid, userFid, odefId;
    if ('isNewItem' in obj) {
      pfid = obj.ParentUniqueId;
      userFid = obj.ObjectName;
      odefId = obj.ScObjDefId;
    }
    else {
      pfid = 0;
      userFid = '';
      odefId = 0;
    }

    var query;
    var args;
    args = [id, fid, pd, inputDate, local_url, pfid, userFid, odefId];

    query = "INSERT INTO jobs_images (id, fid, pd, inputDate, local_url, status, valueType, pfid, userFid, odefId) VALUES (?,?,?,?,?,0,2,?,?,?)";
    _db.transaction(function(tx) {
      tx.executeSql(query,args, dbSuccess, dbError);
      query = "update jobs set isSync = 0 where id = ?";
      args = [id];
      tx.executeSql(query,args, dbSuccess, dbError);
    });
    var dbSuccess = function(tx, results) {
    }
    var dbError = function(e) {
      $log.log('db error in insertJobImage');
    }


  }



  this.updateJobSyncStatus = function(id, status) {
    var deferred = $q.defer();
    var query;
    var args;
    query = "update jobs set isSync = ? where id = ?";
    args = [status, id];
    _db.transaction(function(tx) {
      tx.executeSql(query,args, dbSuccess, dbError);
    });
    var dbSuccess = function(tx, results) {
      deferred.resolve(true);
    }
    var dbError = function(e) {
      $log.log('db error in updateJobSyncStatus');
      deferred.resolve(false);
    }
    return deferred.promise;
  }


  this.insertConfValue = function(ckey, cval) {
    var query = "REPLACE INTO jobs_config (ckey, cval) VALUES (?,?)";
    _db.transaction(function(tx) {
      tx.executeSql(query, [ckey, cval]);
    });
  }
  this.insertJobValue = function(id, fid, pd, inputDate, inputValue, pfid, userFid, odefId) {
    var deferred = $q.defer();
    var query;
    var args;
    args = [id, fid, pd, inputDate, inputValue, pfid, userFid, odefId];
    query = "REPLACE INTO jobs_values (id, fid, pd, inputDate, inputValue, valueType, pfid, userFid, odefId) VALUES (?,?,?,?,?,1,?,?,?)";
    _db.transaction(function(tx) {
      tx.executeSql(query,args, dbSuccess, dbError);
      query = "update jobs set isSync = 0 where id = ?";
      args = [id];
      tx.executeSql(query,args, dbSuccess, dbError);

    });
    var dbSuccess = function(tx, results) {
      deferred.resolve(true);
    }
    var dbError = function(e) {
      $log.log('db error in insertJobValue');
      deferred.resolve(false);
    }
    return deferred.promise;
  }



  this.insertJob = function(id, jobDate, data, jobName, jobId, jobProperties, status) {
    var query = "INSERT INTO jobs (id, jobDate, data, jobName, jobId, jobProperties, status, isSync) VALUES (?,?,?,?,?,?,?,1)";
    var args = [];
    args.push(id);
    args.push(jobDate);
    args.push(data);
    args.push(jobName);
    args.push(jobId);
    args.push(jobProperties);
    args.push(status);
    _db.transaction(function(tx) {
      tx.executeSql(query,args);
    });
  }


  var simpleExecuteSelect = function(query, args) {
    var deferred = $q.defer();
    _db.transaction(function (tx) {
      tx.executeSql(query, args, dbSuccess, dbError);
    });
    var dbSuccess = function(tx, results) {
      deferred.resolve(results);
    }
    var dbError = function(e) {
      $log.log('db error in simpleExecuteSelect');
      deferred.resolve(false);
    }
    return deferred.promise;
  }

  var executeSelect = function(query, args, returnIds, parseJson, returnRows) {
    var deferred = $q.defer();
    var items = [];
    var itemsO = {};
    _db.transaction(function (tx) {
      tx.executeSql(query, args, dbSuccess, dbError);
    });
    var dbSuccess = function(tx, results) {
      if (returnRows) {
        deferred.resolve(results);
      }
      var len = results.rows.length, i;
      for (i = 0; i < len; i++){
        if (returnIds == 1) {
          items.push(results.rows.item(i).id);
        }
        else {
          if (parseJson == 1) {
            var tempItem = results.rows.item(i);
            var statusClass = HelperService.getStatusClass(tempItem.status);
            var o = {id: tempItem.id, jobDate: tempItem.jobDate, jobName: tempItem.jobName, jobProperties: JSON.parse(tempItem.jobProperties), data: JSON.parse(tempItem.data), status: tempItem.status, statusClass:statusClass};
            itemsO[tempItem.id] = o;
          }
          else {
            items.push(results.rows.item(i));
          }
        }
      }
      if (parseJson == 1) {
        deferred.resolve({rows:itemsO});
      }
      else {
        deferred.resolve({rows:items});
      }
    }
    var dbError = function(e) {
      $log.log('db error in executeSelect');
      deferred.resolve([]);
    }
    return deferred.promise;

  }

  this.updateJobStatus = function(jobId, status) {
    var query = "update jobs set status = ? where id = ?";

    _db.transaction(function(tx) {
      tx.executeSql(query, [status, jobId]);
    });
  }

  this.updateJobData = function(jobId, data) {
    var deferred = $q.defer();

    if ($rootScope.mode == "mobile") {
      $cordovaFile.writeFile(cordova.file.dataDirectory, jobId + ".json", data, true).then(function(result) {
        deferred.resolve(true);
        // Success!
      }, function(err) {
        $log.error(err);
        deferred.resolve(false);
        // An error occured. Show a message to the user
      });
    }
    else {

      var query = "update jobs set data = ? where id = ?";

      _db.transaction(function(tx) {
        tx.executeSql(query, [data, jobId], dbSuccess, dbError);
      });
      var dbSuccess = function(tx, results) {
        deferred.resolve(results);
      }
      var dbError = function(e) {
        $log.log('db error in updateJobData');
        deferred.resolve();
      }

    }

    return deferred.promise;
  }

  this.selectJobs = function() {
    return simpleExecuteSelect("SELECT id ,jobDate, jobName, jobId, jobProperties, status FROM jobs order by jobDate desc", []);
  }

  this.selectJobData = function(id) {
    return simpleExecuteSelect("SELECT status, data FROM jobs where id = ?", [id]);
  }

  this.getJobsIdList = function() {
    return executeSelect("SELECT id FROM jobs", [], 1, 0, 0);
  }

  this.resendJobs = function() {

    var query = "UPDATE jobs set isSync = ?";
    var args = [0];
    _db.transaction(function(tx) {
      tx.executeSql(query,args);
    });
  }
  this.getNeedSyncJobsIdList = function() {
    return executeSelect("SELECT id FROM jobs where isSync = 0", [], 1, 0, 0);
  }

  this.getDoneJobsIdList = function() {
    return executeSelect("SELECT id FROM jobs where status = 3 and isSync = 1", [], 1, 0, 0);
  }

  // App Settings
  this.getSettings = function() {

    var defaultSettings = {
      keyboard: [
        {name: '-', value: 'bad'},
        {name: '--', value: 'very bad'},
        {name: '+', value: 'good'},
        {name: '++', value: 'very good'},
        {name: 'LOD', value: 'lodging'},
        {name: 'EM', value: 'emergence'},
        {name: 'HS', value: 'headsmut'}
      ],
      sync:{
        autoSyncInterval: {name: 'Auto synchronize interval', value: 0},
        imageUploadNotify: {name: 'Upload images only on WiFi', value: true}
      }
    };
    // var defaultSettings = {};

    var deferred = $q.defer();

    deferred.resolve(localStorageService.get('settings') || defaultSettings);
    return deferred.promise;
  }

  this.storeSettings = function(settings) {
    localStorageService.set('settings', settings || {});
    return settings;
  }

  // Last sync date
  this.getLastSync = function() {
    var lastSync = localStorageService.get('lastSync') || false;
    return lastSync;
  }

  this.setLastSync = function(now) {
    now = now || Date.now();
    localStorageService.set('lastSync', now);
    return now;
  }

})
