'use strict';
angular.module('ib.services')
  .factory('FieldObjFactory', function ($q, $filter, $rootScope, DbService, HelperService, localStorageService, $log, gettextCatalog, $cordovaFileTransfer, $location, $cordovaFile) {

    var JOB_STATUS_NEW = 0,
      JOB_STATUS_IN_PROG = 1,
      JOB_STATUS_COMPLETE = 2,
      JOB_STATUS_DONE = 3,
      JOB_STATUS_SYNC = 4,
      FIELD_STATUS_PEND = 0,
      FIELD_STATUS_IN_PROG = 1,
      FIELD_STATUS_COMP = 2;
    var service = {};
    var DB_FORM_TYPE_INPUT = 1;
    var DB_FORM_TYPE_DESC = 2;

    // Generic set function
    service.set = function (name, value) {
      service[name] = value;
    };

    var getObjectChildList = function (data, objId) {
      var obj = findObject(data.ScoredObjectsList.ScoredObject, objId);
      if (obj && 'NestedObjList' in obj && 'ScoredObject' in obj.NestedObjList) {
        return [].concat(obj.NestedObjList.ScoredObject);
      }
      return false;
    };

    var getJobRootObjects = function (data) {
      return [].concat(data.ScoredObjectsList.ScoredObject);
    };

    var checkJob = function (id) {
      DbService.getNeedValuesFields(id).then(function (results) {
        if (results.rows.length === 0) {
          DbService.updateJobStatus(id, JOB_STATUS_COMPLETE);
          $rootScope.$broadcast('jobs.jobUpdate', {id: id, status: JOB_STATUS_COMPLETE});
        }
        else {
          DbService.updateJobStatus(id, JOB_STATUS_IN_PROG);
          $rootScope.$broadcast('jobs.jobUpdate', {id: id, status: JOB_STATUS_IN_PROG});
        }
      });
    };

    var findObject = function (objects, objId) {
      for (var i = 0; i < objects.length; i++) {
        if (objects[i].ObjectUniqueId == objId) {
          return objects[i];
        }
        var found;
        if ('NestedObjList' in objects[i]) {
          if (!Array.isArray(objects[i].NestedObjList)) {
            found = findObject([].concat(objects[i].NestedObjList.ScoredObject), objId);
          }
        }
        if (found) {
          return found;
        }
      }
      return false;
    };

    var parseHistoryList = function (pd, prop) {
      var historyList = prop.ValuesList.Value;
      switch (parseInt(pd.fieldType)) {
        case 3: // Select list
        case 5: // Checkbox
          historyList.forEach(function (hItem) {
            hItem.Value = processPropValue(pd, hItem.Value);
          });
          break;
      }
      return historyList;
    };

    var processPropValue = function (pd, val) {

      if (val === '') {
        return val;
      }

      switch (parseInt(pd.fieldType)) {
        case 3: // Select list
        case 4: // Select list
          var option;
          if (pd.selectionValueList) {
            if (angular.isArray(pd.selectionValueList) && pd.selectionValueList.length === 0) {

            }
            else if (angular.isArray(pd.selectionValueList.valueEntry) && pd.selectionValueList.valueEntry.length) {
              option = $filter('filter')(pd.selectionValueList.valueEntry, {key: val}, true)[0];
              if (option) {
                val = option.value;
              }
            }
            else {
              if (pd.selectionValueList.valueEntry.key && pd.selectionValueList.valueEntry.value) {
                if (pd.selectionValueList.valueEntry.key == val) {
                  val = pd.selectionValueList.valueEntry.value;
                }
              }
            }
          }
          break;
        case 5: // Checkbox
          if (parseInt(val) == 1 || val == 'true') {
            val = '-' + gettextCatalog.getString('True') + '-';
          }
          else {
            val = '-' + gettextCatalog.getString('False') + '-';
          }
          break;
        case 8:
          val = $filter('setTimeFormat')(val * 1000, 'date', 'short');
          break;
      }
      return val;
    };

    var getPropValue = function (valuesList) {
      var val = '';
      try {
        if ('Value' in valuesList) {
          if (valuesList.Value) {
            val = valuesList.Value;
          }
        }
        else {
          if (valuesList instanceof Array) {
            if (valuesList[0].Value) {
              val = valuesList[0].Value;
            }
          }
        }
      } catch (e) {
        $log.error(e);
      }
      return val;
    };

    var getParentDisplayStyle = function (prop, sex) {
      var val = '';
      try {
        if (sex == 'male' && prop && prop.MaleParentDisplayStyle) {
          val = prop.MaleParentDisplayStyle;
        }
        else if (sex == 'female' && prop && prop.FemaleParentDisplayStyle) {
          val = prop.FemaleParentDisplayStyle;
        }
      } catch (e) {
        $log.error(e);
      }
      return val;
    };

    var getParentValue = function (prop, sex) {
      var val = '';
      try {
        if (sex == 'male' && prop && prop.MaleParentValue) {
          val = prop.MaleParentValue;
        }
        else if (sex == 'female' && prop && prop.FemaleParentValue) {
          val = prop.FemaleParentValue;
        }
      } catch (e) {
        $log.error(e);
      }
      if (val == '-' || val instanceof Array) {
        val = '';
      }
      return val;
    };

    var getScoreTime = function (valuesList) {
      var time = 0;
      if (valuesList.InputDate) {
        time = valuesList.InputDate;
      }
      else {
        if (valuesList instanceof Array) {
          if (valuesList[0].InputDate) {
            time = valuesList[0].InputDate;
          }
        }
      }
      return time;
    };

    service.findCurrentObj = function (items, id) {
      for (var i = 0; i < items.length; i++) {
        if (items[i].id == id) {
          return items[i];
        }
      }
    };


    service.findObject = function (objects, objId) {
      return findObject(objects, objId);
    };

    service.getObjectParent = function (fieldObj) {
      return {
        ParentUniqueId: fieldObj.ParentUniqueId
      };
    };

    service.getObjectMeta = function (data, fieldObject) {
      var fod = $filter('filter')(data.ScObjDefList.ScObjDef, {ScObjDefId: fieldObject.ScObjDefId}, true)[0];
      return {
        Name: fieldObject.ObjectName || fieldObject.ObjectUniqueId || gettextCatalog.getString('Item'),
        Type: fod.ScObjType || gettextCatalog.getString('Item')
      };
    };


    service.getObjectsInfoForTableView = function(data, objects , pds) {
      var fod;
      if (!angular.isDefined(service.fieldObj)) {
        // Not defined, get fod.
        fod = service.getFod(data, 0);
      }
      else {
        // Get parent fod.
        fod = $filter('filter')(data.ScObjDefList.ScObjDef, {ScObjDefId: service.fieldObj.ScObjDefId}, true)[0];
        // Get child fod.
        fod = $filter('filter')(data.ScObjDefList.ScObjDef, {ScObjDefId: fod.ScObjChildDefId}, true)[0];
      }

      angular.forEach(objects, function(value, key) {
        value.tableValues = {};
        var fieldObj = findObject(data.ScoredObjectsList.ScoredObject, value.id);
        DbService.getFieldObjectValues(value.id).then(function (values) {
          if (!(Array.isArray(fod.PropDefList.PropDef))) {
            fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
          }
          fod.PropDefList.PropDef.forEach(function (pd) {
            if (pds.indexOf(pd.id) == -1) {
              return;
            }
            var val = '';
            var prop = $filter('filter')(fieldObj.PropertiesList.Property, {PropId: pd.id}, true)[0];
            if (angular.isDefined(values[pd.id])) {
              value.tableValues[pd.id] =values[pd.id] ;
              val = processPropValue(pd, values[pd.id]);
              value.tableValues[pd.id] = val;
            }
            else if (fieldObj && fieldObj.PropertiesList && fieldObj.PropertiesList.Property && fieldObj.PropertiesList.Property.length) {
              if (prop) {
                if (prop.ValuesList && prop.ValuesList.Value) {
                  val = processPropValue(pd, getPropValue(prop.ValuesList.Value));
                  value.tableValues[pd.id] =  "(" + val + ")";
                }
              }
            }
          })

        });

      })
    }

    service.getObjectInfo = function (data) {
      // Field object def
      var fod = $filter('filter')(data.ScObjDefList.ScObjDef, {ScObjDefId: service.fieldObj.ScObjDefId}, true)[0];

      var objInfo = [];

      if (service.fieldObj && service.fieldObj.PropertiesList && service.fieldObj.PropertiesList.Property && service.fieldObj.PropertiesList.Property.length) {
        // Iterate property definitions
        if (!(Array.isArray(fod.PropDefList.PropDef))) {
          fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
        }

        fod.PropDefList.PropDef.forEach(function (pd) {
          var val = '';
          var scoreTime;
          var femaleParentDisplayStyle = '';
          var maleParentDisplayStyle = '';
          var maleParentValue = '';
          var femaleParentValue = '';
          var historyList = false;
          var displayStyle = '0';
          // Get object's property values if exist
          var prop = $filter('filter')(service.fieldObj.PropertiesList.Property, {PropId: pd.id}, true)[0];
          if (prop) {
            if (prop.ValuesList && prop.ValuesList.Value) {
              displayStyle = prop.ValuesList.Value.displayStyle || pd.displayStyle || '0';
              if (prop.ValuesList.Value.length > 1) {
                historyList = parseHistoryList(pd, prop);
              }

              val = processPropValue(pd, getPropValue(prop.ValuesList.Value));
              if (val !== '') {
                scoreTime = getScoreTime(prop.ValuesList.Value);
              }
            }
            if (prop.MaleParentValue || prop.FemaleParentValue) {
              femaleParentDisplayStyle = getParentDisplayStyle(prop, 'female');
              maleParentDisplayStyle = getParentDisplayStyle(prop, 'male');
              maleParentValue = processPropValue(pd, getParentValue(prop, 'male'));
              femaleParentValue = processPropValue(pd, getParentValue(prop, 'female'));
            }
          }
          // Hide Empty
          // if (val == '' && !maleParentValue && !femaleParentValue) {
          //   return;
          // }

          var objProp = {
            Name: pd.name,
            Type: pd.fieldType,
            Unit: pd.Units,
            Value: val,
            Time: scoreTime,
            MaleParentValue: maleParentValue,
            FemaleParentValue: femaleParentValue,
            History: historyList,
            displayStyle: displayStyle,
            maleParentDisplayStyle: maleParentDisplayStyle,
            femaleParentDisplayStyle: femaleParentDisplayStyle
          };
          objInfo.push(objProp);
        });
      }

      return objInfo;

    };
    service.getObjectsListItemInfo = function (jobData, entry, fod) {
      return getObjectsListItemInfo(jobData, entry, fod);
    };

    var getObjectsListItemInfo = function (jobData, entry, fod) {
      var obj = {
        title: entry.ObjectName || entry.ObjectUniqueId,
        id: entry.ObjectUniqueId,
        ParentUniqueId: entry.ParentUniqueId,
        enableNew: fod.EnableNew || true,
//        enableNew: fod.EnableNew || false,
        isNewItem: entry.isNewItem || 0,
        fav: false,
        properties: []
      };
      if (entry && entry.PropertiesList && entry.PropertiesList.Property && entry.PropertiesList.Property.length) {
        // get the properties list
        if (!(Array.isArray(fod.PropDefList.PropDef))) {
          fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
        }

        fod.PropDefList.PropDef.forEach(function (pd) {
          try {
            var prop = $filter('filter')(entry.PropertiesList.Property, {PropId: pd.id}, true)[0];
            if (pd && pd.inListView == 1) {
              var val;
              if (prop && prop.ValuesList && prop.ValuesList.Value) {
                val = processPropValue(pd, getPropValue(prop.ValuesList.Value));
              }
              else {
                val = '-';
              }
              var objProp = {
                Name: pd.name,
                Value: val
              };
              obj.properties.push(objProp);
            }
          } catch (e) {
            $log.error(e);
          }
        });
      }
      // Get child info
      var childFod = service.getChildFod(jobData, entry.ObjectUniqueId);
      if (childFod) {
        obj.childType = childFod.ScObjType;
        var childObjs = getObjectChildList(jobData, entry.ObjectUniqueId) || [];
        // ToDo: Run this again after deleting items.
        obj.childObjs = childObjs.filter(function (obj) {
          return obj.deleted !== true;
        }).length;
      }

      return obj;
    };

    service.getItemStatus = function (objId, fieldInfo) {
      var deferred = $q.defer();
      DbService.getFieldStatus(objId).then(function (fieldsStatus) {
        fieldInfo.statusClass = HelperService.getStatusClass(fieldsStatus);
        deferred.resolve(fieldInfo);
      });
      return deferred.promise;
    };

    var getPropList = function (fod) {
      var propList = [];
      if (!(Array.isArray(fod.PropDefList.PropDef))) {
        fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
      }

      var pds = fod.PropDefList.PropDef;
      pds.forEach(function (pd) {
        if (pd && pd.inListView == 1) {
          propList.push(pd.name);
        }
      });
      return propList;
    };

    var getFlatPropListForTableView = function(fod, propList) {

      if (!(Array.isArray(fod.PropDefList.PropDef))) {
        fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
      }

      var pds = fod.PropDefList.PropDef;
      pds.forEach(function (pd) {
        if (pd) {
          var group = false;
          if (pd.inListView == 1 || pd.isDescision == 1 || pd.isInput == 1) {
            group = true;
          }
          }
          if (group !== false) {
            propList.push({
              id: pd.id,
              name: pd.name,
              fieldType: pd.fieldType,
            });
          }
      });
      return propList;
    }


    var getPropListForTableView = function (fod, propList) {

      if (!(Array.isArray(fod.PropDefList.PropDef))) {
        fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
      }

      var pds = fod.PropDefList.PropDef;
      pds.forEach(function (pd) {
        if (pd) {
          var group = false;
          if (pd.inListView == 1) {
            group = 'listView';
          }
          else if (pd.isDescision == 1) {
            group = 'decision'
          }
          else if (pd.isInput == 1) {
            group = 'input';
          }
          if (group !== false) {
            propList[group].push({
              id: pd.id,
              name: pd.name,
              fieldType: pd.fieldType,
            });
          }
        }
      });
      return propList;

    }


    service.getFod = function (data, objId) {
      var obj;
      if (objId) {
        obj = findObject(data.ScoredObjectsList.ScoredObject, objId);
      }
      else {
        obj = [].concat(data.ScoredObjectsList.ScoredObject)[0];
      }
      var fod = $filter('filter')(data.ScObjDefList.ScObjDef, {
        ScObjDefId: obj.ScObjDefId
      }, true)[0];
      // Get child fod
      return fod;
    };
    service.setFod = function (data, objId) {
      var obj;
      if (objId) {
        obj = findObject(data.ScoredObjectsList.ScoredObject, objId);
      }
      else {
        obj = [].concat(data.ScoredObjectsList.ScoredObject)[0];
      }
      var fod = $filter('filter')(data.ScObjDefList.ScObjDef, {ScObjDefId: obj.ScObjDefId}, true)[0];
      // Get child fod
      service.fod = fod;
    };


    service.getChildFod = function (data, objId) {
      var obj;
      if (objId) {
        obj = findObject(data.ScoredObjectsList.ScoredObject, objId);
      }
      else {
        obj = [].concat(data.ScoredObjectsList.ScoredObject)[0];
      }
      var fod = $filter('filter')(data.ScObjDefList.ScObjDef, {
        ScObjDefId: obj.ScObjDefId
      }, true)[0];
      // Get child fod
      if (fod.ScObjChildDefId) {
        fod = $filter('filter')(data.ScObjDefList.ScObjDef, {
          ScObjDefId: fod.ScObjChildDefId
        }, true)[0];
      }
      else {
        fod = false;
      }
      return fod;
    };


    service.getPropList = function (fod) {
      var propList = [];
      if (fod) {
        propList = getPropList(fod);
      }
      return propList;
    };


    service.getPropListForTableView = function (fod) {
      var propList = [];
      if (fod) {
        getFlatPropListForTableView(fod, propList);
      }
      return propList;
    };


    service.getChildType = function (fod) {
      return getChildType(fod);
    };
    var getChildType = function (fod) {

      return fod && fod.ScObjType || false;

    };

    // service.getFieldChildsList = function() {
    //   DbService.getJobFieldChildes();
    // }

    service.setFieldsStatus = function (fields, fieldsStatus) {
      if (fields.length) {
        fields.forEach(function (field) {
          if (fieldsStatus !== null) {
            field.statusClass = HelperService.getStatusClass(fieldsStatus[field.id]);
          }
        });
      }
      return fields;
    };
    service.getObjectsList = function (data, objId, fod, fieldsStatus) {
      var objects = {};
      if (objId === null || objId === 0) {
        objects = getJobRootObjects(data);
      }
      else {
        objects = getObjectChildList(data, objId) || [];
      }

      var objectsList = [];
      if (objects.length) {
        objects.forEach(function (entry) {
          var obj = getObjectsListItemInfo(data, entry, fod);
          if (fieldsStatus !== null) {
            obj.statusClass = HelperService.getStatusClass(fieldsStatus[entry.ObjectUniqueId]);
          }
          obj.ObjectId = entry.ObjectId;
          objectsList.push(obj);
        });
        return objectsList;
      }
      else {
        return objectsList;
      }
    };

    service.setFieldStatus = function (items, id, status) {
      for (var i = 0; i < items.length; i++) {
        if (items[i].id == id) {
          items[i].statusClass = HelperService.getStatusClass(status);
          return;
        }
      }
    };

    service.getSort = function (jobId, objId) {
      var storeSort = localStorageService.get('jobData.sorting') || {};
      storeSort[jobId] = storeSort[jobId] || {};
      return storeSort[jobId][objId];
    };

    service.getSortedItemId = function (orderdlist, objId) {
      for (var i = 0, len = orderdlist.length; i < len; i++) {
        if (orderdlist[i].id == objId) {
          return i;
        }
      }
      //      deferred.resolve(orderdlist[destObj] ? orderdlist[destObj].id : false);

    };
    service.setParentData = function (jobId, objId) {
      var deferred = $q.defer();
      service.getSortedList(jobId, objId).then(function (res) {
        service.getParentValues(objId).then(function (values) {
          deferred.resolve();
        });
      });
      return deferred.promise;
    };

    service.getParentValues = function (objId) {
      var deferred = $q.defer();
      DbService.getFieldObjectValues(objId).then(function (values) {
        service.parentValues = values;
        deferred.resolve(values);
      });
      return deferred.promise;
    };


    /**
     *
     * @param jobId
     * @param parent
     * @param setObj if true set the result of the function in the service to by used by the controller's.
     * @param shouldFilter if true filter the list.
     * @returns {promise}
     */
    service.getSortedList = function (jobId, parent, setObj, shouldFilter) {
      if (!angular.isDefined(setObj)) {
        setObj = true;
      }
      if (!angular.isDefined(shouldFilter)) {
        shouldFilter = true;
      }

      var deferred = $q.defer();
      var storedSort,
        predicate,
        reverse,
        orderdlist;

      if (parent === 0) {
        parent = null;
      }
      DbService.getJobFieldChildes(jobId, parent).then(function (objs) {
        storedSort = service.getSort(jobId, parent || 0) || {};
        predicate = storedSort.predicate || false;
        reverse = storedSort.reverse || false;
        if (predicate) {
          orderdlist = $filter('orderBy')(objs, predicate, reverse);
        }
        else {
          // Keep original order.
          orderdlist = objs;
        }
        if (setObj) {
          service.orderdlist = orderdlist;
        }
        if (shouldFilter && storedSort.titleOnly) {
          orderdlist = $filter('filter')(orderdlist, {title: storedSort.titleOnly});
        }
        else if (shouldFilter && storedSort.filter) {
          orderdlist = $filter('filter')(orderdlist, {$: storedSort.filter});
        }
        if (shouldFilter && storedSort.fav) {
          orderdlist = $filter('filter')(orderdlist, {fav: true}, true);
        }
        if (setObj) {
          service.sortedList = orderdlist;
        }
        deferred.resolve(orderdlist);
      });
      return deferred.promise;
    };


    service.setSort = function (jobId, objId, predicate, reverse, search, titleOnly, showOnlyPending) {
      var storeSort = localStorageService.get('jobData.sorting') || {};
      storeSort[jobId] = storeSort[jobId] || {};
      storeSort[jobId][objId] = {
        predicate: predicate,
        reverse: reverse,
        fav: search.fav,
        filter: search.$,
        titleOnly: titleOnly,
        showOnlyPending: showOnlyPending
      };
      localStorageService.set('jobData.sorting', storeSort);
    };

    // Active item
    service.setActiveItem = function (jobId, parent, objId) {
      var storeActive = localStorageService.get('jobData.activeItems') || {};
      storeActive[jobId] = storeActive[jobId] || {};
      if (objId) {
        storeActive[jobId][parent] = objId;
        localStorageService.set('jobData.activeItems', storeActive);
      }
      return storeActive[jobId][parent];
    };

    service.getActiveItem = function (jobId, parent) {
      return service.setActiveItem(jobId, parent);
    };

    // Favorites
    service.toggleFavorite = function (jobId, objId) {
      var storeFav = localStorageService.get('jobData.favorites') || {};
      storeFav[jobId] = storeFav[jobId] || {};
      storeFav[jobId][objId] = storeFav[jobId][objId] ? false : true;
      localStorageService.set('jobData.favorites', storeFav);
      return storeFav[jobId][objId];
    };

    service.getFavorite = function (jobId, objId) {
      var storeFav = localStorageService.get('jobData.favorites') || {};
      storeFav[jobId] = storeFav[jobId] || {};
      return storeFav[jobId][objId] || false;
    };

    // Info pane state
    service.toggleInfoState = function (jobId) {
      var deferred = $q.defer();
      var storedState = localStorageService.get('jobData.infoPaneExpanded') || {};
      storedState[jobId] = storedState[jobId] === false ? true : false;
      localStorageService.set('jobData.infoPaneExpanded', storedState);
      deferred.resolve(storedState[jobId]);
      return deferred.promise;
    };

    service.getInfoState = function (jobId, objId) {
      var deferred = $q.defer();
      var storedState = localStorageService.get('jobData.infoPaneExpanded') || {};
      deferred.resolve(storedState[jobId] === false ? false : true);
      return deferred.promise;
    };

    // Quick edit form filter
    service.setQuickEditFieldsMap = function (jobId, level, fieldsMap) {
      var quickEdit = localStorageService.get('jobData.quickEdit') || {};
      quickEdit[jobId] = quickEdit[jobId] || {};
      quickEdit[jobId][level] = fieldsMap || [];
      localStorageService.set('jobData.quickEdit', quickEdit);
    };

    service.getQuickEditFieldsMap = function (jobId, level) {
      var quickEdit = localStorageService.get('jobData.quickEdit') || {};
      quickEdit[jobId] = quickEdit[jobId] || {};
      return quickEdit[jobId][level] || [];
    };

    // Table view config
    service.setTableViewConfig = function (jobId, level, config) {
      var storedConfig = localStorageService.get('jobData.tableViewConfig') || {};
      storedConfig[jobId] = storedConfig[jobId] || {};
      storedConfig[jobId][level] = config || [];
      localStorageService.set('jobData.tableViewConfig', storedConfig);
      return storedConfig[jobId][level];
    };

    service.getTableViewConfig = function (jobId, level) {
      var storedConfig = localStorageService.get('jobData.tableViewConfig') || {};
      storedConfig[jobId] = storedConfig[jobId] || {};
      return storedConfig[jobId][level] || [];
    };

    // Table view state (on/off)
    service.setTableViewStatus = function (jobId, level, state) {
      var storedStatus = localStorageService.get('jobData.tableViewStatus') || {};
      storedStatus[jobId] = storedStatus[jobId] || {};
      storedStatus[jobId][level] = state || false;
      localStorageService.set('jobData.tableViewStatus', storedStatus);
      return storedStatus[jobId][level];
    };

    service.getTableViewStatus = function (jobId, level) {
      var storedStatus = localStorageService.get('jobData.tableViewStatus') || {};
      storedStatus[jobId] = storedStatus[jobId] || {};
      return storedStatus[jobId][level] || false;
    };

    // Add new item 'numChildsToAdd' - mark as used
    service.usedNumChildsToAdd = function (jobId, objId) {
      var storeUsed = localStorageService.get('jobData.usedNumChilds') || {};
      storeUsed[jobId] = storeUsed[jobId] || {};
      storeUsed[jobId][objId] = true;
      localStorageService.set('jobData.usedNumChilds', storeUsed);
    };

    service.getUsedNumChildsToAdd = function (jobId, objId) {
      var storeUsed = localStorageService.get('jobData.usedNumChilds') || {};
      storeUsed[jobId] = storeUsed[jobId] || {};
      return storeUsed[jobId][objId] || false;
    };

    var getRequiredIds = function (data, fod) {
      var ids = {};
      if (!(Array.isArray(fod.PropDefList.PropDef))) {
        fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
      }

      $filter('filter')(fod.PropDefList.PropDef, {isRequired: '1'}, true).forEach(function (pd) {
        ids[pd.id] = pd.id;
      });
      return ids;
    };

    service.setFieldObjProp = function (fieldObj, parentValues) {
      if (parentValues && Object.keys(parentValues).length > 0 && fieldObj.PropertiesList && fieldObj.PropertiesList.Property && 'isNewItem' in fieldObj && fieldObj.isNewItem == 1) {
        fieldObj.PropertiesList.Property.forEach(function (parentProp) {
          if ('inheritedValue' in parentProp && parentProp.inheritedValue === true && parentProp.PropId in parentValues) {
            parentProp.ValuesList.Value.Value = parentValues[parentProp.PropId];
            parentProp.ValuesList.Value.displayStyle = 1;
          }
        });
      }
      service.fieldObj = fieldObj;
      return fieldObj;
    };

    service.setJobFieldChildes = function (jobId, fid) {
      var deferred = $q.defer();
      service.getSortedList(jobId, fid, false, false).then(function (objs) {
        service.objs = objs;
        deferred.resolve();
      });
      return deferred.promise;
    };



    service.GetFormToCache = function (fod, jobId, decisions, propList) {
      var objForm = [];
      if (!(Array.isArray(fod.PropDefList.PropDef))) {
        fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
      }

      if (fod.PropDefList.PropDef.length) {
        //  get the properties list
        var serialNumber = 0;
        var filterStr = {};
        filterStr.isInput = '1';
        if (decisions) {
          filterStr.isDescision = '1';
        }
        else {
          filterStr.isDescision = '!1';
        }
        if (!(Array.isArray(fod.PropDefList.PropDef))) {
          fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
        }

        $filter('filter')(fod.PropDefList.PropDef, filterStr, true).forEach(function (pd) {
          try {
            var fieldForm = {};
            fieldForm.required = false;
            fieldForm.odefId = fod.ScObjDefId;
            fieldForm.title = pd.name;
            fieldForm.pd = parseInt(pd.id);
            fieldForm.id = jobId;
            fieldForm.key = pd.id;
            // fieldForm.onChange = 'updated(modelValue,form)';
            // fieldForm.onClick = "clickHandler($event, form)";

            fieldForm.serialNumber = ++serialNumber;

            if ('Units' in pd) {
              fieldForm.unit = pd.Units;
            }
            else {
              fieldForm.unit = '';
            }

            if ('isRequired' in pd) {
              if (parseInt(pd.isRequired) == 1) {
                fieldForm.required = true;
              }
            }

            switch (parseInt(pd.fieldType)) {
              case 1:
                // Range numeric (from to)
                if ('minValue' in pd) {
                  fieldForm.minimum = parseFloat(pd.minValue);
                }
                if ('maxValue' in pd) {
                  fieldForm.maximum = parseFloat(pd.maxValue);
                }
                if ('precision' in pd) {
                  fieldForm.precision = parseFloat(pd.precision);
                }

                fieldForm.type = 'range';
                fieldForm.inputType = 'tel';
                break;
              case 2:
                // Number
                if ('minValue' in pd) {
                  fieldForm.minimum = parseFloat(pd.minValue);
                }
                if ('maxValue' in pd) {
                  fieldForm.maximum = parseFloat(pd.maxValue);
                }
                if ('precision' in pd) {
                  fieldForm.precision = parseFloat(pd.precision);
                }
                fieldForm.type = 'number';
                fieldForm.inputType = 'tel';
                break;
              case 3:
                // Select list
                if (pd.selectionValueList && pd.selectionValueList.valueEntry) {

                  var options = [];
                  // Add empty option as default;
                  if (!fieldForm.required) {
                    var optionNone = {value: '', name: ''};
                    options.unshift(optionNone);
                    fieldForm.currVal = fieldForm.currVal || '';
                  }

                  var option;
                  if (angular.isDefined(pd.selectionValueList) && angular.isArray(pd.selectionValueList.valueEntry) && pd.selectionValueList.valueEntry.length) {
                    pd.selectionValueList.valueEntry.forEach(function (selectVal) {
                      if ('fullValue' in selectVal) {
                        option = {value: selectVal.key, name: selectVal.fullValue, smallValue : selectVal.value };
                      }
                      else {
                        option = {value: selectVal.key, name: selectVal.value};
                      }
                      options.push(option);
                    });
                  }
                  else {
                    if (pd.selectionValueList.valueEntry.key && pd.selectionValueList.valueEntry.value) {
                      if ('fullValue' in pd.selectionValueList.valueEntry) {
                        option = {
                          value: pd.selectionValueList.valueEntry.key,
                          name: pd.selectionValueList.valueEntry.fullValue,
                          smallValue : pd.selectionValueList.valueEntry.value
                        };
                      }
                      else {
                        option = {
                          value: pd.selectionValueList.valueEntry.key,
                          name: pd.selectionValueList.valueEntry.value
                        };
                      }

                      options.push(option);
                    }
                  }
                  fieldForm.type = 'select';
                  fieldForm.titleMap = options;

                }
                break;
              case 4:
                var imgDir;
//                cordova.file.externalApplicationStorageDirectory

                try {
                  imgDir = cordova.file.dataDirectory + "ib/images/" + jobId + "/" + pd.id + "/";
                }
                catch (e) {
                }
//                var imgDir = cordova.file.dataDirectory + "/";
                var imagesOptions = [];
                var imageOption;
                fieldForm.inputType = 'text';
                fieldForm.type = 'images';

//                selectionValueList: ObjectvalueEntry: Array[3]0: Object1: Objectkey: "3"pictureName: "fig-interior-G.jpg"value: "int. G"

                if (angular.isArray(pd.selectionValueList.valueEntry) && pd.selectionValueList.valueEntry.length) {


                  var rootDir;
                  if ('picturesFolderURL' in pd) {
                    rootDir = pd.picturesFolderURL;
                  }
                  else {
                    rootDir = propList.picturesFolderURL;
                  }
                  if (rootDir.substr(-1) !== "/") {
                    rootDir = rootDir + "/";


                  }


                  pd.selectionValueList.valueEntry.forEach(function (selectVal) {
                    try {
                      if (angular.isDefined($cordovaFileTransfer)) {
                        $cordovaFileTransfer.download(encodeURI(rootDir + selectVal.pictureName), imgDir + selectVal.pictureName, function (file) {
                        }, function (error) {
                          console.log(error);
                        });
                        imageOption = {
                          value: selectVal.key,
                          webPath: rootDir + selectVal.pictureName,
                          path: imgDir + selectVal.pictureName,
                          title: selectVal.value
                        };
                      }
                      else {
                        imageOption = {
                          value: selectVal.key,
                          webPath: rootDir + selectVal.pictureName,
                          path: false,
                          title: selectVal.value
                        };
                      }
                    }
                    catch (e) {
                      imageOption = {
                        value: selectVal.key,
                        webPath: rootDir + selectVal.pictureName,
                        path: false,
                        title: selectVal.value
                      };

                      $log.error(e);

                    }

                    imagesOptions.push(imageOption);

                  });
                }
                else {
                  if (pd.imagesList.valueEntry.key && pd.imagesList.valueEntry.value) {
                    try {
                      $cordovaFileTransfer.download(pd.PictureFolder + pd.imagesList.valueEntry.value, imgDir + pd.imagesList.valueEntry.value);
                      imageOption = {
                        value: pd.imagesList.valueEntry.key,
                        webPath: pd.PictureFolder + pd.imagesList.valueEntry.value,
                        path: imgDir + pd.imagesList.valueEntry.value,
                        title: pd.imagesList.valueEntry.title
                      };
                    }
                    catch (e) {
                      imageOption = {
                        value: pd.imagesList.valueEntry.key,
                        webPath: pd.PictureFolder + pd.imagesList.valueEntry.value,
                        path: false,
                        title: pd.imagesList.valueEntry.title
                      };
                      $log.error(e);
                    }
                    imagesOptions.push(imageOption);
                  }
                }
                fieldForm.imagesList = imagesOptions;
                break;
              case 5:
                //Checkbox
                fieldForm.type = 'boolean';
                if (angular.isDefined(fieldForm.default)) {
                  if (fieldForm.default.toLowerCase() === 'true' || fieldForm.default.toLowerCase() === 'yes') {
                    fieldForm.default = true;
                  }
                }
                break;
              case 6:
                // One line text
                fieldForm.inputType = 'string';
                fieldForm.type = 'string';
                fieldForm.width = 'half';
                if ('maxValue' in pd) {
                  fieldForm.maxLength = pd.maxValue;
                }
                if ('scanBarcode' in pd) {
                  if (pd.scanBarcode == "yes" || pd.scanBarcode == true || pd.scanBarcode == "true") {
                    pd.scanBarcode = true;
                  }
                  else {
                    pd.scanBarcode = false;
                  }
                  fieldForm.scanBarcode = pd.scanBarcode;
                }
                else {
                  fieldForm.scanBarcode = false;
                }
                break;
              case 7:
                // Long text
                if ('maxValue' in pd) {
                  fieldForm.maxLength = pd.maxValue;
                }

                fieldForm.type = 'string';
                fieldForm.placeholder = gettextCatalog.getString('Make a comment');
                fieldForm.type = 'textarea';
                break;
              case 8:
                fieldForm.type = 'date';
                break;
            }
            objForm.push(fieldForm);
          } catch (e) {
            $log.error(e);
          }
        });
      }

      return objForm;
    };

    service.getFormFieldInfo = function (fieldObj) {
      var formFieldInfo = {};
      if ('isNewItem' in fieldObj && fieldObj.isNewItem == 1) {
        formFieldInfo.isNew = fieldObj.isNewItem;
        formFieldInfo.pfid = fieldObj.ParentUniqueId;
        formFieldInfo.userFid = fieldObj.ObjectName;
        formFieldInfo.odefId = fieldObj.ScObjDefId;
      }
      else {
        formFieldInfo.isNew = 0;
        formFieldInfo.pfid = 0;
        formFieldInfo.userFid = '';
        formFieldInfo.odefId = 0;
      }
      formFieldInfo.fid = fieldObj.ObjectUniqueId;
      return formFieldInfo;
    };

    service.getProcessedForm = function (jobId, type) {
      var deferred = $q.defer();
      DbService.getJobFormByType(jobId, service.fieldObj.ScObjDefId, type).then(function (form) {
        DbService.getFieldObjectValues(service.fieldObj.ObjectUniqueId).then(function (values) {
          var prop, val;
          form.forEach(function (formField) {
            try {
              formField.oldval = '';
              formField.uniqueIndex = service.fieldObj.ObjectUniqueId.toString() + formField.key;
              if (!(Array.isArray(service.fod.PropDefList.PropDef))) {
                service.fod.PropDefList.PropDef = [service.fod.PropDefList.PropDef];
              }

              var pd = $filter('filter')(service.fod.PropDefList.PropDef, {id: formField.key}, true)[0];
              if (service.fieldObj && service.fieldObj.PropertiesList) {
                var items = $filter('filter')(service.fieldObj.PropertiesList.Property, {PropId: formField.key}, true);
                if (items) {
                  prop = $filter('filter')(service.fieldObj.PropertiesList.Property, {PropId: formField.key}, true)[0];
                }
              }
              // History value/value-list
              if (prop && prop.ValuesList && prop.ValuesList.Value) {
                val = getPropValue(prop.ValuesList.Value);

                formField.oldval = processPropValue(pd, val);
                // History
                formField.historyList = false;
                if (prop.ValuesList.Value.length > 1) {
                  formField.historyList = parseHistoryList(pd, prop);
                }
                // displayStyle
                if ('displayStyle' in prop.ValuesList.Value) {
                  formField.displayStyle = prop.ValuesList.Value.displayStyle;
                }
              }
              // M/F Parents values.
              if (formField.oldval === '' || formField.oldval === '-') {
                formField.maleParentVal = processPropValue(pd, getParentValue(prop, 'male'));
                formField.femaleParentVal = processPropValue(pd, getParentValue(prop, 'female'));
                formField.femaleParentDisplayStyle = getParentDisplayStyle(prop, 'female');
                formField.maleParentDisplayStyle = getParentDisplayStyle(prop, 'male');
              }

              if (formField.key in values) {
                formField.currVal = values[formField.key];

              }

              if (pd) {
                switch (parseInt(pd.fieldType)) {
                  case 2:
                    //Float
                    if (pd.id in values) {
                      formField.currVal = parseFloat(values[pd.id]);
                    }
                    if (pd.precision) {
                      formField.precision = parseInt(pd.precision, 10);
                    }
                    break;
                  case 3:
                    formField.displayValue = formField.titleMap.find(function(item){return item.value == formField.currVal}).name
                    break;
                  case 4:
                    // Select
                    for (var i = 0; i < formField.imagesList.length; i++) {
                      if (formField.imagesList[i].value == formField.currVal) {
                        formField.currVal = formField.imagesList[i].title;
                        break;
                      }
                    }

                    break;
                  case 5:
                    //Checkbox
                    if (formField.currVal === 'true' || formField.currVal === 'yes') {
                      formField.currVal = true;
                    }
                    break;
                  case 8:
                    // Date
                    if (pd.id in values) {
                      formField.currVal = new Date(values[pd.id] * 1000);
                    }
                    break;
                }
              }

            } catch (e) {
              $log.error(e);
            }
          });
          deferred.resolve(form);
        });
      });
      return deferred.promise;
    };

    service.checkField = function (job, fid, fod) {
      var deferred = $q.defer();
      var ids = getRequiredIds(job.data, fod);
      var idsKeys = Object.keys(ids);
      var status;
      DbService.getFieldObjectValues(fid).then(function (values) {
        var valuesKeys = Object.keys(values);
        if (idsKeys.length > 0) {

          var diff = idsKeys.diff(valuesKeys);
          if (diff.length > 0) {
            if (valuesKeys.length > 0) {
              status = FIELD_STATUS_IN_PROG;
            }
            else {
              status = FIELD_STATUS_PEND;
            }
          }
          else {
            status = FIELD_STATUS_COMP;
          }

        }
        else {
          if (valuesKeys.length > 0) {
            status = FIELD_STATUS_COMP;
          }
          else {
            status = FIELD_STATUS_PEND;
          }
        }
        DbService.updateJobField(fid, status);
        checkJob(job.id);
        deferred.resolve(status);
      });
      return deferred.promise;
    };

		/**
		 * If this is the last item, get the
		 * first one from the next tree.
		 */
		service.getNextItem = function(JobsFactory) {
			var deferred = $q.defer();
			var search = $location.search();
			var parent = service.findObject(JobsFactory.job.data.ScoredObjectsList.ScoredObject, search.parentId)
			var sParrent = service.getObjectParent(parent);
			service.getSortedList(JobsFactory.job.id, sParrent.ParentUniqueId, false).then(function (res) {
				for(var i = 0; i < res.length ; i++) {
					if(res[i].id == parent.ObjectUniqueId) {
						if(i <= res.length - 2) {
							var nextUncle = res[i+1];
							service.getSortedList(JobsFactory.job.id, nextUncle.id, false).then(function (res) {
								var result = false;
								if(res.length > 0) {
									result = {
										parentId: res[0].ParentUniqueId,
										objId: res[0].id,
										parent: parent,
									};
								}
								deferred.resolve(result);
							})
						}
					}
				}
			});
			return deferred.promise;
		}

		/**
		 * If this is the last item, get the
		 * first one from the next tree.
		 */
		service.getPrevItem = function(JobsFactory) {
			var deferred = $q.defer();
			var search = $location.search();
			var parent = service.findObject(JobsFactory.job.data.ScoredObjectsList.ScoredObject, search.parentId)
			var sParrent = service.getObjectParent(parent);
			service.getSortedList(JobsFactory.job.id, sParrent.ParentUniqueId, false).then(function (res) {
				for(var i = 0; i < res.length ; i++) {
					if(res[i].id == parent.ObjectUniqueId) {
						if(i != 0) {
							var prevUncle = res[i-1];
							service.getSortedList(JobsFactory.job.id, prevUncle.id, false).then(function (res) {
								var result = false;
								if(res.length > 0) {
									result = {
										parentId: res[res.length - 1].ParentUniqueId ,
										objId: res[res.length - 1].id,
										parent: parent,
									};
								}
								deferred.resolve(result);
							})
						}
					}
				}
			});
			return deferred.promise;
		}

    function getNextAvailableID(id, parent) {
      var found = true,
        uniqueId;
      id = isNaN(id) ? parseInt(id) : id;
      if (!parent || !parent.NestedObjList || !parent.NestedObjList.ScoredObject) {
        return id;
      }
      // Search if 'available' exist in 'parent'
      while (angular.isDefined(found)) {
        uniqueId = id;
        found = parent.NestedObjList.ScoredObject.filter(function (obj) {
          return obj.ObjectUniqueId == uniqueId;
        })[0];
        id += 1;
      }
      return uniqueId;
    }

    var inheritParentValues = function (childFod, fod, parent) {
      var propList = [];

      if (!('PropertiesList' in parent)) {
        return propList;
      }

      if (!(Array.isArray(childFod.PropDefList.PropDef))) {
        childFod.PropDefList.PropDef = [childFod.PropDefList.PropDef];
      }

      childFod.PropDefList.PropDef.forEach(function (childProp) {
        if (!(Array.isArray(fod.PropDefList.PropDef))) {
          fod.PropDefList.PropDef = [fod.PropDefList.PropDef];
        }

        fod.PropDefList.PropDef.forEach(function (parentProp) {
          // Parent-child prop match
          if (childProp.id == parentProp.id) {
            var propValue = '';
            var parentFieldProp = $filter('filter')(parent.PropertiesList.Property, {PropId: childProp.id}, true)[0];

            if (parentFieldProp && parentFieldProp.ValuesList && parentFieldProp.ValuesList.Value) {
              propValue = getPropValue(parentFieldProp.ValuesList.Value);
            }

            var MaleParentValue = getParentValue(parentFieldProp, 'male');
            var FemaleParentValue = getParentValue(parentFieldProp, 'female');
            var FemaleParentDisplayStyle = getParentDisplayStyle(parentFieldProp, 'female');
            var MaleParentDisplayStyle = getParentDisplayStyle(parentFieldProp, 'male');

            var newProp = {
              'PropId': childProp.id,
              'MaleParentDisplayStyle': MaleParentDisplayStyle,
              'FemaleParentDisplayStyle': FemaleParentDisplayStyle,
              'MaleParentValue': MaleParentValue,
              'FemaleParentValue': FemaleParentValue,
              'inheritedValue': true,
              'ValuesList': {
                'Value': {
                  'InputDate': '0',
                  'displayStyle': '0',
                  'Value': propValue
                }
              }
            };
            propList.push(newProp);
          }
        });
      });
      return propList;
    };

    service.getRelatedItems = function(jobId) {
      var self = this;
      var deferred = $q.defer();
      if (self.relatedPlots != false) {
        self.getSortedList(jobId, 0, false, false).then(function (list) {
          list = $filter('filter')(list , function(value, index, array) {
            var item = $filter('filter')(self.relatedPlots, {RelatedPlotId : value.ObjectId})
            if (item.length) {
              return true;
            }
          });
          deferred.resolve(list);
        });
      }
      else {
        deferred.resolve(false);
      }
      return deferred.promise;
    }

    service.enterRelatedMode = function (id) {
      service.relatedMode = id;
    }

    service.getRelatedMode = function () {
      return service.relatedMode;
    }

    service.exitRelatedMode = function () {
      service.relatedMode = false;
    }

    service.setBC = function (bc) {
      var bcArr = []
      bc.forEach(function(b) {
        bcArr.push(b.ObjectId.trim());
      })
      service.bc = bcArr;
    }
    service.getBC = function() {
      return service.bc;
    }


    service.addSubDirs = function(path, dirs, dirA, cb) {
      if (dirs.length) {
        dirA.push(dirs.shift());
        service.checkDir(path, dirA.join("/")).then(function() {
          service.addSubDirs(path, dirs, dirA, cb)
        })
      }
      else {
        cb(dirA.join("/"));
      }
    }

    service.checkDirs = function (path, dirs) {
      var deferred = $q.defer();
      service.addSubDirs(path, dirs, [], function(finalPath) {
        deferred.resolve(finalPath);
      })
      return deferred.promise;
    }

    service.checkDir = function(path, dir) {
      var deferred = $q.defer();
      $cordovaFile.checkDir(path, dir).then(function (success) {
        deferred.resolve(true);
      }, function(err) {
        $cordovaFile.createDir(path, dir).then(function(res) {
          deferred.resolve(true);
        })
      });;
      return deferred.promise;
    }

    service.addNewObjects = function (job, parentID, ObjectUniqueIds) {
      var deferred = $q.defer();
      var parent = findObject(job.data.ScoredObjectsList.ScoredObject, parentID);
      var fod = $filter('filter')(job.data.ScObjDefList.ScObjDef, {ScObjDefId: parent.ScObjDefId}, true)[0];
      var childFod = $filter('filter')(job.data.ScObjDefList.ScObjDef, {ScObjDefId: fod.ScObjChildDefId}, true)[0];
      var fields = [];
      var newObjs = [];
      var parentValues = [];
      if (childFod) {
        parentValues = inheritParentValues(childFod, fod, parent);
      }
      for (var i = 0; i < ObjectUniqueIds.length; i++) {
        var newObj = {
          ObjectUniqueId: getNextAvailableID(parseInt(Date.now() / 1000), parent),
          ParentUniqueId: parentID,
          ObjectName: ObjectUniqueIds[i],
          ObjectId: ObjectUniqueIds[i],
          isNewItem: 1,
          ScObjDefId: fod.ScObjChildDefId,
          Level: parseInt(parent.Level) + 1,
          PropertiesList: {}
        };

        // Inherit value from parent
        newObj.PropertiesList.Property = parentValues;
        // Default value
        if (!(Array.isArray(childFod.PropDefList.PropDef))) {
          childFod.PropDefList.PropDef = [childFod.PropDefList.PropDef];
        }

        childFod.PropDefList.PropDef.forEach(function (childProp) {
          if ('defaultValue' in childProp) {
            DbService.insertJobValue(job.id, newObj.ObjectUniqueId, childProp.id, Date.now() / 1000, childProp.defaultValue, parentID, ObjectUniqueIds[i], fod.ScObjChildDefId);
          }
        });

        if (parent.NestedObjList) {
          if (Array.isArray(parent.NestedObjList)) {
            parent.NestedObjList = {};
          }
        }
        else {
          parent.NestedObjList = {};
        }
        parent.NestedObjList.ScoredObject = parent.NestedObjList.ScoredObject || [];
        parent.NestedObjList.ScoredObject.push(newObj);
        fields.push([fod.ScObjChildDefId, newObj.ObjectUniqueId, job.id, parentID, parseInt(parent.Level) + 1, newObj.ObjectName]);
        newObjs.push(newObj);
      }
      DbService.updateJobData(job.id, JSON.stringify(job.data)).then(function () {
        DbService.insertJobFields(fields, 1);
        DbService.updateJobSyncStatus(job.id, 0).then(function () {
          deferred.resolve(newObjs);
        });
      });
      return deferred.promise;
    };

    return service;
  });
