'use strict';
angular.module('ib.services')
  .service('LogSelectDbService', function($window, $q, LogDbService) {
    var LOG_DB_NAME = "iblog";
    var _db;
    this.run = function() {
      createDb();
    }
    var createDb = function() {
      try {
        _db = $window.sqlitePlugin.openDatabase({ name: LOG_DB_NAME});
      } catch (e) {
        _db = $window.openDatabase(LOG_DB_NAME, '1.0', 'IB', 10 * 3072 * 1024);
      }
    }
    this.deleteLogMsg = function(id) {
      var query = "DELETE FROM ib_log where id = ?";
      // LogDbService.db.transaction(function(tx) {
      //   tx.executeSql(query,[id], dbSuccess, dbError);
      // });
      var dbSuccess = function(tx, results) {
      }
      var dbError = function(e) {
        console.log('error in create db');
      }
    }
    this.getLogs = function() {
      var deferred = $q.defer();
      var query = "SELECT id, logMsg FROM ib_log";
      // LogDbService.db.transaction(function (tx) {
      //   tx.executeSql(query, [], dbSuccess, dbError);
      // });
      var dbSuccess = function(tx, results) {
        var len = results.rows.length, i;
        var items = [];
        for (i = 0; i < len; i++){
          items.push({'id' : results.rows.item(i).id, 'logMsg' : JSON.parse(results.rows.item(i).logMsg)});
        }
        deferred.resolve(items);
      }
      var dbError = function(e) {
        console.log('error in getLogs');
        deferred.resolve([]);
      }
      return deferred.promise;
    }
  })
  .service('LogDbService', function($window) {
    var LOG_DB_NAME = "iblog";
    this.run = function(needUpdate) {
      this.createDb(needUpdate);
    }
    this.createDb = function(needUpdate) {
      try {
        this.db = $window.sqlitePlugin.openDatabase({ name: LOG_DB_NAME});
      } catch (e) {
        this.db = $window.openDatabase(LOG_DB_NAME, '1.0', 'IB', 10 * 3072 * 1024);
      }
      this.createTables(needUpdate);
    }
    this.createTables = function(needUpdate) {
      this.db.transaction(function(tx) {
        if (needUpdate) {
          tx.executeSql("drop table ib_log", []);
        }
        tx.executeSql("CREATE TABLE IF NOT EXISTS ib_log(id integer primary key, logMsg text)", []);
      })
    }
    this.insertLogMsg = function(logMsg) {
      var query = "INSERT INTO ib_log (logMsg) VALUES (?)";
      this.db.transaction(function(tx) {
        tx.executeSql(query,[logMsg], dbSuccess, dbError);
      });
      var dbSuccess = function(tx, results) {
      }
      var dbError = function(e) {
        console.log('error in insertLogMsg');
      }
    }
  })
  .service('logService', function($rootScope, $http, $q, LogSelectDbService) {
    this.sync = function() {
      var deferred = $q.defer();
      var syncProm = [];
      LogSelectDbService.getLogs().then(function(results) {
        if (results.length > 0) {
          results.forEach(function (logO) {
            syncProm.push(_sync(logO));
          });
        }
        $q.all(syncProm).then(function () {
          deferred.resolve(true);
        });

      });
      return deferred.promise;
    }
    var _sync = function(logO) {
      var deferred = $q.defer();
      sendLM(logO).then(function() {
        deferred.resolve(true);
      });
      return deferred.promise;
    }
    var sendLM = function(logO) {
      var deferred = $q.defer();
      $http.post($rootScope.graylogEndPoint , logO.logMsg,
                {timeout: 300000, warningAfter: 5000, ignoreAuthModule: true ,headers: {'Content-Type' : 'text/plain', 'Access-Control-Request-Headers' : undefined, 'X-CSRF-Token': undefined}, 'withCredentials' : false})
        .success(function (data, status, headers, config) {
          if (status == 202) {
            LogSelectDbService.deleteLogMsg(logO.id);
          }
          deferred.resolve();
        }).error(function (data, status, headers, config) {
          deferred.resolve();
        });
      return deferred.promise;
    }
  }).service('$log', ['$window', 'LogDbService', function($window, LogDbService) {
    var logToConsole = true;
    var logToServer = true;
    var logAll = true;
    var logLevels = {log : 6, debug : 7, warn : 4, info : 1, error : 3, custom : 6};
    var extraDef = {};

    this.setExtraDef = function(ibVersion, ibPackage) {
      extraDef.version = '1.1';
      extraDef.host = "BreedIT Mobile";
      extraDef._appVersion = ibVersion;
      extraDef._appPackage =  ibPackage;
      if ('device' in $window) {
        extraDef._deviceModel = $window.device.model;
        extraDef._devicePlatform = $window.device.platform;
        extraDef._deviceUUID = $window.device.uuid;
        extraDef._deviceVersion = $window.device.version;
      }
      else {
        extraDef._deviceModel = $window.navigator.userAgent;
        extraDef._devicePlatform = $window.navigator.platform;
        //@TODO deviceData.uuid = set once on localStorage from timestamp;
        extraDef._deviceUUID = 'yuval-pc';
        extraDef._deviceVersion = $window.navigator.appVersion;
      }
    }
    this.overrideExtraDef = function(data) {
      angular.forEach(data, function(value, key) {
        this[key] = value;
      }, extraDef);
    }
    var createLogMessage = function(msg, type, extra, longMessage) {
      var gelfMsg;
      try {
        extra = extra || {};
        longMessage = longMessage || false;
        var level = logLevels[type] || 1;
        var timestamp = Date.now() / 1000;
        gelfMsg = {'timestamp' : timestamp, 'version' : extraDef.version, 'host' : extraDef.host, 'short_message' : msg, 'level' : level};
        if (longMessage) {
          gelfMsg.full_message = longMessage;
        }
        angular.forEach(extraDef, function(value, key) {
          this[key] = value;
        }, gelfMsg);
        angular.forEach(extra, function(value, key) {
          this[key] = value;
        }, gelfMsg);
      }
      catch (e) {
        console.log(e);
        return false;
      }
      return gelfMsg;
    }
    var writeLogMessage = function(msg, type, extra, longMessage) {
      try {
        if (logToConsole) {
          if (type == 'error') {
            console.log(msg);
          }
        }
        if (logToServer) {
          if (logAll || type in logLevels) {
            var gelfMsg = createLogMessage(msg, type, extra, longMessage);
            if (gelfMsg) {
              // LogDbService.insertLogMsg(JSON.stringify(gelfMsg));
            }
          }
        }
      }
      catch (e) {
        console.log(e);
      }
    }
    this.log = function(msg, extra, longMessage) {
      writeLogMessage(msg, 'log', extra, longMessage);
    };
    this.debug = function(msg, extra, longMessage) {
      writeLogMessage(msg, 'debug', extra, longMessage);
    };
    this.info = function(msg, extra, longMessage) {
      writeLogMessage(msg, 'info', extra, longMessage);
    };
    this.warn = function(msg, extra, longMessage) {
      writeLogMessage(msg, 'warn', extra, longMessage);
    };
    this.error = function(e, extra) {
      console.log(e.stack);
      writeLogMessage(e.message, 'error', extra);
    };
    this.custom = function(msg, type, extra, longMessage) {
      type = type || 'custom';
      writeLogMessage(msg, type, extra, longMessage);
    }
  }]).factory('logToServerInterceptor', ['$q', '$log', '$rootScope', function ($q, $log, $rootScope) {
    return {
      'request': function (config) {
        if (config.url.indexOf('http') == -1 || config.url.indexOf($rootScope.graylogEndPoint) != -1) {
          return config;
        }
        config.msBeforeAjaxCall = new Date().getTime();

        var longMessageO = {};
        var longMessage;
        longMessageO.config = config;
        longMessage = JSON.stringify(longMessageO);
        $log.log("httpReq", {'_method' : config.method, '_url' : config.url, '_logGroup' : 'http'}, longMessage);
        return config;
      },
      'response': function (response) {
        try {
          if (response.config.url.indexOf('http') == -1 || response.config.url.indexOf($rootScope.graylogEndPoint) != -1) {
            return response;
          }
          var msAfterAjaxCall = new Date().getTime();
          var timeTakenInMs =
            msAfterAjaxCall - response.config.msBeforeAjaxCall;
          $log.log('httpReq success', {'_method' : response.config.method, '_url' : response.config.url, '_timeTakenInMs': timeTakenInMs, '_logGroup' : 'http'});
          if (response.config.warningAfter) {
            if (timeTakenInMs > response.config.warningAfter) {
              $log.warn('httpReq long request warn', {'_method' : response.config.method, '_url' : response.config.url, '_timeTakenInMs': timeTakenInMs, '_logGroup' : 'http'});
            }
          }
        } catch (e) {
          $log.error(e);
        }

        return response;
      },
      'responseError': function (rejection) {
        try {
          if (rejection.config.url.indexOf('http') == -1 || rejection.config.url.indexOf($rootScope.graylogEndPoint) != -1) {
            return $q.reject(rejection);
          }
          var msAfterAjaxCall = new Date().getTime();
          var timeTakenInMs =
            msAfterAjaxCall - rejection.config.msBeforeAjaxCall;
          var extra = {'_method' : rejection.config.method, '_url' : rejection.config.url, '_timeTakenInMs': timeTakenInMs, '_logGroup' : 'http'};
          var msg = 'http request error status ' + rejection.status;
          extra._status = status;
          if ('statusText' in rejection) {
            extra._statusText = rejection.statusText;
            msg += ' : ' + rejection.statusText;
          }
          var longMessageO = {};
          var longMessage;
          longMessageO.config = rejection.config;
          if ('data' in rejection && Array.isArray(rejection.data)) {
            longMessageO.data = rejection.data;
          }
          longMessage = JSON.stringify(longMessageO);
          if (rejection.status == 0) {
            extra._timeout = true;
          }
          $log.custom(msg, 'httpError', extra, longMessage);
        } catch (e) {
          $log.error(e);
        }
        return $q.reject(rejection);
      }
    };
  }]).factory('$exceptionHandler', ['$log', function($log) {
    return function(exception, cause) {
      try {
        if (!exception.processed || exception.processed == false) {
          exception.message += ' (caused by "' + cause + '")';
          exception.processed = true;
          $log.error(exception);
        }

      } catch (e) {
        console.log(e);
      }
      throw exception;
    };
  }]);
