"use strict";
angular.module('ib.services')
  .service('RestService', function ($rootScope, $http, $filter, $q, $log, $cordovaNetwork, localStorageService, DbService, FieldObjFactory, $cordovaFile) {


    var JOB_STATUS_NEW = 0,
      JOB_STATUS_IN_PROG = 1,
      JOB_STATUS_COMPLETE = 2,
      JOB_STATUS_DONE = 3,
      JOB_STATUS_SYNC = 4,
      FIELD_STATUS_PEND = 0,
      FIELD_STATUS_IN_PROG = 1,
      FIELD_STATUS_COMP = 2;

    this.setHttp = function () {
      var token = localStorageService.get('token');
      if (token) {
        $http.defaults.headers.common['X-CSRF-Token'] = token;  // Step 1
        $http.defaults.headers.put['X-CSRF-Token'] = token;  // Step 1
      }
    }
    this.getFieldsList = function (id, fieldsList, data) {
      return getFieldsList(id, fieldsList, data);
    }


    var getFieldsList = function (id, fieldsList, data) {
      var processedList = [];
      fieldsList.forEach(function (fid) {
        var fod;
        if (fid == 0) {
          fod = FieldObjFactory.getFod(data, fid);
        }
        else {
          fod = FieldObjFactory.getChildFod(data, fid);
        }
        processedList.push([fid, id, JSON.stringify(FieldObjFactory.getObjectsList(data, fid, fod, null))]);
      });
      return processedList;
    }


    var getJobDataFromServer = function (url) {
      var deferred = $q.defer();

      try {
        var status = false;
        $http.get(url, {timeout: 300000, warningAfter: 5000, 'withCredentials': false, handleError: true})
          .success(function (data, status, headers, config) {
            var dataStr = JSON.stringify(data);
            console.log(data);
            console.log(dataStr);

            if ($rootScope.mode == "mobile") {
              $cordovaFile.writeFile(cordova.file.dataDirectory, data.jobNid + ".json", dataStr, true).then(function (result) {
                console.log(result);
                //Get file name
                insertJobData(deferred, data, "");
                // Success!
              }, function (err) {
                $log.error(err);
                deferred.resolve(false);
                // An error occured. Show a message to the user            });
              });
            }
            else {
              insertJobData(deferred, data, dataStr);
            }
          }).error(function (data, status, headers, config) {
          $log.warn("error in getJobDataFromServer data = '" + data + "' : status = '" + status + "'");
          deferred.resolve(false);
        })
      } catch (e) {
        deferred.resolve(true);
      }
      return deferred.promise;
    }


    var insertJobData = function (deferred, data, dataStr) {
      DbService.insertJob(data.jobNid, data.Date, dataStr, data.JobName, data.JobId,
        JSON.stringify(data.JobProperties), JOB_STATUS_NEW);
      var pFields = [];
      var parentsList = [];
      processJobFields(data.jobNid, data.ScoredObjectsList.ScoredObject, pFields, parentsList);
      DbService.insertJobFields(pFields, 0).then(function () {
        DbService.insertJobFieldsChildes(getFieldsList(data.jobNid, parentsList, data));
        DbService.insertJobForms(getFormsList(data.jobNid, data.ScObjDefList.ScObjDef, data.JobProperties.JobProperty));
        deferred.resolve(true);
      });

    }

    var getFormsList = function (id, fodList, propList) {
      var processedList = [];

      fodList.forEach(function (fod) {
        processedList.push([id, fod.ScObjDefId, JSON.stringify(FieldObjFactory.GetFormToCache(fod, id, false, propList)), 1]);
        processedList.push([id, fod.ScObjDefId, JSON.stringify(FieldObjFactory.GetFormToCache(fod, id, true, propList)), 2]);
      });
      return processedList;

    }

    this.getFormsList = function (id, fodList, propList) {
      return getFormsList(id, fodList, propList);
    }
    this.processJobFields = function (id, fields, pFields, parentsList) {
      return processJobFields(id, fields, pFields, parentsList);
    }
    var processJobFields = function (id, fields, pFields, parentsList) {
      if (fields) {
        for (var i = 0; i < fields.length; i++) {
          pFields.push([fields[i].ScObjDefId, fields[i].ObjectUniqueId, id, fields[i].ParentUniqueId, fields[i].Level, '']);
          if (parentsList.indexOf(fields[i].ParentUniqueId) == -1) {
            parentsList.push(fields[i].ParentUniqueId);
          }
          if ("NestedObjList" in fields[i]) {
            processJobFields([].concat(id), fields[i].NestedObjList.ScoredObject, pFields, parentsList);
          }
        }
      }
    }


    this.sendJobValuesToServer = function (nid) {
      var deferred = $q.defer();
      try {
        DbService.getJobValues(nid).then(function (jobValues) {
          DbService.getJobSyncedImages(nid).then(function (picValues) {
            var values = jobValues.rows.concat(picValues.rows);
            if (values.length > 0) {
              $http.put($rootScope.endPoint + '/appjob/' + nid + '.json', {rows: values}, {
                timeout: 300000,
                warningAfter: 5000
              })
                .success(function (data, status, headers, config) {
                  deferred.resolve(true);
                })
                .error(function (data, status, headers, config) {
                  $log.warn("error in sendJobValuesToServer data = '" + data + "' : status = '" + status + "'");
                  deferred.resolve(false);
                })
            }
            else {
              deferred.resolve(true);
            }
          });
        });
      } catch (e) {
        $log.error(e);
        deferred.resolve(false);
      }
      return deferred.promise;
    }

    this.getJobsFromServer = function () {
      var deferred = $q.defer();
      $http.get($rootScope.endPoint + '/appjob.json', {timeout: 300000, warningAfter: 5000})
        .success(function (data, status, headers, config) {
          var isUpdate = false;
          var hasData = false;
          var hasNeedToDelete = false;
          var urls;
          if (data.resStatus) {
            if (data.count > 0) {
              hasData = true;
              urls = data.urls;
            }
            if (data.deleteJobsCount > 0) {
              hasNeedToDelete = true;
            }

            DbService.getJobsIdList().then(function (results) {
              var updatedData = null;
              var prom = [];

              if (results.rows.length > 0) {
                if (hasData) {
                  //We have data on the server and on the client.
                  //Get the jobs that we have only on the server.
                  var diff = data.data.diff(results.rows);
                  if (diff.length > 0) {
                    updatedData = diff;
                  }
                }
              }
              else {
                if (hasData) {
                  //We have data only on the server
                  updatedData = data.data;
                }
              }
              if (updatedData != null) {
                isUpdate = true;
                updatedData.forEach(function (nid) {
                  prom.push(getJobDataFromServer(urls[nid]));
                });
              }

              if (hasNeedToDelete) {
                isUpdate = true;
                prom.push(DbService.deleteJobData(data.deleteJobs));
              }

              $q.all(prom).then(function () {
                deferred.resolve(isUpdate);
              });
            });
          }
          else {
            deferred.resolve(isUpdate);
          }
        })
        .error(function (data, status, headers, config) {
          $log.warn("error in getJobsFromServer data = '" + data + "' : status = '" + status + "'");
        });
      return deferred.promise;
    }

  })
