angular.module('ib.services')
.service('SyncService', function($q, DbService, RestService, $http, $rootScope, $log, JobsFactory) {
    this.checkSyncLogic = function(connectionType, settings) {
      var deferred = $q.defer();
      if (connectionType == 1) { // WiFi
        // May sync
        deferred.resolve(1);
      }
      if (connectionType == -1) { // no connection
        // Do nothing
        //Off line sync
        deferred.resolve(-1);

      }
      else if (connectionType == 0) { // Connected, not WiFi
        DbService.isJobsHasImages().then(function(hasImages) {
          if (hasImages == true) {
            if (settings.sync.imageUploadNotify.value) {
              // Prompt the user about the connection type.
              deferred.resolve(0);
            }
            else {
              // No need for popup - may sync
              deferred.resolve(1);
            }
          }
          else { // No images to sync
            // May sync
            deferred.resolve(1);
          }
        });
      }
      return deferred.promise;
    }
    this.sync = function() {

      var deferred = $q.defer();
      var syncProm = [];
      var doneProm = [];

      DbService.getNeedSyncJobsIdList().then(function(results) {
        if (results.rows.length > 0) {
          results.rows.forEach(function (id) {
            syncProm.push(_sync(id));
          });
        }
        $q.all(syncProm).then(function () {
          DbService.getDoneJobsIdList().then(function(results) {
            if (results.rows.length > 0) {
              results.rows.forEach(function (id) {
                doneProm.push(_markJobAsDone(id));
              });
            }
            $q.all(doneProm).then(function () {
              deferred.resolve();
            });
          });
        });
      });
      return deferred.promise;
    }

    var _markJobAsDone = function(nid) {
      var deferred = $q.defer();
      $http.put($rootScope.endPoint + '/appjob/' + nid + '.json' , {status:3})
        .success(function (data, status, headers, config) {

          DbService.updateJobStatus(nid, 4);
          if (JobsFactory.job.id == nid) {
            JobsFactory.job.status = 4;
            $rootScope.$broadcast('jobs.jobUpdate');
          }
          deferred.resolve(true);
        })
        .error(function (data, status, headers, config) {
          $log.warn("error in _markJobAsDone data = '" + data + "' : status = '" + status + "'");
          deferred.resolve(false);
        });
      return deferred.promise;

    }

    var _sync = function(id) {
      var deferred = $q.defer();
      DbService.sendJobFieldsToServer(id).then(function(status) {
        if (status == true) {
          DbService.processJobImages(id).then(function(data){
            if (data.status == true) {
              RestService.sendJobValuesToServer(id).then(function(status) {
                if (status == true) {
                  DbService.updateJobSyncStatus(id, 1).then(function(status) {
                    deferred.resolve(status);
                  })
                }
                else {
                  deferred.resolve(false);
                }
              });
            }
            else {
              deferred.resolve(false);
            }
          });
        }
        else {
          deferred.resolve(false);
        }
      });
      return deferred.promise;
    }
})

